<?xml version="1.0" standalone="no"?>
<xsl:stylesheet version="1.0"
           xmlns:svg="http://www.w3.org/2000/svg"
           xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
           xmlns:math="http://exslt.org/math"
           xmlns:exsl="http://exslt.org/common"
           xmlns:xlink="http://www.w3.org/1999/xlink"
           extension-element-prefixes="math">
           
<xsl:output method="xml" 
			version="1.0" 
			encoding="UTF-8" 
			indent="yes"
	        doctype-public="-//W3C//DTD SVG 1.0//EN"
		    doctype-system="http://www.w3.org/TR/SVG/DTD/svg10.dtd"/>
			
<!--
<xsl:variable name="BIF_GAP" select="4"/>				
<xsl:variable name="INF_H"   select="ceiling($BIF_H div 2)"/>				
<CPS><xsl:value-of select="@MODCLASS"/></CPS>
-->


<!-- ======================= DEF BLOCK =================================== -->
<xsl:template name="Define_AllStacks"> 
	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(@EAST &lt; /EDKSYSTEM/BLKDSHAPES/@STACK_HORIZ_WIDTH)]">
			
		<xsl:call-template name="Define_Stack">
			<xsl:with-param name="stackIdx"  select="@EAST"/>
		</xsl:call-template>
		
	</xsl:for-each>	
</xsl:template>
	
	
<xsl:template name="Define_Stack"> 
	<xsl:param name="stackIdx"  select="100"/>
	
	<!-- Define the stack's peripheral shapes-->	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and not(@MODCLASS = 'MEMORY_UNIT'))]"> 
			
		<xsl:for-each select="MODULE">
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			<xsl:variable name="modType_" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE=$modInst_)]/@MODTYPE"/>
			<xsl:call-template name="Define_Peripheral"> 
				<xsl:with-param name="modInst"    select="$modInst_"/>
				<xsl:with-param name="modType"    select="$modType_"/>
				<xsl:with-param name="shapeId"    select="../@SHAPE_ID"/>
				<xsl:with-param name="horizIdx"   select="../@STACK_HORIZ_INDEX"/>
				<xsl:with-param name="vertiIdx"   select="../@SHAPE_VERTI_INDEX"/>
			</xsl:call-template>		
		</xsl:for-each>	
		
	</xsl:for-each>
	
	<!-- Define the stack's memory shapes-->	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@MODCLASS='MEMORY_UNIT'))]">
		<xsl:call-template name="Define_MemoryUnit"> 
			<xsl:with-param name="shapeId"  select="@SHAPE_ID"/>
		</xsl:call-template>
	</xsl:for-each>
	
	
	<!-- Define the stack's processors-->	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[@INSTANCE and @BIFS_W and @BIFS_H and (@STACK_HORIZ_INDEX = $stackIdx)]">	
		<xsl:call-template name="Define_Processor"/>		
	</xsl:for-each>	
		
	<!-- Make an inventory of all the things in this processor's stack -->
	<xsl:variable name="pstackW_">
		<xsl:call-template name="_calc_Stack_Width"> 
			<xsl:with-param name="stackIdx"  select="$stackIdx"/>
		</xsl:call-template>		
	</xsl:variable>
		
	<xsl:variable name="pstackH_">
		<xsl:call-template name="_calc_Stack_Height"> 
			<xsl:with-param name="stackIdx"  select="$stackIdx"/>
		</xsl:call-template>		
	</xsl:variable>
	
	<xsl:variable name="procW_"    select="$periMOD_W"/>
	<xsl:variable name="procX_"    select="(ceiling($pstackW_ div 2) - ceiling($procW_ div 2))"/>
	
	<xsl:variable name="numSBSs_"     select="count(/EDKSYSTEM/BLKDSHAPES/SBSSHAPES/MODULE)"/>	
	<xsl:variable name="sbs_H_"    select="($numSBSs_ * $SBS_H)"/>
	<xsl:variable name="sbsGap_"   select="($PROC2SBS_GAP + $sbs_H_)"/>
	
	<xsl:variable name="stack_name_">
		<xsl:call-template name="_gen_Stack_Name"> 
			<xsl:with-param name="horizIdx" select="$stackIdx"/>
		</xsl:call-template>		
	</xsl:variable>	
	
<!--	
		<xsl:message>Horiz index<xsl:value-of select="$stackIdx"/></xsl:message>
		<xsl:message>Drawing stack <xsl:value-of select="$stack_name_"/></xsl:message>
-->	
		
		<!-- Now use all this stuff to draw the stack-->	
		<symbol id="{$stack_name_}">
			<rect x="0"
				  y="0"
			      rx="6" 
			      ry="6" 
		          width = "{$pstackW_}"
		          height= "{$pstackH_}"
			      style="fill:{$COL_BG}; stroke:none;"/>
			
		
			<!-- First draw the the processor's peripherals-->	
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(@STACK_HORIZ_INDEX = $stackIdx)]">
				<xsl:sort select="@STACK_VERTI_INDEX" data-type="number"/>
				
				<xsl:variable name="shapeW_"    select="(@MODS_W * $periMOD_W)"/>
				<xsl:variable name="shapeX_"    select="(ceiling($pstackW_ div 2) - ceiling($shapeW_ div 2))"/>
				<xsl:variable name="shapeY_">
					<xsl:call-template name="_calc_Stack_Shape_Y">
						<xsl:with-param name="horizIdx"  select="@STACK_HORIZ_INDEX"/>
						<xsl:with-param name="vertiIdx"  select="@SHAPE_VERTI_INDEX"/>
					</xsl:call-template>
				</xsl:variable>  
				
				<xsl:variable name="stack_SymName_">
					<xsl:call-template name="_gen_Stack_SymbolName"> 
						<xsl:with-param name="horizIdx" select="@STACK_HORIZ_INDEX"/>
						<xsl:with-param name="vertiIdx" select="@SHAPE_VERTI_INDEX"/>
					</xsl:call-template>		
				</xsl:variable>
				
<!--				
				<xsl:message>Drawing stack peripheral <xsl:value-of select="$stack_SymName_"/> at <xsl:value-of select="$shapeY_"/></xsl:message>
-->				
				
			 	<use   x="{$shapeX_}"  y="{$shapeY_}"  xlink:href="#{$stack_SymName_}"/> 
			
			</xsl:for-each>
			
			
			<!-- Then draw the slave buckets for the shared busses that this processor is master to -->	
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX = $stackIdx)]">	
				<xsl:sort select="@SHAPE_VERTI_INDEX" data-type="number"/>
			
				<xsl:variable name="bucketW_"   select="(($MOD_BKTLANE_W * 2) + (($periMOD_W * @MODS_W) + ($MOD_BUCKET_G * (@MODS_W - 1))))"/>
				<xsl:variable name="bucketX_"   select="(ceiling($pstackW_ div 2) - ceiling($bucketW_ div 2))"/>
				
				<xsl:variable name="bucketY_">
					<xsl:call-template name="_calc_Stack_Shape_Y">
						<xsl:with-param name="horizIdx"  select="@STACK_HORIZ_INDEX"/>
						<xsl:with-param name="vertiIdx"  select="@SHAPE_VERTI_INDEX"/>
					</xsl:call-template>
				</xsl:variable>  
				
<!--				
				<xsl:message>SBS Bucket Y <xsl:value-of select="$bucketY_"/></xsl:message>
-->				
				
				 <use  x="{$bucketX_}"  y="{$bucketY_}"  xlink:href="#sbsbucket_{@BUSNAME}"/> 
				 
				 <text class="ipclass"
					   x="{$bucketX_}" 
					   y="{$bucketY_ - 4}">SLAVES OF <xsl:value-of select="@BUSNAME"/></text>	
			</xsl:for-each>
			
			<!-- Then draw the the processor itself -->	
			<!-- First draw the the processor's itself-->	
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@STACK_HORIZ_INDEX = $stackIdx)]">
				<xsl:sort select="@SHAPE_VERTI_INDEX" data-type="number"/>
				
				<xsl:variable name="procY_">
					<xsl:call-template name="_calc_Stack_Shape_Y">
						<xsl:with-param name="horizIdx"  select="@STACK_HORIZ_INDEX"/>
						<xsl:with-param name="vertiIdx"  select="@SHAPE_VERTI_INDEX"/>
					</xsl:call-template>
				</xsl:variable>  
				
				<xsl:variable name="stack_SymName_">
					<xsl:call-template name="_gen_Stack_SymbolName"> 
						<xsl:with-param name="horizIdx" select="@STACK_HORIZ_INDEX"/>
						<xsl:with-param name="vertiIdx" select="@SHAPE_VERTI_INDEX"/>
					</xsl:call-template>		
				</xsl:variable>
				
			 	<use   x="{$procX_}"  y="{$procY_}"  xlink:href="#{$stack_SymName_}"/> 
				
				<xsl:if test = "not(@IS_LIKEPROC)">
					<text class="ipclass"
						x="{$procX_}" 
						y="{$procY_ - 4}">PROCESSOR</text>		
				</xsl:if>			
				  
				<xsl:if test = "@IS_LIKEPROC = 'TRUE'">
					<text class="ipclass"
						x="{$procX_}" 
						y="{$procY_ - 4}">USER MODULE</text>		
				</xsl:if>			
			
			</xsl:for-each>
		</symbol>
</xsl:template>	

	
	
<xsl:template name="Define_BusLaneSpaces"> 
	
	<xsl:variable name="lastStack_" select="(/EDKSYSTEM/BLKDSHAPES/@STACK_HORIZ_WIDTH) - 1"/>
	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[@EAST]">
		<xsl:sort select="@EAST" data-type="number"/>
			
		<xsl:call-template name="Define_BusLaneSpace">
			<xsl:with-param name="stackToEast"  select="@EAST"/>
		</xsl:call-template>
	</xsl:for-each>	
	
<!--	
	<xsl:message>Last Stack <xsl:value-of select="$lastStack_"/></xsl:message>
-->	
	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(@WEST = $lastStack_)]">
		<xsl:call-template name="Define_BusLaneSpace">
			<xsl:with-param name="stackToWest"  select="$lastStack_"/>
		</xsl:call-template>
	</xsl:for-each>	
			
</xsl:template>
	
	
<xsl:template name="Define_BusLaneSpace"> 
	
	<xsl:param name="stackToEast"  select="'NONE'"/>
	<xsl:param name="stackToWest"  select="'NONE'"/>
	
	<xsl:variable name="stkToEast_">
		<xsl:choose>
			<xsl:when test="not($stackToEast = 'NONE')"><xsl:value-of select="$stackToEast"/></xsl:when>
<!--			
			<xsl:when test="not($stackToWest = 'NONE')"><xsl:value-of select="($stackToWest + 1)"/></xsl:when>
-->	
			<xsl:otherwise>NONE</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>	
	
	<xsl:variable name="stkToWest_">
		<xsl:choose>
			<xsl:when test="not($stackToWest  = 'NONE')"><xsl:value-of select="$stackToWest"/></xsl:when>
			<xsl:when test="(not($stackToEast = 'NONE') and not($stackToEast = '0'))"><xsl:value-of select="($stackToEast - 1)"/></xsl:when>
			<xsl:otherwise>NONE</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>	
	
<!--	
	<xsl:message>Stack to West <xsl:value-of select="$stkToWest_"/></xsl:message>
	<xsl:message>Stack to East <xsl:value-of select="$stkToEast_"/></xsl:message>
-->	
	
<!--	
	<xsl:message>Stack abv diff  <xsl:value-of select="$stack_H_diff_"/></xsl:message>
-->	
	
	<xsl:variable name="spaceAbvSbs_H_">
		<xsl:call-template name="_calc_Space_AbvSbs_Height">
			<xsl:with-param name="stackToEast"  select="$stackToEast"/>
			<xsl:with-param name="stackToWest"  select="$stackToWest"/>
		</xsl:call-template>
	</xsl:variable>	
	
	<xsl:variable name="spaceBlwSbs_H_">
		<xsl:call-template name="_calc_Space_BlwSbs_Height">
			<xsl:with-param name="stackToEast"  select="$stackToEast"/>
			<xsl:with-param name="stackToWest"  select="$stackToWest"/>
		</xsl:call-template>
	</xsl:variable>	
	
	<xsl:variable name="numSbs_"      select="count(/EDKSYSTEM/BLKDSHAPES/SBSSHAPES/MODULE)"/>	
	<xsl:variable name="totalSbs_H_"  select="($numSbs_ * $SBS_H)"/>
	
	<xsl:variable name="space_H_" select="($spaceAbvSbs_H_ + $PROC2SBS_GAP + $totalSbs_H_ + $spaceBlwSbs_H_)"/>
	<xsl:variable name="space_W_">
		<xsl:call-template name="_calc_Space_Width"> 
			<xsl:with-param name="stackToEast"  select="$stackToEast"/>
			<xsl:with-param name="stackToWest"  select="$stackToWest"/>
		</xsl:call-template>		
	</xsl:variable>
	
<!--	
	<xsl:variable name="sbs_Y_" select="$spaceAbvSbs_H_ + $BIF_H"/>
-->	
	
	<xsl:variable name="sbs_Y_" select="$spaceAbvSbs_H_ + $PROC2SBS_GAP"/>
	
	<xsl:variable name="space_name_">
		<xsl:call-template name="_gen_Space_Name"> 
			<xsl:with-param name="stackToEast"  select="$stackToEast"/>
			<xsl:with-param name="stackToWest"  select="$stackToWest"/>
		</xsl:call-template>		
	</xsl:variable>
	
	
<!--	
	<xsl:message>Stack Width <xsl:value-of select="$space_W_"/></xsl:message>
	<xsl:message>Space Name <xsl:value-of select="$space_name_"/></xsl:message>
	<xsl:message>Stack Abv <xsl:value-of select="$spaceAbvSbs_H_"/></xsl:message>
	<xsl:message>Stack Blw <xsl:value-of select="$spaceBlwSbs_H_"/></xsl:message>
	<xsl:message>Total Sbs <xsl:value-of select="$totalSbs_H_"/></xsl:message>
-->	
	
	<xsl:variable name = "stackToWest_W_">
		<xsl:choose>
			<xsl:when test="(($stackToEast = '0')   and     ($stackToWest = 'NONE'))">0</xsl:when>
			<xsl:when test="(($stackToEast = 'NONE') and not($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_Width">
					<xsl:with-param name="stackIdx"  select="$stackToWest"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="(not($stackToEast = '0') and not($stackToEast = 'NONE') and ($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_Width">
					<xsl:with-param name="stackIdx"  select="($stackToEast - 1)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name = "stackToEast_W_">
		<xsl:call-template name="_calc_Stack_Width">
			<xsl:with-param name="stackIdx"  select="$stackToEast"/>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name ="extSpaceWest_W_" select="ceiling($stackToWest_W_ div 2)"/>
	<xsl:variable name ="extSpaceEast_W_" select="ceiling($stackToEast_W_ div 2)"/>
	
<!--	
	<xsl:message>Stack To West <xsl:value-of select="$stackToWest_W_"/></xsl:message>
	<xsl:message>Stack To East <xsl:value-of select="$stackToEast_W_"/></xsl:message>
-->	
	
	<symbol id="{$space_name_}">
<!--		
		<rect x="0"
			  y="0"
	          width = "{$space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_}"
		      height= "{$space_H_}"
			  style="fill:{$COL_WHITE}; stroke:none;"/>
-->		
		
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[((@EAST = $stackToEast) or (($stackToEast = 'NONE') and (@WEST = $stackToWest)))]/BUSCONNLANE[@BUSSTD and @BUSNAME]">
				
<!--				
				<xsl:message>Bus Std of conn <xsl:value-of select="@BUSSTD"/></xsl:message>
-->				
				<xsl:variable name="busStd_"          select="@BUSSTD"/>
				<xsl:variable name="busName_"         select="@BUSNAME"/>
				<xsl:variable name="lane_X_"          select="@BUSLANE_X"/>
				<xsl:variable name="laneInSpace_X_"   select="($extSpaceWest_W_ + ($lane_X_ * $BUS_LANE_W))"/>
				
				<xsl:variable name="bus_color_">
					<xsl:call-template name="BusType2Color">
						<xsl:with-param name="busType" select="@BUSSTD"/>
					</xsl:call-template>	
				</xsl:variable>
				
				
				<xsl:choose>
					
<!-- 
		 ===========================================================
			Handle Bucket connections to the shared busses.
		 ===========================================================
-->
					
				<xsl:when test="@BUSLANE_X and @IS_BKTCONN and BUSCONN[@BIFRANK] and /EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $busName_)]/@BUSINDEX">
						
					<xsl:variable name="sbs_idx_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE= $busName_)]/@BUSINDEX"/>
					<xsl:variable name="sbs_name_" select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@BUSINDEX = $sbs_idx_)]/@BUSNAME"/>
					
					<xsl:variable name="sbs_bc_y_" select="($sbs_Y_ + ($sbs_idx_ * $SBS_H))"/>
					
					<xsl:variable name="bktshp_hori_idx_" select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@BUSINDEX = $sbs_idx_)]/@STACK_HORIZ_INDEX"/>
					<xsl:variable name="bktshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@BUSINDEX = $sbs_idx_)]/@SHAPE_VERTI_INDEX"/>
			   		
<!--					
					<xsl:message>SBS index  <xsl:value-of select="$sbs_idx_"/></xsl:message>
					<xsl:message>SBS name  <xsl:value-of select="$sbs_name_"/></xsl:message>
					<xsl:message>Bucket horiz  <xsl:value-of select="$bktshp_hori_idx_"/></xsl:message>
					<xsl:message>Bucket verti  <xsl:value-of select="$bktshp_vert_idx_"/></xsl:message>
-->						
						<xsl:variable name="bktshp_Y_">
							<xsl:call-template name="_calc_Stack_Shape_Y">
								<xsl:with-param name="horizIdx"  select="$bktshp_hori_idx_"/>
								<xsl:with-param name="vertiIdx"  select="$bktshp_vert_idx_"/>
							</xsl:call-template>
						</xsl:variable>
						
						<xsl:variable name="sbsStack_H_diff_">
							<xsl:choose>
								<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
								<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
				
									<xsl:variable name="stackToWest_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
									</xsl:variable>
				
									<xsl:variable name="stackToEast_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
										</xsl:call-template>
									</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
									<xsl:if test="($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_)">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:if>	
<!--									
									<xsl:if test="($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_)">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:if>	
-->	
									<xsl:if test="not($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_)">0</xsl:if>	
								</xsl:when>
							</xsl:choose>
						</xsl:variable>
	
<!--						
        <xsl:message>Bkt Y of <xsl:value-of select="$bktshp_hori_idx_"/> is <xsl:value-of select="$bktshp_Y_"/></xsl:message>
		<xsl:message>Stack horiz  <xsl:value-of select="$sbsshp_hori_idx_"/></xsl:message>
		<xsl:message>Stack verti  <xsl:value-of select="$sbsshp_vert_idx_"/></xsl:message>
		<xsl:message>Bus Idx of <xsl:value-of select="$busName_"/> is <xsl:value-of select="$sbs_idx_"/></xsl:message>
		<xsl:message>Stack To West <xsl:value-of select="$stackToWest"/></xsl:message>
		<xsl:message>Stack To East <xsl:value-of select="$stackToEast"/></xsl:message>
		<xsl:message>Stack To West W <xsl:value-of select="$stackToWest_W_"/></xsl:message>
		<xsl:message>Stack To East W <xsl:value-of select="$stackToEast_W_"/></xsl:message>
		<xsl:message>vert Y1 <xsl:value-of select="$vert_line_y1_"/></xsl:message>
		<xsl:message>vert Y2 <xsl:value-of select="$vert_line_y2_"/></xsl:message>
		<rect x="0"
			  y="0"
	          width = "{$space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_}"
		      height= "{$space_H_}"
			  style="fill:{$COL_RED}; stroke:none;"/>
-->		
						<use   x="{$laneInSpace_X_}"   y="{$sbs_bc_y_ - ceiling($BIFC_H div 4)}"  xlink:href="#{$busStd_}_busconn_{BUSCONN/@BIFRANK}"/>
						
						<xsl:variable name="vert_line_x_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2))"/>
						
						<xsl:variable name="vert_line_y1_" select="($sbs_Y_    + ($sbs_idx_ * $SBS_H) + ceiling($BIFC_W div 2))"/>
						<xsl:variable name="vert_line_y2_" select="($bktshp_Y_ + ceiling($periMOD_W div 2) + $sbsStack_H_diff_)"/>
						
						
					<xsl:variable name="bcInSpace_X_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2) - ceiling($BUS_ARROW_W div 2))"/>
						<line x1="{$vert_line_x_}"
				 		      y1="{$vert_line_y1_}" 
				  		      x2="{$vert_line_x_}"
				  			  y2="{$vert_line_y2_}" 
				  			  style="stroke:{$bus_color_};stroke-width:1"/>
						
						<xsl:variable name="horz_line_y_"   select="$vert_line_y2_"/>
						<xsl:variable name="horz_line_x1_"  select="$vert_line_x_"/>
						<xsl:variable name="horz_line_x2_" select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_)"/>
						
						<line x1="{$horz_line_x1_}"
				 		      y1="{$horz_line_y_}" 
				  		      x2="{$horz_line_x2_}"
				  			  y2="{$horz_line_y_}" 
				  			  style="stroke:{$bus_color_};stroke-width:1"/>
						
					</xsl:when>
					
<!--
		 ===========================================================
			Handle Processor's Shared bus connections.
		 ===========================================================
-->
					<xsl:when test="@BUSLANE_X and @IS_SBSCONN and BUSCONN[@BIF_Y and @IS_PROCONN and @INSTANCE and @BUSINTERFACE] and /EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $busName_)]/@BUSINDEX">
						
						<xsl:variable name="sbs_idx_" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE= $busName_)]/@BUSINDEX"/>
						
						<xsl:variable name="sbs_bc_y_" select="($sbs_Y_ + ($sbs_idx_ * $SBS_H))"/>
						
						<xsl:variable name="procInst_" select="BUSCONN/@INSTANCE"/>
						
						<xsl:variable name="procBif_Y_"    select="((($BIF_H + $BIF_GAP) * BUSCONN/@BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
						<xsl:variable name="procBifName_"  select="BUSCONN/@BUSINTERFACE"/>
						<xsl:variable name="procBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInst_)]/BUSINTERFACE[(@NAME = $procBifName_)]/@BIF_X"/>
						<xsl:variable name="procBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInst_)]/BUSINTERFACE[(@NAME = $procBifName_)]/@BIFRANK"/>
						
						<xsl:variable name="procshp_hori_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInst_)]/@STACK_HORIZ_INDEX"/>
						<xsl:variable name="procshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInst_)]/@SHAPE_VERTI_INDEX"/>
						
<!--						
						<xsl:message>Stack horiz  <xsl:value-of select="$procshp_hori_idx_"/></xsl:message>
						<xsl:message>Stack verti  <xsl:value-of select="$procshp_vert_idx_"/></xsl:message>
						<xsl:message>Proc Bif Y   <xsl:value-of select="$procBif_Y_"/></xsl:message>
-->						
						
						<xsl:variable name="procshp_Y_">
							<xsl:call-template name="_calc_Stack_Shape_Y">
								<xsl:with-param name="horizIdx"  select="$procshp_hori_idx_"/>
								<xsl:with-param name="vertiIdx"  select="$procshp_vert_idx_"/>
							</xsl:call-template>
						</xsl:variable>
						
	
						<xsl:variable name="procStack_H_diff_">
							<xsl:choose>
								<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
								<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
				
									<xsl:variable name="stackToWest_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
									</xsl:variable>
				
									<xsl:variable name="stackToEast_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
										</xsl:call-template>
									</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
									<xsl:choose>
										<xsl:when test="(($procshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
											<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
										</xsl:when>	
										<xsl:when test="(($procshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
											<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
										</xsl:when>	
										<xsl:otherwise>0</xsl:otherwise>	
									</xsl:choose>	
								</xsl:when>
							</xsl:choose>
						</xsl:variable>
						
<!--						
						<xsl:message>Proc diff  <xsl:value-of select="$procStack_H_diff_"/></xsl:message>
						<xsl:message>Proc inst  <xsl:value-of select="$procInst_"/></xsl:message>
						<xsl:message>Proc Bif Name <xsl:value-of select="$procBifName_"/></xsl:message>
						<xsl:message>Proc Bif Rank <xsl:value-of select="$procBifRank_"/></xsl:message>
-->
						<use   x="{$laneInSpace_X_}"   y="{$sbs_bc_y_ - ceiling($BIFC_H div 4)}"  xlink:href="#{$busStd_}_busconn_{$procBifRank_}"/>
						
						<xsl:variable name="vert_line_x_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2))"/>
						
						<xsl:variable name="vert_line_y1_" select="($procshp_Y_ + $procBif_Y_ + ceiling($BIF_H div 2) + $procStack_H_diff_)"/>
						<xsl:variable name="vert_line_y2_"  select="($sbs_Y_    + ($sbs_idx_ * $SBS_H) + ceiling($BIFC_W div 2))"/>
						
						<line x1="{$vert_line_x_}"
				 		      y1="{$vert_line_y1_}" 
				  		      x2="{$vert_line_x_}"
				  			  y2="{$vert_line_y2_}" 
				  			  style="stroke:{$bus_color_};stroke-width:1"/>
						
						<xsl:variable name="horz_line_y_"   select="$vert_line_y1_"/>
						<xsl:variable name="horz_line_x1_"  select="$vert_line_x_"/>
						<xsl:variable name="horz_line_x2_">
							<xsl:choose>
								<xsl:when test="$procBifSide_ = '0'">
							 		<xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_)"/>
								</xsl:when>
								<xsl:otherwise>0</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						
						<line x1="{$horz_line_x1_}"
				 		      y1="{$horz_line_y_}" 
				  		      x2="{$horz_line_x2_}"
				  			  y2="{$horz_line_y_}" 
				  			  style="stroke:{$bus_color_};stroke-width:1"/>
						
						
	  				</xsl:when>
					
<!--
		 ===========================================================
			Handle non Processor Shared bus connections.
		 ===========================================================
-->
				
					
					<xsl:when test="@BUSLANE_X and @IS_SBSCONN and BUSCONN[@BIF_Y and not(@IS_PROCONN) and @INSTANCE and @BUSINTERFACE] and /EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $busName_)]/@BUSINDEX">
						
						<xsl:variable name="sbs_idx_" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE= $busName_)]/@BUSINDEX"/>
						
						<xsl:variable name="sbs_bc_y_" select="($sbs_Y_ + ($sbs_idx_ * $SBS_H))"/>
						
						<xsl:variable name="cmplxInst_" select="BUSCONN/@INSTANCE"/>
						
						<xsl:variable name="cmplxBif_Y_"    select="((($BIF_H + $BIF_GAP) * BUSCONN/@BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
						<xsl:variable name="cmplxBifName_"  select="BUSCONN/@BUSINTERFACE"/>
						<xsl:variable name="cmplxBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $cmplxInst_)]/BUSINTERFACE[(@NAME = $cmplxBifName_)]/@BIF_X"/>
						<xsl:variable name="cmplxBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $cmplxInst_)]/BUSINTERFACE[(@NAME = $cmplxBifName_)]/@BIFRANK"/>
						
						<xsl:variable name="cmplxshp_hori_idx_" select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $cmplxInst_)])]/@STACK_HORIZ_INDEX"/>
						<xsl:variable name="cmplxshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $cmplxInst_)])]/@SHAPE_VERTI_INDEX"/>
						
<!--						
						<xsl:message>Stack horiz  <xsl:value-of select="$cmplxshp_hori_idx_"/></xsl:message>
						<xsl:message>Stack verti  <xsl:value-of select="$cmplxshp_vert_idx_"/></xsl:message>
						<xsl:message>Proc Bif Y   <xsl:value-of select="$procBif_Y_"/></xsl:message>
-->						
						
						<xsl:variable name="cmplxshp_Y_">
							<xsl:call-template name="_calc_Stack_Shape_Y">
								<xsl:with-param name="horizIdx"  select="$cmplxshp_hori_idx_"/>
								<xsl:with-param name="vertiIdx"  select="$cmplxshp_vert_idx_"/>
							</xsl:call-template>
						</xsl:variable>
						
						<xsl:variable name="cmplxStack_H_diff_">
							<xsl:choose>
								<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
								<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
				
									<xsl:variable name="stackToWest_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
									</xsl:variable>
				
									<xsl:variable name="stackToEast_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
										</xsl:call-template>
									</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($cmplxshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($cmplxshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
									
							</xsl:when>
						</xsl:choose>
					</xsl:variable>
						
<!--						
						<xsl:message>Proc diff  <xsl:value-of select="$procStack_H_diff_"/></xsl:message>
						<xsl:message>Proc inst  <xsl:value-of select="$procInst_"/></xsl:message>
						<xsl:message>Proc Bif Name <xsl:value-of select="$procBifName_"/></xsl:message>
						<xsl:message>Proc Bif Rank <xsl:value-of select="$procBifRank_"/></xsl:message>
-->
						<use   x="{$laneInSpace_X_}"   y="{$sbs_bc_y_ - ceiling($BIFC_H div 4)}"  xlink:href="#{$busStd_}_busconn_{$cmplxBifRank_}"/>
						
						<xsl:variable name="vert_line_x_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2))"/>
						
						<xsl:variable name="vert_line_y1_" select="($cmplxshp_Y_ + $cmplxBif_Y_ + ceiling($BIF_H div 2) + $cmplxStack_H_diff_)"/>
						<xsl:variable name="vert_line_y2_"  select="($sbs_Y_    + ($sbs_idx_ * $SBS_H) + ceiling($BIFC_W div 2))"/>
						
						<line x1="{$vert_line_x_}"
				 		      y1="{$vert_line_y1_}" 
				  		      x2="{$vert_line_x_}"
				  			  y2="{$vert_line_y2_}" 
				  			  style="stroke:{$bus_color_};stroke-width:1"/>
						
						<xsl:variable name="horz_line_y_"   select="$vert_line_y1_"/>
						<xsl:variable name="horz_line_x1_"  select="$vert_line_x_"/>
						<xsl:variable name="horz_line_x2_">
							<xsl:choose>
								<xsl:when test="$cmplxBifSide_ = '0'">
							 		<xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_)"/>
								</xsl:when>
								<xsl:otherwise>0</xsl:otherwise>
							</xsl:choose>
						</xsl:variable>
						
						<line x1="{$horz_line_x1_}"
				 		      y1="{$horz_line_y_}" 
				  		      x2="{$horz_line_x2_}"
				  			  y2="{$horz_line_y_}" 
				  			  style="stroke:{$bus_color_};stroke-width:1"/>
						
	  				</xsl:when>
					
<!-- 
		 ===========================================================
			Handle connections from processors to Memory UNITs
		 ===========================================================
-->
				
					<xsl:when test="@BUSLANE_X and @IS_MEMCONN and not(@IS_MULTISTK) and BUSCONN[@BIF_Y and @IS_PROCONN and @INSTANCE and @BUSINTERFACE]">
						
						<xsl:variable name="bcInSpace_X_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2) - ceiling($BUS_ARROW_W div 2))"/>
						
						<xsl:variable name="procInstance_" select="BUSCONN[@IS_PROCONN]/@INSTANCE"/>
<!--						
						<xsl:message>Found Processor <xsl:value-of select="$procInstance_"/></xsl:message>	
						<xsl:variable name="memInstance_"  select="BUSCONN[not(@IS_PROCONN)]/@INSTANCE"/>
-->	
						
						<xsl:variable name="mem_procshp_hori_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInstance_)]/@STACK_HORIZ_INDEX"/>
						<xsl:variable name="mem_procshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInstance_)]/@SHAPE_VERTI_INDEX"/>
						
						<xsl:variable name="mem_procshp_Y_">
							<xsl:call-template name="_calc_Stack_Shape_Y">
								<xsl:with-param name="horizIdx"  select="$mem_procshp_hori_idx_"/>
								<xsl:with-param name="vertiIdx"  select="$mem_procshp_vert_idx_"/>
							</xsl:call-template>
						</xsl:variable>
						
						<xsl:variable name="cmplxStack_H_diff_">
							<xsl:choose>
								<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
								<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
				
									<xsl:variable name="stackToWest_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
									</xsl:variable>
				
									<xsl:variable name="stackToEast_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
										</xsl:call-template>
									</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($mem_procshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($mem_procshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
									
							</xsl:when>
						</xsl:choose>
					</xsl:variable>
						
																				
						<xsl:variable name="mem_procStack_H_diff_">
							<xsl:choose>
								<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
								<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
			
									<xsl:variable name="stackToWest_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
									</xsl:variable>
				
									<xsl:variable name="stackToEast_AbvSbs_H_">
										<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
										</xsl:call-template>
									</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
									<xsl:choose>
										<xsl:when test="(($mem_procshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
											<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
										</xsl:when>	
										<xsl:when test="(($mem_procshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
											<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
										</xsl:when>	
										<xsl:otherwise>0</xsl:otherwise>	
									</xsl:choose>	
									
								</xsl:when>
							</xsl:choose>
						</xsl:variable>
						
						<!-- Store the conns in a variable -->	
						<xsl:variable name="memConn_heights_">
	
							<xsl:for-each select="BUSCONN">
								
								<xsl:variable name="bifName_"       select="@BUSINTERFACE"/>
							
								<xsl:choose>
									<xsl:when test="@IS_PROCONN and @BIF_Y">
										
<!--										
										<xsl:message>Proc <xsl:value-of select="$procInstance_"/></xsl:message>
-->										
										
										<xsl:variable name="procBif_Y_"    select="((($BIF_H + $BIF_GAP) * @BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
										<xsl:variable name="procBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
										<xsl:variable name="procBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
										
										<xsl:variable name="bcProc_Y_"     select="($mem_procshp_Y_ + $procBif_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $mem_procStack_H_diff_)"/>
										
										<MEMCONN X="{$bcInSpace_X_}" Y="{$bcProc_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$procBifRank_}" BIFSIDE="{$procBifSide_}"/>
										
									</xsl:when>
									
									<xsl:otherwise>
										
										<xsl:variable name="memcInstance_"  select="@INSTANCE"/>
										<xsl:variable name="memcshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[MODULE[(@INSTANCE = $memcInstance_)]]/@SHAPE_VERTI_INDEX"/>
										<xsl:variable name="memcBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $memcInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
										<xsl:variable name="memcBif_Y_"    select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $memcInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_Y"/>
										
										<xsl:variable name="memshp_Y_">
											<xsl:call-template name="_calc_Stack_Shape_Y">
												<xsl:with-param name="horizIdx"  select="$mem_procshp_hori_idx_"/>
												<xsl:with-param name="vertiIdx"  select="$memcshp_vert_idx_"/>
											</xsl:call-template>
										</xsl:variable>
										
										<xsl:variable name="procBif_Y_"   select="((($BIF_H + $BIF_GAP) * @BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
										<xsl:variable name="memcConn_Y_">
											<xsl:choose>
												<xsl:when test="(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[MODULE[(@INSTANCE = $memcInstance_)]]/@MODS_H = 1)">
<!--													
													<xsl:message>Found a only a memory controller on <xsl:value-of select="$memcInstance_"/></xsl:message>
-->													
													<xsl:value-of  select="($memshp_Y_ + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP) +  ($memcBif_Y_ * ($BIF_H + $BIF_GAP)) + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $cmplxStack_H_diff_)"/>
												</xsl:when>
												<xsl:otherwise>
													<xsl:value-of  select="($memshp_Y_ + $periMOD_H + $BIF_GAP + ($memcBif_Y_ * ($BIF_H + $BIF_GAP)) + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $cmplxStack_H_diff_)"/>
												</xsl:otherwise>
											</xsl:choose>
										</xsl:variable>
						
										
<!--										
										<xsl:message>Mem C <xsl:value-of select="$memcInstance_"/></xsl:message>
										<xsl:message>Mem Y <xsl:value-of select="$memshp_Y_"/></xsl:message>
										<xsl:message>Mem IDX <xsl:value-of select="$memcshp_vert_idx_"/></xsl:message>
										<xsl:message>Mem SIDE <xsl:value-of select="$memcBifSide_"/></xsl:message>
										<xsl:variable name="memcConn_Y_"   select="($memshp_Y_ + $periMOD_H + $BIF_GAP + ($memcBif_Y_ * ($BIF_H + $BIF_GAP)) + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $cmplxStack_H_diff_)"/>
-->										
										<xsl:variable name="memcBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $memcInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
										
										
										<MEMCONN X="{$bcInSpace_X_}" Y="{$memcConn_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$memcBifRank_}" BIFSIDE="{$memcBifSide_}"/>
										
						
									</xsl:otherwise>
									
								</xsl:choose>
								
							</xsl:for-each>
							
						</xsl:variable>
						
						
						<!-- Draw the busconnection and horizontal lines.-->						
						<xsl:for-each select="exsl:node-set($memConn_heights_)/MEMCONN">
							
							
							<xsl:variable name="horz_line_y_" select="(@Y + ceiling($BIFC_H div 2))"/>
						
							<xsl:variable name="horz_line_x1_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2))"/>
							<xsl:variable name="horz_line_x2_">
								<xsl:choose>
									<xsl:when test="@BIFSIDE = '0'">
							 			<xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_)"/>
									</xsl:when>
									<xsl:otherwise>0</xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
						
							<line x1="{$horz_line_x1_}"
				 		      	  y1="{$horz_line_y_}" 
				  		      	  x2="{$horz_line_x2_}"
				  			      y2="{$horz_line_y_}" 
				  			      style="stroke:{$bus_color_};stroke-width:1"/>
							
							<use   x="{@X}"   y="{@Y}"  xlink:href="#{@BUSSTD}_busconn_{@BIFRANK}"/>
							
						</xsl:for-each>
						
						<xsl:variable name="busTop_" select="math:min(exsl:node-set($memConn_heights_)/MEMCONN/@Y)"/>
						<xsl:variable name="busBot_" select="math:max(exsl:node-set($memConn_heights_)/MEMCONN/@Y)"/>
						
						<xsl:call-template name="Draw_P2PBus">
							<xsl:with-param name="busX"    select="$bcInSpace_X_"/>
							<xsl:with-param name="busTop"  select="$busTop_"/>
							<xsl:with-param name="busBot"  select="$busBot_"/>
							<xsl:with-param name="busStd"  select="$busStd_"/>
							<xsl:with-param name="busName" select="$busName_"/>
						</xsl:call-template>
						
  				</xsl:when>
										
<!-- 
		 ===========================================================
			Handle generic Point to Point connections
		 ===========================================================
-->
					
				<xsl:when test="@BUSLANE_X and not(@IS_MULTISTK) and not(@IS_MEMCONN) and not(@IS_SBSCONN) and BUSCONN[@BIF_Y and @INSTANCE and @BUSINTERFACE and not(@IS_SPLITCONN)]">
						
					<xsl:variable name="bcInSpace_X_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2) - ceiling($BUS_ARROW_W div 2))"/>
						
					<xsl:variable name="p2pInstance_" select="BUSCONN[(@BIF_Y)]/@INSTANCE"/>
					
					<xsl:variable name="p2pshp_hori_idx_">
						<xsl:choose>
							<xsl:when test="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $p2pInstance_)]">
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $p2pInstance_)]/@STACK_HORIZ_INDEX"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $p2pInstance_)])]/@STACK_HORIZ_INDEX"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>		
					
					<xsl:variable name="p2pshp_vert_idx_">
						<xsl:choose>
							<xsl:when test="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $p2pInstance_)]">
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $p2pInstance_)]/@SHAPE_VERTI_INDEX"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $p2pInstance_)])]/@SHAPE_VERTI_INDEX"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>		
					
<!--					
					<xsl:variable name="p2pshp_hori_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInstance_)]/@STACK_HORIZ_INDEX"/>
					<xsl:variable name="p2pshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInstance_)]/@SHAPE_VERTI_INDEX"/>
-->					
						
					<xsl:variable name="p2pshp_Y_">
						<xsl:call-template name="_calc_Stack_Shape_Y">
							<xsl:with-param name="horizIdx"  select="$p2pshp_hori_idx_"/>
							<xsl:with-param name="vertiIdx"  select="$p2pshp_vert_idx_"/>
						</xsl:call-template>
					</xsl:variable>
						
					<xsl:variable name="cmplxStack_H_diff_">
						<xsl:choose>
							<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
							<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
				
								<xsl:variable name="stackToWest_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
									</xsl:call-template>
								</xsl:variable>
				
								<xsl:variable name="stackToEast_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
									</xsl:call-template>
								</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($p2pshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($p2pshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
									
							</xsl:when>
						</xsl:choose>
					</xsl:variable>
						
																				
					<xsl:variable name="procStack_H_diff_">
						<xsl:choose>
							<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
							<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
			
								<xsl:variable name="stackToWest_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
								</xsl:variable>
				
								<xsl:variable name="stackToEast_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
									</xsl:call-template>
								</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($p2pshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($p2pshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
									
								</xsl:when>
							</xsl:choose>
						</xsl:variable>
						
						<!-- Store the conns in a variable -->	
						<xsl:variable name="p2pConn_heights_">
	
							<xsl:for-each select="BUSCONN">
								
								<xsl:variable name="bifName_" select="@BUSINTERFACE"/>
							
								<xsl:choose>
									<xsl:when test="@IS_PROCONN and @BIF_Y">
										
<!--										
										<xsl:message>Proc <xsl:value-of select="$procInstance_"/></xsl:message>
-->										
										<xsl:variable name="procBif_Y_"   select="((($BIF_H + $BIF_GAP) * @BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
										
										<xsl:variable name="procBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $p2pInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
										<xsl:variable name="procBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $p2pInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
										
										<xsl:variable name="bcProc_Y_"     select="($p2pshp_Y_ + $procBif_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $procStack_H_diff_)"/>
										
										
										<MEMCONN X="{$bcInSpace_X_}" Y="{$bcProc_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$procBifRank_}" BIFSIDE="{$procBifSide_}" STACK_ID=""/>
										
									</xsl:when>
									
									<xsl:otherwise>
										
										<xsl:variable name="modInstance_"     select="@INSTANCE"/>
										<xsl:variable name="modshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[MODULE[(@INSTANCE = $modInstance_)]]/@SHAPE_VERTI_INDEX"/>
										<xsl:variable name="modBifSide_"      select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $modInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
										<xsl:variable name="modBif_Y_"        select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $modInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_Y"/>
										<xsl:variable name="modBc_Y_"         select="((($BIF_H + $BIF_GAP) * $modBif_Y_) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
										
<!--										
										<xsl:message>Memory Instance <xsl:value-of select="$procInstance_"/></xsl:message>
-->										
						
										<xsl:variable name="modshp_Y_">
											<xsl:call-template name="_calc_Stack_Shape_Y">
												<xsl:with-param name="horizIdx"  select="$p2pshp_hori_idx_"/>
												<xsl:with-param name="vertiIdx"  select="$modshp_vert_idx_"/>
											</xsl:call-template>
										</xsl:variable>
										
										<xsl:variable name="modBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $modInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
										<xsl:variable name="bcMod_Y_"     select="($modshp_Y_ + $modBc_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $cmplxStack_H_diff_)"/>
										
<!--										
										<xsl:message>Bc Bif Y <xsl:value-of select="$modBif_Y_"/></xsl:message>	
										<xsl:message>Bc Mod Y <xsl:value-of select="$modBc_Y_"/></xsl:message>	
-->										
										<MEMCONN X="{$bcInSpace_X_}" Y="{$bcMod_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$modBifRank_}" BIFSIDE="{$modBifSide_}"/>
						
									</xsl:otherwise>
									
									
								</xsl:choose>
								
							</xsl:for-each>
							
						</xsl:variable>
						
						
						<!-- Draw the busconnection and horizontal lines.-->						
						<xsl:for-each select="exsl:node-set($p2pConn_heights_)/MEMCONN">
							
							
							<xsl:variable name="horz_line_y_" select="(@Y + ceiling($BIFC_H div 2))"/>
						
							<xsl:variable name="horz_line_x1_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2))"/>
							<xsl:variable name="horz_line_x2_">
								<xsl:choose>
									<xsl:when test="@BIFSIDE = '0'">
							 			<xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_)"/>
									</xsl:when>
									<xsl:otherwise>0</xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
						
							<line x1="{$horz_line_x1_}"
				 		      	  y1="{$horz_line_y_}" 
				  		      	  x2="{$horz_line_x2_}"
				  			      y2="{$horz_line_y_}" 
				  			      style="stroke:{$bus_color_};stroke-width:1"/>
							
							<use   x="{@X}"   y="{@Y}"  xlink:href="#{@BUSSTD}_busconn_{@BIFRANK}"/>
							
						</xsl:for-each>
						
						<xsl:variable name="busTop_" select="math:min(exsl:node-set($p2pConn_heights_)/MEMCONN/@Y)"/>
						<xsl:variable name="busBot_" select="math:max(exsl:node-set($p2pConn_heights_)/MEMCONN/@Y)"/>
						
						<xsl:variable name="topRnk_" select="(exsl:node-set($p2pConn_heights_)/MEMCONN[(@Y = $busTop_)]/@BIFRANK)"/>
						<xsl:variable name="botRnk_" select="(exsl:node-set($p2pConn_heights_)/MEMCONN[(@Y = $busBot_)]/@BIFRANK)"/>
						
<!--						
						<xsl:message>Bot Rank  <xsl:value-of select="$botRnk_"/></xsl:message>	
-->						
						
						
						<xsl:call-template name="Draw_P2PBus">
							<xsl:with-param name="busX"    select="$bcInSpace_X_"/>
							<xsl:with-param name="busTop"  select="$busTop_"/>
							<xsl:with-param name="busBot"  select="$busBot_"/>
							<xsl:with-param name="topRnk"  select="$topRnk_"/>
							<xsl:with-param name="botRnk"  select="$botRnk_"/>
							<xsl:with-param name="busStd"  select="$busStd_"/>
							<xsl:with-param name="busName" select="$busName_"/>
						</xsl:call-template>
						
 				</xsl:when>
					
<!-- 
		 ===========================================================
			Handle MultiStack Point to Point connections
		 ===========================================================
-->
					
				<xsl:when test="@BUSLANE_X and (@IS_MULTISTK) and not(@IS_SBSCONN) and BUSCONN[@BIF_Y and @IS_PROCONN and @INSTANCE and @BUSINTERFACE]">
					
				<!-- Store the connections in a variable -->
					<xsl:variable name="bcInSpace_X_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2) - ceiling($BUS_ARROW_W div 2))"/>
					
					<xsl:variable name="multiConns_">
						
						<xsl:for-each select="BUSCONN">
							
							<xsl:variable name="bifName_"      select="@BUSINTERFACE"/>
						
							<xsl:variable name="multiInstance_" select="@INSTANCE"/>
						
							<xsl:variable name="mulshp_hori_idx_">
								<xsl:choose>
									<xsl:when test="@IS_PROCONN">
										<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $multiInstance_)]/@STACK_HORIZ_INDEX"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $multiInstance_)])]/@STACK_HORIZ_INDEX"/>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:variable> 
							
							<xsl:variable name="mulshp_vert_idx_">
								<xsl:choose>
									<xsl:when test="@IS_PROCONN">
										<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $multiInstance_)]/@SHAPE_VERTI_INDEX"/>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $multiInstance_)])]/@SHAPE_VERTI_INDEX"/>
									</xsl:otherwise>
								</xsl:choose>
							</xsl:variable> 
<!--							
							<xsl:message>Shape Horiz <xsl:value-of select="$mulshp_hori_idx_"/></xsl:message>
							<xsl:message>Shape Verti <xsl:value-of select="$mulshp_vert_idx_"/></xsl:message>
-->	
							
							<xsl:variable name="mulshp_Y_">
								<xsl:call-template name="_calc_Stack_Shape_Y">
									<xsl:with-param name="horizIdx"  select="$mulshp_hori_idx_"/>
									<xsl:with-param name="vertiIdx"  select="$mulshp_vert_idx_"/>
								</xsl:call-template>
							</xsl:variable>
						
						<xsl:variable name="cmplxStack_H_diff_">
							<xsl:choose>
								<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
								<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
				
									<xsl:variable name="stackToWest_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
									</xsl:call-template>
								</xsl:variable>
				
								<xsl:variable name="stackToEast_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
									</xsl:call-template>
								</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($mulshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($mulshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
									
							</xsl:when>
						</xsl:choose>
					</xsl:variable>
						
																				
					<xsl:variable name="procStack_H_diff_">
						<xsl:choose>
							<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
							<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
			
								<xsl:variable name="stackToWest_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
											<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
										</xsl:call-template>
								</xsl:variable>
				
								<xsl:variable name="stackToEast_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
									</xsl:call-template>
								</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($mulshp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($mulshp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
									
								</xsl:when>
							</xsl:choose>
						</xsl:variable>
							
						<xsl:choose>
							
							<xsl:when test="@IS_PROCONN and @BIF_Y">
										
								<xsl:variable name="procBif_Y_"   select="((($BIF_H + $BIF_GAP) * @BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
										
								<xsl:variable name="procBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $multiInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
								<xsl:variable name="procBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $multiInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
										
								<xsl:variable name="bcProc_Y_"     select="($mulshp_Y_ + $procBif_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $procStack_H_diff_)"/>
										
								<MULTICONN X="{$bcInSpace_X_}" Y="{$bcProc_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$procBifRank_}" BIFSIDE="{$procBifSide_}"/>
										
							</xsl:when>
									
							<xsl:otherwise>
								
											
								<xsl:variable name="modType_"         select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $multiInstance_)]/@MODCLASS"/>
								<xsl:variable name="modBifSide_"      select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $multiInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
								<xsl:variable name="modBif_Y_"        select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $multiInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_Y"/>
								
<!--								
								<xsl:message>Mod Inst <xsl:value-of select="$multiInstance_"/></xsl:message>	
								<xsl:message>Mod Type <xsl:value-of select="$modType_"/></xsl:message>	
-->								
								<xsl:variable name="modBc_Y_">
									<xsl:choose>
										<xsl:when test="($modType_ = 'MEMORY_CNTLR')">
								      		<xsl:value-of select="$periMOD_H + $MOD_LANE_H + ((($BIF_H + $BIF_GAP) * $modBif_Y_))"/>
										</xsl:when>
										<xsl:otherwise>
								      		<xsl:value-of select="((($BIF_H + $BIF_GAP) * $modBif_Y_) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
										</xsl:otherwise>
									</xsl:choose>
								</xsl:variable>   
						
<!--								
								<xsl:variable name="modshp_Y_">
									<xsl:call-template name="_calc_Stack_Shape_Y">
										<xsl:with-param name="horizIdx"  select="$mem_procshp_hori_idx_"/>
										<xsl:with-param name="vertiIdx"  select="$modshp_vert_idx_"/>
									</xsl:call-template>
								</xsl:variable>
-->								
								
									<xsl:variable name="modBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $multiInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
									<xsl:variable name="bcMod_Y_"     select="($mulshp_Y_ + $modBc_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $cmplxStack_H_diff_)"/>
										
<!--										
										<xsl:message>Bc Bif Y <xsl:value-of select="$modBif_Y_"/></xsl:message>	
										<xsl:message>Bc Mod Y <xsl:value-of select="$modBc_Y_"/></xsl:message>	
-->										
									<MULTICONN X="{$bcInSpace_X_}" Y="{$bcMod_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$modBifRank_}" BIFSIDE="{$modBifSide_}"/>
						
								</xsl:otherwise>
						</xsl:choose>	
									
						</xsl:for-each>
					</xsl:variable>
					
						<!-- Draw the busconnection and horizontal lines.-->						
						<xsl:for-each select="exsl:node-set($multiConns_)/MULTICONN">
							
							
							<xsl:variable name="horz_line_y_" select="(@Y + ceiling($BIFC_H div 2))"/>
						
							<xsl:variable name="horz_line_x1_"  select="($laneInSpace_X_ +  ceiling($BIFC_W div 2))"/>
							<xsl:variable name="horz_line_x2_">
								<xsl:choose>
									<xsl:when test="@BIFSIDE = '0'">
							 			<xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_)"/>
									</xsl:when>
									<xsl:otherwise>0</xsl:otherwise>
								</xsl:choose>
							</xsl:variable>
						
							<line x1="{$horz_line_x1_}"
				 		      	  y1="{$horz_line_y_}" 
				  		      	  x2="{$horz_line_x2_}"
				  			      y2="{$horz_line_y_}" 
				  			      style="stroke:{$bus_color_};stroke-width:1"/>
<!--							
-->							
							<use   x="{@X}"   y="{@Y}"  xlink:href="#{@BUSSTD}_busconn_{@BIFRANK}"/>
							
						</xsl:for-each>
						
						<xsl:variable name="busTop_" select="math:min(exsl:node-set($multiConns_)/MULTICONN/@Y)"/>
						<xsl:variable name="busBot_" select="math:max(exsl:node-set($multiConns_)/MULTICONN/@Y)"/>
						
						<xsl:variable name="topRnk_" select="(exsl:node-set($multiConns_)/MULTICONN[(@Y = $busTop_)]/@BIFRANK)"/>
						<xsl:variable name="botRnk_" select="(exsl:node-set($multiConns_)/MULTICONN[(@Y = $busBot_)]/@BIFRANK)"/>
<!--						
						<xsl:message>Bot Rank  <xsl:value-of select="$botRnk_"/></xsl:message>	
-->						
						
						<xsl:call-template name="Draw_P2PBus">
							<xsl:with-param name="busX"    select="$bcInSpace_X_"/>
							<xsl:with-param name="busTop"  select="$busTop_"/>
							<xsl:with-param name="busBot"  select="$busBot_"/>
							<xsl:with-param name="topRnk"  select="$topRnk_"/>
							<xsl:with-param name="botRnk"  select="$botRnk_"/>
							<xsl:with-param name="busStd"  select="$busStd_"/>
							<xsl:with-param name="busName" select="$busName_"/>
						</xsl:call-template>
					
				</xsl:when>
						
<!-- 
		 ===========================================================
			Handle Processor to processor connections
		 ===========================================================
-->
				<xsl:when test="(@IS_PROC2PROC and (count(BUSCONN[@BIF_Y and @INSTANCE and @BUSINTERFACE]) = 2))">
<!--					
					<xsl:message>Found a proc 2 proc connection </xsl:message>	
-->					
	
					<xsl:variable name="pr2pr_stkToWest_"   select="math:min(BUSCONN/@STACK_HORIZ_INDEX)"/>
					<xsl:variable name="pr2pr_stkToEast_"   select="math:max(BUSCONN/@STACK_HORIZ_INDEX)"/>
<!--					
					
					<xsl:message>stack to West  <xsl:value-of select="$pr2pr_stkToWest_"/></xsl:message>
					<xsl:message>stack to East  <xsl:value-of select="$pr2pr_stkToEast_"/></xsl:message>
-->	
					
					<xsl:variable name="proc2procConn_heights_">
	
						<xsl:for-each select="BUSCONN">
					
							<xsl:variable name="procInstance_" select="@INSTANCE"/>
							<xsl:variable name="bifName_"      select="@BUSINTERFACE"/>
						
							<xsl:variable name="procshp_hori_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInstance_)]/@STACK_HORIZ_INDEX"/>
							<xsl:variable name="procshp_vert_idx_" select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInstance_)]/@SHAPE_VERTI_INDEX"/>
						
							<xsl:variable name="procshp_Y_">
								<xsl:call-template name="_calc_Stack_Shape_Y">
									<xsl:with-param name="horizIdx"  select="$procshp_hori_idx_"/>
									<xsl:with-param name="vertiIdx"  select="$procshp_vert_idx_"/>
								</xsl:call-template>
							</xsl:variable>
						
							<xsl:variable name="procStack_H_diff_">
								<xsl:choose>
									
									<xsl:when test="   (($pr2pr_stkToEast_ = 'NONE') or ($pr2pr_stkToWest_ = 'NONE'))">0</xsl:when>
									<xsl:when test="not(($pr2pr_stkToEast_ = 'NONE') or ($pr2pr_stkToWest_ = 'NONE'))">
			
										<xsl:variable name="stackToWest_AbvSbs_H_">
											<xsl:call-template name="_calc_Stack_AbvSbs_Height">
												<xsl:with-param name="stackIdx"  select="$pr2pr_stkToWest_"/>
											</xsl:call-template>
										</xsl:variable>
				
										<xsl:variable name="stackToEast_AbvSbs_H_">
											<xsl:call-template name="_calc_Stack_AbvSbs_Height">
												<xsl:with-param name="stackIdx"  select="$pr2pr_stkToEast_"/>
											</xsl:call-template>
										</xsl:variable>
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
										<xsl:choose>
											<xsl:when test="(($procshp_hori_idx_ = $pr2pr_stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
												<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
											</xsl:when>	
											<xsl:when test="(($procshp_hori_idx_ = $pr2pr_stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
												<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
											</xsl:when>	
											<xsl:otherwise>
<!--												
												<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
												<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->												
												0
											</xsl:otherwise>	
										</xsl:choose>	
										
									</xsl:when>
								</xsl:choose>
							</xsl:variable>
						
							<!-- Store the conns in a variable -->	
							<xsl:variable name="procBif_Y_"   select="((($BIF_H + $BIF_GAP) * @BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
											
							<xsl:variable name="procBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
							<xsl:variable name="procBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
									
							<xsl:variable name="bcInSpace_X_">
								<xsl:choose>
									<xsl:when test="$procBifSide_ = '1'"><xsl:value-of select="ceiling($periMOD_W div 2)"/></xsl:when>
									<xsl:when test="$procBifSide_ = '0'"><xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_ - ceiling($periMOD_W div 2) - $BIFC_W)"/></xsl:when>
								</xsl:choose>
								
							</xsl:variable> 
							
							<xsl:variable name="bcProc_Y_"     select="($procshp_Y_ + $procBif_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $procStack_H_diff_)"/>
							
							
<!--							
							<xsl:message>Conn X <xsl:value-of select="$bcInSpace_X_"/></xsl:message>
							<xsl:message>Conn Y <xsl:value-of select="$bcProc_Y_"/></xsl:message>
-->							
										
							<PR2PRCONN X="{$bcInSpace_X_}" Y="{$bcProc_Y_}" BUSSTD="{$busStd_}" BIFRANK="{$procBifRank_}" BIFSIDE="{$procBifSide_}" SHAPE_ID="{$procshp_hori_idx_}"/>
						</xsl:for-each>
					</xsl:variable>
					
					<xsl:variable name="pr2prLeft_"   select="math:min(exsl:node-set($proc2procConn_heights_)/PR2PRCONN/@SHAPE_ID)"/>
					<xsl:variable name="pr2prRght_"   select="math:max(exsl:node-set($proc2procConn_heights_)/PR2PRCONN/@SHAPE_ID)"/>
				
					<xsl:variable name="pr2pr_stack_Left_X_">
						<xsl:call-template name="_calc_Stack_X"> 
							<xsl:with-param name="stackIdx"  select="$pr2prLeft_"/>
						</xsl:call-template>		
					</xsl:variable>	
					
					<xsl:variable name="pr2pr_stack_Rght_X_">
						<xsl:call-template name="_calc_Stack_X"> 
							<xsl:with-param name="stackIdx"  select="$pr2prRght_"/>
						</xsl:call-template>		
					</xsl:variable>	
					
<!--					
					<xsl:message>Left stack X <xsl:value-of select="$pr2pr_stack_Left_X_"/></xsl:message>
					<xsl:message>Rght stack X <xsl:value-of select="$pr2pr_stack_Rght_X_"/></xsl:message>
-->					
					<xsl:variable name="pr2pr_space_W_" select="($pr2pr_stack_Rght_X_ - $pr2pr_stack_Left_X_)"/>
						
					
					<xsl:variable name="pr2pr_extStackEast_W_">
						<xsl:call-template name="_calc_Stack_Width">
							<xsl:with-param name="stackIdx"  select="$pr2prRght_"/>
						</xsl:call-template>
					</xsl:variable>
					
					<xsl:variable name="pr2pr_extStackWest_W_">
						<xsl:call-template name="_calc_Stack_Width">
							<xsl:with-param name="stackIdx"  select="$pr2prLeft_"/>
						</xsl:call-template>
					</xsl:variable>
					
<!--					
					<xsl:message>Space W <xsl:value-of select="$pr2pr_space_W_"/></xsl:message>
					<xsl:message>Rght stack <xsl:value-of select="$pr2pr_extStackEast_W_"/></xsl:message>
					<xsl:message>Left stack <xsl:value-of select="$pr2pr_extStackWest_W_"/></xsl:message>
-->					
	
					<xsl:variable name="connLeft_X_" select="ceiling($periMOD_W div 2)"/>
					<xsl:variable name="connRght_X_" select="($pr2pr_space_W_ - ceiling($pr2pr_extStackWest_W_ div 2) + ceiling($pr2pr_extStackEast_W_ div 2) - ceiling($periMOD_W div 2) - $BIFC_W)"/>
					
					<!-- Draw the busconnections .-->						
					<xsl:for-each select="exsl:node-set($proc2procConn_heights_)/PR2PRCONN">
							<xsl:variable name="conn_X_">
								<xsl:choose>
									<xsl:when test="@BIFSIDE = '1'"><xsl:value-of select="ceiling($periMOD_W div 2)"/></xsl:when>
									<xsl:when test="@BIFSIDE = '0'"><xsl:value-of select="($pr2pr_space_W_ - ceiling($pr2pr_extStackWest_W_ div 2) + ceiling($pr2pr_extStackEast_W_ div 2) - ceiling($periMOD_W div 2) - $BIFC_W)"/></xsl:when>
<!--									
									<xsl:when test="@BIFSIDE = '0'"><xsl:value-of select="($pr2pr_space_W_ + $pr2pr_extStackWest_W_ + $pr2pr_extStackEast_W_ - ceiling($periMOD_W div 2) - $BIFC_W)"/></xsl:when>
-->	
								</xsl:choose>
							</xsl:variable> 
							
						
						<use   x="{$conn_X_}"   y="{@Y}"  xlink:href="#{@BUSSTD}_busconn_{@BIFRANK}"/>
					</xsl:for-each>
					
					<xsl:variable name="bc_Y_"     select="math:min(exsl:node-set($proc2procConn_heights_)/PR2PRCONN/@Y)"/>
					<xsl:variable name="bcLeft_"   select="math:min(exsl:node-set($proc2procConn_heights_)/PR2PRCONN/@X)"/>
					<xsl:variable name="bcRght_"   select="math:max(exsl:node-set($proc2procConn_heights_)/PR2PRCONN/@X)"/>
					
					<xsl:variable name="leftRnk_"  select="(exsl:node-set($proc2procConn_heights_)/PR2PRCONN[(@X = $bcLeft_)]/@BIFRANK)"/>
					<xsl:variable name="rghtRnk_"  select="(exsl:node-set($proc2procConn_heights_)/PR2PRCONN[(@X = $bcRght_)]/@BIFRANK)"/>
						
					<xsl:call-template name="Draw_Proc2ProcBus">
						<xsl:with-param name="bc_Y"     select="$bc_Y_"/>
							
						<xsl:with-param name="bcLeft_X" select="$connLeft_X_ + $BIFC_W"/>
						<xsl:with-param name="bcRght_X" select="$connRght_X_"/>
						
						<xsl:with-param name="leftRnk"  select="$leftRnk_"/>
						<xsl:with-param name="rghtRnk"  select="$rghtRnk_"/>
						
						<xsl:with-param name="busStd"   select="$busStd_"/>
						<xsl:with-param name="busName"  select="$busName_"/>
					</xsl:call-template>
				
				</xsl:when>
					
					
					
<!-- 
		 ===========================================================
			Handle Split connections, (connections that go between adjacent stacks)
		 ===========================================================
-->
				<xsl:when test="(BUSCONN[@BIF_Y and @INSTANCE and @BUSINTERFACE and @IS_SPLITCONN])">
					
					<xsl:variable name="bifName_"      select="BUSCONN/@BUSINTERFACE"/>
					<xsl:variable name="shpInstance_"  select="BUSCONN/@INSTANCE"/>
					
<!--					
					<xsl:message>Found a split connection on <xsl:value-of select="$shpInstance_"/></xsl:message>	
-->					
						
					
					<xsl:variable name="shp_hori_idx_">
						
						<xsl:choose>
							<xsl:when  test="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $shpInstance_)]">
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $shpInstance_)]/@STACK_HORIZ_INDEX"/>
							</xsl:when>
							
							<xsl:when  test="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $shpInstance_)])]">
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $shpInstance_)])]/@STACK_HORIZ_INDEX"/>
							</xsl:when>
							<xsl:otherwise>_unknown_</xsl:otherwise>
						</xsl:choose>		
						
					</xsl:variable> 
					
					<xsl:variable name="shp_vert_idx_">
						
						<xsl:choose>
							<xsl:when  test="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $shpInstance_)]">
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $shpInstance_)]/@SHAPE_VERTI_INDEX"/>
							</xsl:when>
							
							<xsl:when  test="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $shpInstance_)])]">
								<xsl:value-of select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $shpInstance_)])]/@SHAPE_VERTI_INDEX"/>
							</xsl:when>
							<xsl:otherwise>_unknown_</xsl:otherwise>
						</xsl:choose>		
						
					</xsl:variable> 
					
					<xsl:variable name="splitshp_Width_">
						
						<xsl:choose>
							<xsl:when  test="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $shpInstance_)]">
								<xsl:value-of select="$periMOD_W"/>
							</xsl:when>
							
							<xsl:when  test="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $shpInstance_)])]/@MODS_W">
<!--								
								<xsl:message>Using mods width on <xsl:value-of select="$shpInstance_"/></xsl:message>
-->								
								<xsl:value-of select="(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(MODULE[(@INSTANCE = $shpInstance_)])]/@MODS_W * $periMOD_W)"/>
							</xsl:when>
							<xsl:otherwise>
								<xsl:value-of select="$periMOD_W"/>
							</xsl:otherwise>
						</xsl:choose>		
						
					</xsl:variable> 
					
<!--					
					<xsl:message>Found width of <xsl:value-of select="$splitshp_Width_"/></xsl:message>
-->					
	
					
					<xsl:variable name="splitshp_Y_">
						<xsl:call-template name="_calc_Stack_Shape_Y">
							<xsl:with-param name="horizIdx"  select="$shp_hori_idx_"/>
							<xsl:with-param name="vertiIdx"  select="$shp_vert_idx_"/>
						</xsl:call-template>
					</xsl:variable>
					
						
					<xsl:variable name="splitStack_H_diff_">
						<xsl:choose>
							<xsl:when test="   (($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">0</xsl:when>
							<xsl:when test="not(($stkToEast_ = 'NONE') or ($stkToWest_ = 'NONE'))">
			
								<xsl:variable name="stackToWest_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToWest_"/>
									</xsl:call-template>
								</xsl:variable>
				
								<xsl:variable name="stackToEast_AbvSbs_H_">
									<xsl:call-template name="_calc_Stack_AbvSbs_Height">
										<xsl:with-param name="stackIdx"  select="$stkToEast_"/>
									</xsl:call-template>
								</xsl:variable>
				
<!--				
				<xsl:message>stack to west H <xsl:value-of select="$stackToWest_AbvSbs_H_"/></xsl:message>
				<xsl:message>stack to east H <xsl:value-of select="$stackToEast_AbvSbs_H_"/></xsl:message>
-->				
								<xsl:choose>
									<xsl:when test="(($shp_hori_idx_ = $stkToEast_) and ($stackToWest_AbvSbs_H_ &gt; $stackToEast_AbvSbs_H_))">
										<xsl:value-of select="($stackToWest_AbvSbs_H_ - $stackToEast_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:when test="(($shp_hori_idx_ = $stkToWest_) and ($stackToEast_AbvSbs_H_ &gt; $stackToWest_AbvSbs_H_))">
										<xsl:value-of select="($stackToEast_AbvSbs_H_ - $stackToWest_AbvSbs_H_)"/>
									</xsl:when>	
									<xsl:otherwise>0</xsl:otherwise>	
								</xsl:choose>	
										
							</xsl:when>
						</xsl:choose>
					</xsl:variable>
						
					
					<xsl:variable name="splitBif_Y_"    select="((($BIF_H + $BIF_GAP) * BUSCONN/@BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
					<xsl:variable name="splitBifRank_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $shpInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIFRANK"/>
					<xsl:variable name="splitBifSide_"  select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $shpInstance_)]/BUSINTERFACE[(@NAME = $bifName_)]/@BIF_X"/>
									
					<xsl:variable name="bcInSpace_X_">
						<xsl:choose>
							<xsl:when test="$splitBifSide_ = '1'"><xsl:value-of select="ceiling($splitshp_Width_ div 2)"/></xsl:when>
							<xsl:when test="$splitBifSide_ = '0'"><xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_ - ceiling($splitshp_Width_ div 2) - $BIFC_W)"/></xsl:when>
						</xsl:choose>
								
					</xsl:variable> 
					
					<xsl:variable name="bcBus_X_">
						<xsl:choose>
							<xsl:when test="$splitBifSide_ = '1'"><xsl:value-of select="$bcInSpace_X_"/></xsl:when>
							<xsl:when test="$splitBifSide_ = '0'"><xsl:value-of select="($space_W_ + $extSpaceWest_W_ + $extSpaceEast_W_ - ceiling($periMOD_W div 2) - $BIFC_W)"/></xsl:when>
						</xsl:choose>
								
					</xsl:variable> 
							
					<xsl:variable name="bcSplit_Y_">
						<xsl:choose>
							<xsl:when test="(BUSCONN/@IS_MEMCONN)">
<!--								
								<xsl:message>Found memory conn split connection on <xsl:value-of select="$shpInstance_"/> </xsl:message>
-->	
								<xsl:value-of select="($splitshp_Y_ + $periMOD_H + $BIF_GAP + (BUSCONN/@BIF_Y * ($BIF_H + $BIF_GAP)) + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $splitStack_H_diff_)"/>
							</xsl:when>     
							<xsl:otherwise>
								<xsl:value-of select="($splitshp_Y_ + $splitBif_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $splitStack_H_diff_)"/>
							</xsl:otherwise>
						</xsl:choose>
					</xsl:variable>    
<!--								
								<xsl:value-of select="($splitshp_Y_ + $splitBif_Y_ + ceiling($BIF_H div 2) - ceiling($BIFC_H div 2) + $splitStack_H_diff_)"/>
-->								
<!--					
					<xsl:message>VERTI INDEX <xsl:value-of select="$shp_vert_idx_"/></xsl:message>	
					<xsl:message>BIF Y <xsl:value-of select="$splitBif_Y_"/></xsl:message>	
					<xsl:message>HORIZ INDEX <xsl:value-of select="$shp_hori_idx_"/></xsl:message>	
-->					
					
					<use   x="{$bcInSpace_X_}"   y="{$bcSplit_Y_}"  xlink:href="#{@BUSSTD}_busconn_{$splitBifRank_}"/>
					
					
					<xsl:call-template name="Draw_SplitConnBus">
						<xsl:with-param name="bc_Y"    select="$bcSplit_Y_"/>
						<xsl:with-param name="bc_X"    select="$bcInSpace_X_"/>
						
						<xsl:with-param name="bc_Rnk"  select="$splitBifRank_"/>
						<xsl:with-param name="bc_Side" select="$splitBifSide_"/>
						
						<xsl:with-param name="busStd"  select="$busStd_"/>
						<xsl:with-param name="busName" select="$busName_"/>
					</xsl:call-template>
					
				</xsl:when>
										
					
				</xsl:choose>
								
			</xsl:for-each>
	</symbol>
			
</xsl:template>	

	
	
<xsl:template name="Define_PromotedComplexStacks"> 
	
	<xsl:variable name="numSBSs_"     select="count(/EDKSYSTEM/BLKDSHAPES/SBSSHAPES/MODULE)"/>	
	<xsl:for-each select="/EDKSYSTEM//BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(@CSTACK_INDEX) and (@IS_PROMOTED)]">
	
		<xsl:variable name="cstackW_" select="$periMOD_W"/>	
		<xsl:variable name="cstackH_">	
			<xsl:call-template name="_calc_CStackShapesAbv_Height">
				<xsl:with-param name="cstackIndex"  select="@CSTACK_INDEX"/>
				<xsl:with-param name="cstackModY"   select="@MODS_H"/>
			</xsl:call-template>	
		</xsl:variable>
		
<!--		
		<xsl:message>Found stack of <xsl:value-of select="$cstackH_"/></xsl:message>
-->		

		<xsl:variable name="cstackIdx_" select="@CSTACK_INDEX"/>
		
		<!-- Define the stack's peripheral shapes -->	
		<xsl:for-each select="MODULE">
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			<xsl:variable name="modType_" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $modInst_)]/@MODTYPE"/>
			<xsl:call-template name="Define_Peripheral"> 
				<xsl:with-param name="modInst" select="$modInst_"/>
				<xsl:with-param name="modType" select="$modType_"/>
			</xsl:call-template>		
		</xsl:for-each>	
		
		<xsl:variable name="sbsH_"     select="($numSBSs_   * $SBS_H)"/>

		<!-- Define the processor's bus lanes and lines -->	
		<xsl:for-each select="BUSCONNS">	
		
			<xsl:variable name="busConnId_">cstack_<xsl:value-of select="../@CSTACK_INDEX"/></xsl:variable>

			<xsl:call-template name="Define_BusConnLanes">		
				<xsl:with-param name="procInst" select="$busConnId_"/>
				<xsl:with-param name="procH"    select="0"/>
				<xsl:with-param name="procW"    select="$periMOD_W"/>
				<xsl:with-param name="procY"    select="0"/>
				<xsl:with-param name="memUH"    select="0"/>
				<xsl:with-param name="sbsGap"   select="$sbsH_"/>
				<xsl:with-param name="pstackH"  select="$cstackH_"/>
				<xsl:with-param name="pstackW"  select="$cstackW_"/>
			</xsl:call-template>		
			
			<xsl:call-template name="Define_BusConnLines">		
				<xsl:with-param name="procInst" select="$busConnId_"/>
				<xsl:with-param name="procH"    select="0"/>
				<xsl:with-param name="procW"    select="$periMOD_W"/>
				<xsl:with-param name="procY"    select="0"/>
				<xsl:with-param name="memUH"    select="0"/>
				<xsl:with-param name="sbsGap"   select="$sbsH_"/>
				<xsl:with-param name="pstackH"  select="$cstackH_"/>
				<xsl:with-param name="pstackW"  select="$cstackW_"/>
			</xsl:call-template>		
		</xsl:for-each>
		
		<!-- Now use all this stuff to draw the stack-->	
		
		<symbol id="cstack_{@CSTACK_INDEX}">
		
			<rect x="0"
				  y="0"
			      rx="6" 
			      ry="6" 
		          width = "{$cstackW_}"
		          height= "{$cstackH_}"
			      style="fill:{$COL_BG}; stroke:none;"/>
			
			<!--  Define the cstack -->	
			<!-- First draw the the processor's peripherals-->	
			<xsl:for-each select="MODULE">
				<xsl:variable name="cstackInst_" select="@INSTANCE"/>
							
				<xsl:variable name="cstack_y_">	
					<xsl:call-template name="_calc_CStackShapesAbv_Height">
						<xsl:with-param name="cstackIndex"  select="../@CSTACK_INDEX"/>
						<xsl:with-param name="cstackModY"   select="@CSTACK_MODS_Y"/>
					</xsl:call-template>	
				</xsl:variable>
					
	 			<use   x="0"  y="{$cstack_y_}"  xlink:href="#symbol_{$cstackInst_}"/> 
			</xsl:for-each>
		</symbol>
		
		
		<!--    And now combine buslanes, bus lines etc into the entire group for the processor-->	
		<symbol id="cgroup_{@CSTACK_INDEX}">
		
			<xsl:variable name="busConnsWestW_">
				<xsl:if test="BUSCONNS[(@ORIENTED = 'WEST')]">
			  		<xsl:value-of select="BUSCONNS[(@ORIENTED = 'WEST')]/@BUSLANE_W"/>	
				</xsl:if>
				<xsl:if test="not(BUSCONNS[(@ORIENTED = 'WEST')])">0</xsl:if>
			</xsl:variable>
				
			<xsl:variable name="busConnsEastW_">
				<xsl:if test="BUSCONNS[(@ORIENTED = 'EAST')]">
			  		<xsl:value-of select="BUSCONNS[(@ORIENTED = 'EAST')]/@BUSLANE_W"/>	
				</xsl:if>
				<xsl:if test="not(BUSCONNS[(@ORIENTED = 'EAST')])">0</xsl:if>
			</xsl:variable>
			
			<xsl:variable name="busConnsWestX_"  select="0"/>
			<xsl:variable name="cstackX_"        select="($busConnsWestW_ * $BUS_LANE_W)"/>
			<xsl:variable name="busConnsEastX_"  select="($cstackX_ + $periMOD_W + 1)"/>
					
			<!-- Draw the Bus lanes -->	
			<xsl:if test="BUSCONNS[(@ORIENTED = 'EAST')]">
				<use  x="{$busConnsEastX_}"  y="0"  xlink:href="#buslanes_cstack_{@CSTACK_INDEX}_EAST"/> 
			</xsl:if>		
			<xsl:if test="BUSCONNS[(@ORIENTED = 'WEST')]">
				<use  x="{$busConnsWestX_}"  y="0"  xlink:href="#buslanes_cstack_{@CSTACK_INDEX}_WEST"/> 
			</xsl:if>		
			
			<use  x="{$cstackX_}"            y="{$sbsH_}"  xlink:href="#cstack_{@CSTACK_INDEX}"/> 
			
			<!-- Draw the Bus Lines on top of the lanes -->	
			<xsl:if test="BUSCONNS[(@ORIENTED = 'EAST')]">
				<use  x="{$busConnsEastX_ - ceiling($cstackW_ div 2)}"  y="0"  xlink:href="#buslines_cstack_{@CSTACK_INDEX}_EAST"/> 
			</xsl:if>		
			
			<xsl:if test="BUSCONNS[(@ORIENTED = 'WEST')]">
				<use  x="{$busConnsWestX_}"                             y="0"  xlink:href="#buslines_cstack_{@CSTACK_INDEX}_WEST"/> 
			</xsl:if>		

		</symbol>
	</xsl:for-each>		
	
</xsl:template>	

	
		
	
<xsl:template name="__Define_ComplexStack__"> 
	<xsl:param name="cstkIndex" select="100"/>
	
<!--	
	<xsl:message>Drawing multiprocessor stack at <xsl:value-of select="$stackBlkd_X"/></xsl:message>
	<xsl:message>Number of memory units in stack are <xsl:value-of select="$numMemusInStack_"/></xsl:message>
	<xsl:message>Width of mpstack is <xsl:value-of select="$mpStack_W_"/></xsl:message>
-->	
	
	<xsl:variable name="numPerisInStk_"  select="count(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@CSTACK_INDEX = $cstkIndex) and (@MODCLASS = 'PERIPHERAL'))])"/>	
	<xsl:variable name="numMemusInStk_"  select="count(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@CSTACK_INDEX = $cstkIndex) and (@MODCLASS = 'MEMORY_UNIT'))])"/>	
	
	<xsl:variable name="cstk_Width">
		<xsl:if test="($numMemusInStk_ &gt; 0)">
			<xsl:value-of select="($periMOD_W * 2)"/>
		</xsl:if>
		<xsl:if test="not($numMemusInStk_ &gt; 0)">
			<xsl:value-of select="$periMOD_W"/>
		</xsl:if>
	</xsl:variable>

	<xsl:variable name="cstk_Height_">	
		<xsl:call-template name="_calc_CStackShapesAbv_Height">
			<xsl:with-param name="cstkIndex"  select="$cstkIndex"/>
		</xsl:call-template>	
	</xsl:variable>
		
<!--	
	<xsl:variable name="mpstackH_" select="(($numPerisInStack_  * ($periMOD_H + $BIF_H)) + ($numMemusInStack_  * (($periMOD_H * 2) + $BIF_H)))"/>
-->	
	
	<!-- Define the multistack's peripherals -->	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@CSTACK_INDEX = $cstkIndex) and (@MODCLASS='PERIPHERAL'))]">	
		
		<xsl:for-each select="MODULE">
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			<xsl:variable name="modType_" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $modInst_)]/@MODTYPE"/>
			<xsl:call-template name="Define_Peripheral"> 
				<xsl:with-param name="modInst"    select="$modInst_"/>
				<xsl:with-param name="modType"    select="$modType_"/>
				<xsl:with-param name="cstkIndex"  select="../@CSTACK_INDEX"/>
				<xsl:with-param name="cstkMods_Y" select="../@CSTACK_MODS_Y"/>
			</xsl:call-template>		
		</xsl:for-each>	
	</xsl:for-each>
	
<!--		
		<xsl:variable name="mpstackMemuInst_">mp_memu_<xsl:value-of select="$stackBlkd_X"/>_<xsl:value-of select="@MPSTACK_MEMUS_Y"/></xsl:variable>
-->
		
	<!-- Define the multistack's memory blocks -->	
	<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@CSTACK_INDEX = $cstkIndex) and (@MODCLASS='MEMORY_UNIT'))]">	
		<xsl:call-template name="Define_MemoryUnit"> 
			<xsl:with-param name="cstkIndex"  select="@CSTACK_INDEX"/>
			<xsl:with-param name="cstkMods_Y" select="@CSTACK_MODS_Y"/>
		</xsl:call-template>
	</xsl:for-each>
	
	<xsl:variable name="stack_name_">
		<xsl:call-template name="_gen_CStack_StackName"> 
			<xsl:with-param name="cstkIndex"  select="$cstkIndex"/>
		</xsl:call-template>		
	</xsl:variable>
	
	<symbol id="{$stack_name_}">
			
<!--			
			<rect x="0"
				  y="0"
			      rx="6" 
			      ry="6" 
		          width = "{$mpstackW_}"
		          height= "{$mpstackH_}"
			      style="fill:{$COL_YELLOW}; stroke:none;"/>
-->	
				  
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(@CSTACK_INDEX = $cstkIndex)]">
				<xsl:sort select="@CSTACK_MODS_Y" data-type="number"/>
				
				<xsl:variable name="symbol_name_">
					<xsl:call-template name="_gen_CStack_SymbolName"> 
						<xsl:with-param name="cstkIndex"  select="@CSTACK_INDEX"/>
						<xsl:with-param name="cstkMods_Y" select="@CSTACK_MODS_Y"/>
					</xsl:call-template>		
				</xsl:variable>
			
				<xsl:variable name="shapeInst_" select="(MODULE/@INSTANCE)"/>
				<xsl:variable name="shapeW_"    select="(@MODS_W * $periMOD_W)"/>
				<xsl:variable name="shapeH_"    select="(@MODS_H * $periMOD_H)"/>
				
				<xsl:variable name="shapeX_"    select="(ceiling($cstk_Width div 2) - ceiling($shapeW_ div 2))"/>
				<xsl:variable name="shapeY_">	
					<xsl:call-template name="_calc_CStackShapesAbv_Height">
						<xsl:with-param name="cstkIndex"   select="@CSTACK_INDEX"/>
						<xsl:with-param name="cstkMods_Y"  select="@CSTACK_MODS_Y"/>
					</xsl:call-template>	
				</xsl:variable>
				
<!--				
				<xsl:message><xsl:value-of select="$symbol_name_"/> ====== <xsl:value-of select="$shapeY_"/></xsl:message>
-->				
			
				 <use   x="{$shapeX_}"  y="{$shapeY_}"  xlink:href="#{$symbol_name_}"/> 
			</xsl:for-each>
			
<!--				
				<xsl:variable name="mpstackMemuInst_">mp_memu_<xsl:value-of select="$stackBlkd_X"/>_<xsl:value-of select="@MPSTACK_MEMUS_Y"/></xsl:variable>
				<xsl:message>Name of memory units is <xsl:value-of select="$mpstackMemuInst_"/></xsl:message>
-->				
			<!-- Then draw the memory above the processor -->	
<!--			
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@CSTACK_INDEX = $cstkIndex) and (@MODCLASS='MEMORY_UNIT'))]">	
				<xsl:sort select="@CSTACK_MODS_Y" data-type="number"/>
				
				<xsl:variable name="shapeW_"  select="(@MODS_W * $periMOD_W)"/>
				<xsl:variable name="shapeH_"  select="(@MODS_H * $periMOD_H)"/>
				
				<xsl:variable name="memuY_"   select="(@MPSTACK_MEMUS_Y * (($periMOD_H * 2) + $BIF_H))"/>
				<xsl:variable name="periY_"   select="(@MPSTACK_MODS_Y  *  ($periMOD_H      + $BIF_H))"/>
				
				<xsl:variable name="shapeX_"  select="(ceiling($mpstackW_ div 2) - ceiling($shapeW_ div 2))"/>
				<xsl:variable name="shapeY_"  select="($memuY_ + $periY_)"/>
				
				<xsl:variable name="symbol_name_">
					<xsl:call-template name="_gen_CStack_SymbolName"> 
						<xsl:with-param name="cstkIndex"  select="$cstkIndex"/>
						<xsl:with-param name="cstkMods_Y" select="$cstkMods_Y"/>
					</xsl:call-template>		
				</xsl:variable>
						
				<use   x="{$shapeX_}"  y="{$shapeY_}"  xlink:href="#{$symbol_name_}"/> 
			</xsl:for-each>
-->	
		</symbol>		  
</xsl:template>	


<xsl:template name="Define_ProcessorMemory"> 
	<xsl:param name="procInst"    select="'_processor_'"/>
	
	<xsl:variable name="mods_h_"  select="@MODS_H"/>
	<xsl:variable name="mods_w_"  select="@MODS_W"/>
	
	<!-- first define its symbols as individual modules -->	
	<xsl:for-each select="MODULE[@MODCLASS='MEMORY']">
		<xsl:variable name="modInst_" select="@INSTANCE"/>
		<xsl:variable name="modType_" select="/EDKSYSTEM/MODULES/MODULE[@INSTANCE=$modInst_]/@MODTYPE"/>
		
		<xsl:call-template name="Define_Peripheral"> 
			<xsl:with-param name="modVori"  select="'normal'"/>
			<xsl:with-param name="modInst"  select="$modInst_"/>
			<xsl:with-param name="modType"  select="$modType_"/>
		</xsl:call-template>		
	</xsl:for-each>	
	
	<xsl:for-each select="MODULE[@MODCLASS='MEMORY_CNTLR']">
		<xsl:variable name="modInst_" select="@INSTANCE"/>
		<xsl:variable name="modType_" select="/EDKSYSTEM/MODULES/MODULE[@INSTANCE=$modInst_]/@MODTYPE"/>
		
		<xsl:call-template name="Define_Peripheral"> 
			<xsl:with-param name="modVori"  select="'rot180'"/>
			<xsl:with-param name="modInst"  select="$modInst_"/>
			<xsl:with-param name="modType"  select="$modType_"/>
		</xsl:call-template>		
	</xsl:for-each>	
	
	<xsl:variable name="memW_" select="($periMOD_W * $mods_w_)"/>
	<xsl:variable name="memH_" select="($periMOD_H * $mods_h_)"/>
	
	<xsl:variable name="memory_name_">
		<xsl:call-template name="_gen_CStack_SymbolName"> 
			<xsl:with-param name="cstkIndex"  select="../@CSTACK_INDEX"/>
			<xsl:with-param name="cstkMods_Y" select="../@CSTACK_MODS_Y"/>
		</xsl:call-template>		
	</xsl:variable>
	
<!--	
	<xsl:message>The mp stack name is <xsl:value-of select="$mp_stack_name_"/></xsl:message>
-->	
		
    <symbol id="{$memory_name_}">

		<rect x="0"
		      y="0"
			  rx="6" 
			  ry="6" 
		      width = "{$memW_}"
		      height= "{$memH_}"
			  style="fill:{$COL_BG}; stroke:{$COL_WHITE}; stroke-width:2"/>		
			  
		<!-- Draw the memory block-->		  
		<xsl:for-each select="MODULE[@MODCLASS='MEMORY']">	
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			
			 <use  x="{ceiling($memW_ div 2) - ($periMOD_W div 2)}"  
				   y="0"  
				   xlink:href="#symbol_{$modInst_}"/> 
		</xsl:for-each>
		
		<xsl:for-each select="MODULE[(@MODCLASS='MEMORY_CNTLR') and (@ORIENTED = 'WEST')]">	
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			
			 <use  x="0"  
				   y="{$periMOD_H}"  
				   xlink:href="#symbol_{$modInst_}"/> 
		</xsl:for-each>
		
		<xsl:for-each select="MODULE[(@MODCLASS='MEMORY_CNTLR') and (@ORIENTED = 'EAST')]">	
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			
			 <use  x="{$periMOD_W}"  
				   y="{$periMOD_H}"  
				   xlink:href="#symbol_{$modInst_}"/> 
		</xsl:for-each>
		
		<xsl:for-each select="MODULE[(@MODCLASS='MEMORY_CNTLR') and (@ORIENTED = 'CENTER')]">	
			<xsl:variable name="modInst_" select="@INSTANCE"/>
			
			 <use  x="{ceiling($memW_ div 2) - ($periMOD_W div 2)}"  
				   y="{$periMOD_H}"  
				   xlink:href="#symbol_{$modInst_}"/> 
		</xsl:for-each>
		
	</symbol>			  
	
</xsl:template>	

<xsl:template name="Define_Processor">
	<xsl:param name="procInst"  select="@INSTANCE"/>
	<xsl:param name="modType"   select="/EDKSYSTEM/MODULES/MODULE[@INSTANCE=$procInst]/@MODTYPE"/>
<!--	
	<xsl:param name="procType"  select="/EDKSYSTEM/MODULES/MODULE[@INSTANCE=$procInst]/@PROCTYPE"/>
-->	
	
	<xsl:variable name="label_y_">
		<xsl:value-of select="$MOD_LANE_H"/>	
	</xsl:variable>
	
<!--	
	<xsl:message>The proctype is <xsl:value-of select="$procType"/></xsl:message>	
-->
	
	<xsl:variable name="procH_" select="(($MOD_LANE_H * 2) + (($BIF_H + $BIF_GAP) * @BIFS_H) + ($MOD_LABEL_H + $BIF_GAP))"/>	
	<xsl:variable name="procW_" select="(($MOD_LANE_W * 2) + (($BIF_W             * @BIFS_W) + $BIF_GAP))"/>	
	
	<xsl:variable name="procColor">
		<xsl:choose>
			<xsl:when test="contains($modType,'microblaze')"><xsl:value-of select="$COL_PROC_BG_MB"/></xsl:when>
			<xsl:when test="contains($modType,'ppc')"><xsl:value-of select="$COL_PROC_BG_PP"/></xsl:when>
<!--			
			<xsl:when test="$procType = 'MICROBLAZE'"><xsl:value-of select="$COL_PROC_BG_MB"/></xsl:when>
			<xsl:when test="$procType = 'POWERPC'"><xsl:value-of select="$COL_PROC_BG_PP"/></xsl:when>
-->	
			<xsl:otherwise>
				<xsl:value-of select="$COL_PROC_BG_USR"/>	
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
<!--	
	<xsl:message>The proc color is <xsl:value-of select="$procColor"/></xsl:message>	
-->	
	

	<xsl:variable name="procName_">
		<xsl:call-template name="_gen_Stack_SymbolName"> 
			<xsl:with-param name="horizIdx" select="@STACK_HORIZ_INDEX"/>
			<xsl:with-param name="vertiIdx" select="@SHAPE_VERTI_INDEX"/>
		</xsl:call-template>		
	</xsl:variable>	
	
<!--	
	<xsl:message>The proc name is <xsl:value-of select="$procName_"/></xsl:message>	
-->	
	
    <symbol id="{$procName_}">

		<rect x="0"
		      y="0"
			  rx="6" 
			  ry="6" 
		      width = "{$procW_}"
		      height= "{$procH_}"
			  style="fill:{$procColor}; stroke:{$COL_WHITE}; stroke-width:2"/>		
			  
			  
		<rect x="{ceiling($procW_ div 2) - ceiling($MOD_LABEL_W div 2)}"
		      y="{$MOD_LANE_H}"
			  rx="3" 
			  ry="3" 
		      width= "{$MOD_LABEL_W}"
		      height="{$MOD_LABEL_H}"
			  style="fill:{$COL_WHITE}; stroke:none;"/>		
			  
		<text class="bciptype" 
			  x="{ceiling($procW_ div 2)}"
			  y="{$MOD_LANE_H + 8}">
				<xsl:value-of select="$modType"/>
		</text>
				
		<text class="bciplabel" 
			  x="{ceiling($procW_ div 2)}"
			  y="{$MOD_LANE_H + 16}">
				<xsl:value-of select="$procInst"/>
	   </text>
	   
	   
	  	<xsl:if test="/EDKSYSTEM/MODULES/MODULE[@INSTANCE=$procInst]/@GROUP">
	  	
		<rect x="{ceiling($periMOD_W div 2) - ceiling($MOD_LABEL_W div 2)}"
		      y="{$MOD_LANE_H + $BIF_H + ceiling($BIF_H div 3) - 2}"
			  rx="3" 
			  ry="3" 
		      width= "{$MOD_LABEL_W}"
		      height="{$BIF_H}"
			  style="fill:{$COL_IORING_LT}; stroke:none;"/>		
			  
	
	   	   <text class="ioplblgrp"  x="{ceiling($periMOD_W div 2)}" y="{$MOD_LANE_H + $BIF_H + ceiling($BIF_H div 3) + 12}">
			   <xsl:value-of select="/EDKSYSTEM/MODULES/MODULE[@INSTANCE=$procInst]/@GROUP"/>
	   		</text>
	   
	  	</xsl:if> 
	   
	   
		<xsl:for-each select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $procInst)]/BUSINTERFACE[(@BIF_X and @BIF_Y)]">
			
			<xsl:variable name="bif_busstd_">
				<xsl:choose>
					<xsl:when test="@BUSSTD">
						<xsl:value-of select="@BUSSTD"/>	
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="'TRS'"/>	
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			
			<xsl:variable name="bif_buscol_">
				<xsl:call-template name="BusType2Color">
					<xsl:with-param name="busType" select="$bif_busstd_"/>
				</xsl:call-template>
			</xsl:variable>
		
			
			<xsl:variable name="bif_name_">
				<xsl:choose>
					<xsl:when test="string-length(@NAME) &lt;= 5">
						<xsl:value-of select="@NAME"/>	
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="substring(@NAME,0,5)"/>	
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			
			<xsl:variable name="bif_x_"  select="(($BIF_W * @BIF_X) + ($BIF_GAP * @BIF_X) + ($MOD_LANE_W * 1))"/>
			<xsl:variable name="bif_y_"  select="((($BIF_H + $BIF_GAP) * @BIF_Y) + ($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP))"/>
			
			<xsl:variable name="horz_line_y_" select="($bif_y_ + ceiling($BIFC_H div 2))"/>
			
			<xsl:variable name="horz_line_x1_">
				<xsl:choose>
					<xsl:when test="@BIF_X = '0'">0</xsl:when>
					<xsl:otherwise><xsl:value-of select="($periMOD_W - $MOD_LANE_W)"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			
			<xsl:variable name="horz_line_x2_">
				<xsl:choose>
					<xsl:when test="@BIF_X = '0'"><xsl:value-of select="$MOD_LANE_W"/></xsl:when>
					<xsl:otherwise><xsl:value-of select="$periMOD_W + 1"/></xsl:otherwise>
				</xsl:choose>
			</xsl:variable>
			
			
			<line x1="{$horz_line_x1_}" 
			  	  y1="{$horz_line_y_ - 2}"
			      x2="{$horz_line_x2_}" 
			      y2="{$horz_line_y_ - 2}" 
			      style="stroke:{$bif_buscol_};stroke-width:1"/>
			  
			<use  x="{$bif_x_}"   y="{$bif_y_}"  xlink:href="#{$bif_busstd_}_Bif"/>
				
			<text class="biflabel" 
				  x="{$bif_x_ + ceiling($BIF_W div 2)}"
				  y="{$bif_y_ + ceiling($BIF_H div 2) + 3}">
					<xsl:value-of select="$bif_name_"/>
			</text>
			
			
			
		</xsl:for-each>
		
		<xsl:variable name="interrupt_cntlr_">
			<xsl:choose>
				<xsl:when test="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE=$procInst)]/@INTERRUPT_CNTLR">
					<xsl:value-of select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE=$procInst)]/@INTERRUPT_CNTLR"/>
				</xsl:when>
				<xsl:otherwise>"_no_interrupt_cntlr_"</xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
			
<!--		
		<xsl:message> The intc index should <xsl:value-of select="$interrupt_cntlr_"/></xsl:message>
		<xsl:message> The intc index is <xsl:value-of select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $interrupt_cntlr_)]/@INTCINDEX"/></xsl:message>
-->		
		<xsl:if test="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $interrupt_cntlr_)]/@INTCINDEX">
			
			<xsl:variable name="intr_col_">
				<xsl:call-template name="intcIdx2Color">
					<xsl:with-param name="intcIdx" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $interrupt_cntlr_)]/@INTCINDEX"/>
				</xsl:call-template>	
			</xsl:variable>
			
			<xsl:call-template name="_draw_InterruptedProc">
				<xsl:with-param name="intr_col" select="$intr_col_"/>
				<xsl:with-param name="intr_x"   select="($periMOD_W - ceiling($INTR_W div 2))"/>
				<xsl:with-param name="intr_y"   select="3"/>
				<xsl:with-param name="intr_idx" select="/EDKSYSTEM/MODULES/MODULE[(@INSTANCE = $interrupt_cntlr_)]/@INTCINDEX"/>
			</xsl:call-template>	
		</xsl:if>
		
	</symbol>			  
</xsl:template>
<!-- ======================= END DEF BLOCK ============================ -->

<!-- ======================= UTILITY FUNCTIONS ============================ -->
<xsl:template name="_calc_Proc_Height">
	<xsl:param name="procInst"  select="_processor_"/>
	
	<xsl:variable name="tot_bifs_h_">
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInst)]/@BIFS_H)">0</xsl:if>
		
		<xsl:if test="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInst)]/@BIFS_H">
			<xsl:variable name="bifs_h_" select="(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@INSTANCE = $procInst)]/@BIFS_H)"/>
			<xsl:value-of select="(($BIF_H + $BIF_GAP) * $bifs_h_)"/>	
		</xsl:if>
	</xsl:variable>	
	
	<xsl:value-of select="(($MOD_LANE_H * 2) + $tot_bifs_h_ + ($MOD_LABEL_H + $BIF_GAP))"/>	
</xsl:template>

<xsl:template name="_calc_Max_Proc_Height">

	<!-- Store the heights in a variable -->	
	<xsl:variable name="proc_heights_">
	
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE)">
			<PROC HEIGHT="0"/>
		</xsl:if>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE">
			<xsl:variable name="procInst_" select="@INSTANCE"/> 
			<xsl:variable name="proc_height_">
				<xsl:call-template name="_calc_Proc_Height">	
					<xsl:with-param name="procInst" select="$procInst_"/>
				</xsl:call-template>	
			</xsl:variable>
			
<!--			
			<xsl:message>Found Proc height as <xsl:value-of select="$proc_height_"/></xsl:message>
-->			
			<PROC HEIGHT="{$proc_height_}"/>
		</xsl:for-each>
	</xsl:variable>
	
	<!-- Return the max of them -->	
<!--	
	<xsl:message>Found Proc ax as <xsl:value-of select="math:max(exsl:node-set($proc_heights_)/PROC/@HEIGHT)"/></xsl:message>
-->	

	<xsl:value-of select="math:max(exsl:node-set($proc_heights_)/PROC/@HEIGHT)"/>
</xsl:template>


<xsl:template name="_calc_Proc_MemoryUnits_Height">
	<xsl:param name="procInst"  select="_processor_"/>
	
	<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@PROCESSOR = $procInst) and (@MODCLASS = 'MEMORY_UNIT'))])">0</xsl:if>
	
	<xsl:if test="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@PROCESSOR = $procInst) and (@MODCLASS='MEMORY_UNIT'))]">
		
	<xsl:variable name="peri_gap_">
		<xsl:choose>
			<xsl:when test="not(@CSTACK_INDEX)">
				<xsl:value-of select="$BIF_H"/>
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>	
	</xsl:variable>	
			
		
		<!-- Store the all memory unit heights in a variable -->
		<xsl:variable name="memU_heights_">
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@PROCESSOR = $procInst) and (@MODCLASS='MEMORY_UNIT'))]">
<!--				
				<xsl:variable name="unitId_" select="@PSTACK_MODS_Y"/>
-->				
				<xsl:variable name="unitHeight_">
					<xsl:call-template name="_calc_MemoryUnit_Height">	
						<xsl:with-param name="shapeId" select="@SHAPE_ID"/>
					</xsl:call-template>	
				</xsl:variable>
				
				<MEM_UNIT HEIGHT="{$unitHeight_ + $peri_gap_}"/>
			</xsl:for-each>
		</xsl:variable>
		
		<xsl:value-of select="sum(exsl:node-set($memU_heights_)/MEM_UNIT/@HEIGHT)"/>
	</xsl:if>
</xsl:template>
	

<xsl:template name="_calc_Proc_Peripherals_Height">
	<xsl:param name="procInst"  select="_processor_"/>
	
	<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@PROCESSOR = $procInst) and not(@MODCLASS = 'MEMORY_UNIT'))])">0</xsl:if>
	
	<xsl:if test="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@PROCESSOR = $procInst) and not(@MODCLASS='MEMORY_UNIT'))]">
	
		<xsl:variable name="peri_gap_">
			<xsl:if test="@CSTACK_INDEX">
				<xsl:value-of select="$BIF_H"/>
			</xsl:if>
			<xsl:if test="not(@IS_CSTACK)">0</xsl:if>
		</xsl:variable>
	
		<!-- Store the all peripheral heights in a variable -->
		<xsl:variable name="peri_heights_">
			
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@PROCESSOR = $procInst) and not(@MODCLASS='MEMORY_UNIT'))]">
				<xsl:for-each select="MODULE">
<!--					
					<xsl:message><xsl:value-of select="@INSTANCE"/></xsl:message>		
-->					
					<xsl:variable name="peri_height_">
						<xsl:call-template name="_calc_PeriShape_Height">	
							<xsl:with-param name="shapeInst" select="@INSTANCE"/>
						</xsl:call-template>	
					</xsl:variable>
					<PERI HEIGHT="{$peri_height_ + $peri_gap_}"/>
				</xsl:for-each>		
			</xsl:for-each>
		</xsl:variable>
		
		<xsl:value-of select="sum(exsl:node-set($peri_heights_)/PERI/@HEIGHT)"/>
	</xsl:if>
</xsl:template>
	
	
<xsl:template name="_calc_Space_AbvSbs_Height">
	<xsl:param name="stackToEast"  select="'NONE'"/>
	<xsl:param name="stackToWest"  select="'NONE'"/>
	
	
	<xsl:variable name = "stackAbvSbs_West_H_">
		<xsl:choose>
			<xsl:when test="(($stackToEast = '0')   and     ($stackToWest = 'NONE'))">0</xsl:when>
			<xsl:when test="(($stackToEast = 'NONE') and not($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_AbvSbs_Height">
					<xsl:with-param name="stackIdx"  select="$stackToWest"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="(not($stackToEast = '0') and ($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_AbvSbs_Height">
					<xsl:with-param name="stackIdx"  select="($stackToEast - 1)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
	<xsl:variable name = "stackAbvSbs_East_H_">
		<xsl:call-template name="_calc_Stack_AbvSbs_Height">
			<xsl:with-param name="stackIdx"  select="$stackToEast"/>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name="stackAbvSbs_heights_">
		<STACK HEIGHT="{$stackAbvSbs_East_H_}"/>
		<STACK HEIGHT="{$stackAbvSbs_West_H_}"/>
	</xsl:variable>
	
	<xsl:value-of select="math:max(exsl:node-set($stackAbvSbs_heights_)/STACK/@HEIGHT)"/>
</xsl:template>

	
<xsl:template name="_calc_Space_BlwSbs_Height">
	<xsl:param name="stackToEast"  select="'NONE'"/>
	<xsl:param name="stackToWest"  select="'NONE'"/>
		
	<xsl:variable name = "stackBlwSbs_West_H_">
		<xsl:choose>
			<xsl:when test="(($stackToEast = '0')    and    ($stackToWest = 'NONE'))">0</xsl:when>
			<xsl:when test="(($stackToEast = 'NONE') and not($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_BlwSbs_Height">
					<xsl:with-param name="stackIdx"  select="$stackToWest"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="(not($stackToEast = '0') and    ($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_BlwSbs_Height">
					<xsl:with-param name="stackIdx"  select="($stackToEast - 1)"/>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
	</xsl:variable>
	
	
	<xsl:variable name = "stackBlwSbs_East_H_">
		<xsl:call-template name="_calc_Stack_BlwSbs_Height">
			<xsl:with-param name="stackIdx"  select="$stackToEast"/>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:variable name="stackBlwSbs_heights_">
		<STACK HEIGHT="{$stackBlwSbs_East_H_}"/>
		<STACK HEIGHT="{$stackBlwSbs_West_H_}"/>
	</xsl:variable>
	
	<xsl:value-of select="math:max(exsl:node-set($stackBlwSbs_heights_)/STACK/@HEIGHT)"/>
</xsl:template>
	

	
<xsl:template name="_calc_Stack_AbvSbs_Height">
	<xsl:param name="stackIdx"  select="100"/>
<!--	
	<xsl:message>^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^</xsl:message>
-->	
	
	<xsl:if test="(not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@IS_ABVSBS))]) and
				   not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@STACK_HORIZ_INDEX = $stackIdx)]))"><xsl:value-of select="$PROC2SBS_GAP"/></xsl:if>
	
	<xsl:if test="((/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@IS_ABVSBS))]) or
				   (/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@STACK_HORIZ_INDEX = $stackIdx)]))">
		
<!--			
		<xsl:variable name="peri_gap_">
			<xsl:value-of select="$BIF_H"/>
			<xsl:choose>
				<xsl:when test="(@SHAPE_VERTI_INDEX)">
				</xsl:when>
				<xsl:otherwise>0</xsl:otherwise>
			</xsl:choose>	
		</xsl:variable>	
-->			
			
<!--		
		<xsl:message>The gap is <xsl:value-of select="$peri_gap_"/></xsl:message>
		<xsl:message>The gap is <xsl:value-of select="$peri_gap_"/></xsl:message>
		<xsl:message>================================</xsl:message>
		<xsl:message>================================</xsl:message>
		<xsl:message>This is above <xsl:value-of select="@INSTANCE"/></xsl:message>
		<xsl:message><xsl:value-of select="@INSTANCE"/> : <xsl:value-of select="$peri_height_"/></xsl:message>
-->	
	
	
		<!-- Store the all peripheral heights in a variable -->
		<xsl:variable name="peri_heights_">
			
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and not(@MODCLASS = 'MEMORY_UNIT') and (@IS_ABVSBS))]">
				<xsl:for-each select="MODULE">
<!--					
					<xsl:message>This is above <xsl:value-of select="@INSTANCE"/></xsl:message>
-->					
					
					<xsl:variable name="peri_height_">
<!--						
						<xsl:call-template name="_calc_Shape_Height">	
							<xsl:with-param name="shapeId" select="@SHAPE_ID"/>
						</xsl:call-template>	
-->	 
	
						<xsl:call-template name="_calc_PeriShape_Height">	
							<xsl:with-param name="shapeInst" select="@INSTANCE"/>
						</xsl:call-template>	
					</xsl:variable>
					
					<PERI HEIGHT="{$peri_height_ + $BIF_H}"/>
				</xsl:for-each>
			</xsl:for-each>
			
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@MODCLASS = 'MEMORY_UNIT') and (@IS_ABVSBS))]">
			
				<xsl:variable name="memu_height_">
					<xsl:call-template name="_calc_MemoryUnit_Height">	
						<xsl:with-param name="shapeId" select="@SHAPE_ID"/>
					</xsl:call-template>	
				</xsl:variable>
			
<!--				
				<xsl:message>Mem_Unit : <xsl:value-of select="@SHAPE_ID"/> : <xsl:value-of select="$memu_height_ + $peri_gap_"/></xsl:message>
-->				
				<PERI HEIGHT="{$memu_height_ + $BIF_H}"/>
			
			</xsl:for-each>
			
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[((@STACK_HORIZ_INDEX = $stackIdx) and (@IS_ABVSBS))]">
					
				<xsl:variable name="proc_height_">
					<xsl:call-template name="_calc_PeriShape_Height">	
						<xsl:with-param name="shapeInst" select="@INSTANCE"/>
					</xsl:call-template>	
				</xsl:variable>
				
<!--				
		<xsl:message>===================================</xsl:message>
		<xsl:message>Processor : <xsl:value-of select="@INSTANCE"/> : <xsl:value-of select="$peri_height_ + $peri_gap_"/></xsl:message>
				<PERI HEIGHT="{$proc_height_ + $PROC2SBS_GAP }"/>
-->					
				<PERI HEIGHT="{$proc_height_ + $BIF_H}"/>
				
			</xsl:for-each>
		
		</xsl:variable>
		
<!--		
	<xsl:message><xsl:value-of select="@INSTANCE"/> : <xsl:value-of select="$peri_height_ + $peri_gap_"/></xsl:message>
	<xsl:message>================================</xsl:message>
-->
		
<!--		
	<xsl:message>^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^</xsl:message>
-->		
		<xsl:value-of select="sum(exsl:node-set($peri_heights_)/PERI/@HEIGHT)"/>
	</xsl:if>
	
</xsl:template>
	
<xsl:template name="_calc_Stack_BlwSbs_Height">
	<xsl:param name="stackIdx"  select="100"/>
	
<!--	
-->	
			
		<!-- Store the all peripheral heights in a variable -->
		<xsl:variable name="stack_heights_">
			
			<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@IS_BLWSBS))])">
				<STACKSHAPE HEIGHT="0"/>
			</xsl:if>
			
			<xsl:if test="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@IS_BLWSBS))]">
<!--		
		<xsl:message>vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv</xsl:message>
-->	
	
				<xsl:variable name="peri_gap_">
					<xsl:choose>
						<xsl:when test="(@SHAPE_VERTI_INDEX)">
							<xsl:value-of select="$BIF_H"/>
						</xsl:when>
						<xsl:otherwise>0</xsl:otherwise>
					</xsl:choose>	
				</xsl:variable>	
				
				<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and not(@MODCLASS = 'MEMORY_UNIT') and (@IS_BLWSBS))]">
					<xsl:for-each select="MODULE">
<!--					
					<xsl:message>This is below <xsl:value-of select="@INSTANCE"/></xsl:message>
-->	
						<xsl:variable name="peri_height_">
							<xsl:call-template name="_calc_PeriShape_Height">	
								<xsl:with-param name="shapeInst" select="@INSTANCE"/>
							</xsl:call-template>	
						</xsl:variable>
						
						<STACKSHAPE HEIGHT="{$peri_height_ + $peri_gap_}"/>
					</xsl:for-each>
				</xsl:for-each>
		
				<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $stackIdx) and (@MODCLASS = 'MEMORY_UNIT') and (@IS_BLWSBS))]">
			
					<xsl:variable name="memu_height_">
						<xsl:call-template name="_calc_MemoryUnit_Height">	
							<xsl:with-param name="shapeId" select="@SHAPE_ID"/>
						</xsl:call-template>	
					</xsl:variable>
			
					<STACKSHAPE HEIGHT="{$memu_height_ + $peri_gap_}"/>
				
<!--				
				<xsl:message>Mem_Unit : <xsl:value-of select="@SHAPE_ID"/> : <xsl:value-of select="$memu_height_ + $peri_gap_"/></xsl:message>
-->	
			
			</xsl:for-each>
		</xsl:if>
			
		<xsl:variable name="sbsBuckets_H_">
			<xsl:call-template name="_calc_Stack_SbsBuckets_Height">
				<xsl:with-param name="stackIdx" select="$stackIdx"/>
			</xsl:call-template>	
		</xsl:variable>
			
			<STACKSHAPE HEIGHT="{$sbsBuckets_H_}"/>
<!--			
			<xsl:message>Sbs Bucket H : <xsl:value-of select="$sbsBuckets_H_"/></xsl:message>
-->
		</xsl:variable>
		
<!--		
		<xsl:message>vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv</xsl:message>
-->		
		<xsl:value-of select="sum(exsl:node-set($stack_heights_)/STACKSHAPE/@HEIGHT)"/>
	
</xsl:template>
	

<xsl:template name="_calc_Stack_SbsBuckets_Height">
	<xsl:param name="stackIdx"  select="1000"/>
	
	<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX = $stackIdx)])">0</xsl:if>
	
	<xsl:if test="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX = $stackIdx)]">
	
		<!-- Store the all buckets heights in a variable -->
		<xsl:variable name="bkt_heights_">
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX = $stackIdx)]">
		
				<xsl:variable name="bkt_height_">
					<xsl:call-template name="_calc_SbsBucket_Height">	
						<xsl:with-param name="bucketId" select="@BUSINDEX"/>
					</xsl:call-template>	
				</xsl:variable>
<!--				
				<xsl:message>Found shared buckets height as <xsl:value-of select="$bkt_height_"/></xsl:message>
-->				
				<BKT HEIGHT="{$bkt_height_ + $BIF_H}"/>
			</xsl:for-each>
		</xsl:variable>
		
		<xsl:value-of select="sum(exsl:node-set($bkt_heights_)/BKT/@HEIGHT)"/>
	</xsl:if>
</xsl:template>


<xsl:template name="_calc_Max_Stack_BlwSbs_Height">

	<!-- Store the heights in a variable -->	
	<xsl:variable name="blwSbs_heights_">
		
<!--			
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE)">
			<BLW HEIGHT="0"/>
		</xsl:if>
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE">
			<xsl:variable name="procInst_" select="@INSTANCE"/> 
			<xsl:variable name="blwSbs_">
				<xsl:call-template name="_calc_Stack_PerisBlwSbs_Height">	
					<xsl:with-param name="procInst" select="$procInst_"/>
				</xsl:call-template>	
			</xsl:variable>
			<xsl:message>Found Blw Sbs height as <xsl:value-of select="$blwSbs_"/></xsl:message>
			<BLW HEIGHT="{$blwSbs_}"/>
		</xsl:for-each>
-->			
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(@EAST &lt; /EDKSYSTEM/BLKDSHAPES/@STACK_HORIZ_WIDTH)]">
	
<!--			
			<xsl:message>Found a space of index <xsl:value-of select="@EAST"/></xsl:message>
-->	
			
			<xsl:variable name="stack_height_">
				<xsl:call-template name="_calc_Stack_BlwSbs_Height">
					<xsl:with-param name="stackIdx"  select="@EAST"/>
				</xsl:call-template>
			</xsl:variable>
			
			
			<BLW HEIGHT="{$stack_height_}"/>
			
		</xsl:for-each>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(@WEST = (/EDKSYSTEM/BLKDSHAPES/@STACK_HORIZ_WIDTH -1))]">
			
<!--			
			<xsl:message>Last stack of index <xsl:value-of select="@WEST"/></xsl:message>
-->			
			
			<xsl:variable name="stack_height_">
				<xsl:call-template name="_calc_Stack_BlwSbs_Height">
					<xsl:with-param name="stackIdx"  select="@WEST"/>
				</xsl:call-template>
			</xsl:variable>
			
			
			<BLW HEIGHT="{$stack_height_}"/>
			
		</xsl:for-each>
		
		
	</xsl:variable>
	
<!--	
	<xsl:message>Found Blw Sbs max as <xsl:value-of select="math:max(exsl:node-set($blwSbs_heights_)/BLW/@HEIGHT)"/></xsl:message>
-->	
	<!-- Return the max of them -->	
	<xsl:value-of select="math:max(exsl:node-set($blwSbs_heights_)/BLW/@HEIGHT)"/>
</xsl:template>
	
	
<xsl:template name="_calc_Max_Stack_AbvSbs_Height">

	<!-- Store the heights in a variable -->	
	<xsl:variable name="abvSbs_heights_">
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(@EAST &lt; /EDKSYSTEM/BLKDSHAPES/@STACK_HORIZ_WIDTH)]">
			
<!--			
			<xsl:message>Found a space of index <xsl:value-of select="@EAST"/></xsl:message>
-->	
			
			<xsl:variable name="stack_height_">
				<xsl:call-template name="_calc_Stack_AbvSbs_Height">
					<xsl:with-param name="stackIdx"  select="@EAST"/>
				</xsl:call-template>
			</xsl:variable>
			
<!--			
			<xsl:message>Found stack of width <xsl:value-of select="$stack_width_"/></xsl:message>
			<xsl:message>==============================</xsl:message>
-->			
			
			<ABV HEIGHT="{$stack_height_}"/>
			
		</xsl:for-each>
		
		
	</xsl:variable>
	
<!--	
	<xsl:message>Found Blw Sbs max as <xsl:value-of select="math:max(exsl:node-set($blwSbs_heights_)/BLW/@HEIGHT)"/></xsl:message>
-->	
	<!-- Return the max of them -->	
	<xsl:value-of select="math:max(exsl:node-set($abvSbs_heights_)/ABV/@HEIGHT)"/>
</xsl:template>
	
	
<xsl:template name="_calc_Max_Proc_PerisAbvSbs_Height">
	<xsl:param name="procInst"  select="_processor_"/>
	
	<!-- Store the heights in a variable -->	
	<xsl:variable name="abvSbs_heights_">
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE)">
			<ABV HEIGHT="0"/>
		</xsl:if>
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE">
			<xsl:variable name="procInst_" select="@INSTANCE"/> 
<!--			
			<xsl:message>Found Blw Sbs height as <xsl:value-of select="$blwSbs_"/></xsl:message>
			<ABV HEIGHT="{$pAbvSbs_}"/>
-->			
			
			<xsl:variable name="pAbvSbs_">
				<xsl:call-template name="_calc_Proc_PerisAbvSbs_Height">	
					<xsl:with-param name="procInst" select="$procInst_"/>
				</xsl:call-template>	
			</xsl:variable>
			
			<xsl:variable name="memUs_">
				<xsl:call-template name="_calc_Proc_MemoryUnits_Height">	
					<xsl:with-param name="procInst" select="$procInst_"/>
				</xsl:call-template>	
			</xsl:variable>
			
<!--			
			<xsl:message>Found Peris Above height as <xsl:value-of select="$pAbvSbs_"/></xsl:message>
			<xsl:message>Found MemUs Above height as <xsl:value-of select="$memUs_"/></xsl:message>
			<xsl:message>Found PAbv Above height as <xsl:value-of select="($pAbvSbs_ + $memUs_)"/></xsl:message>
-->			
			<ABV HEIGHT="{$pAbvSbs_ + $memUs_}"/>
		</xsl:for-each>
	</xsl:variable>
	
<!--	
	<xsl:message>Found Abv Sbs max as <xsl:value-of select="math:max(exsl:node-set($abvSbs_heights_)/ABV/@HEIGHT)"/></xsl:message>
-->	
	
	<!-- Return the max of them -->	
	<xsl:value-of select="math:max(exsl:node-set($abvSbs_heights_)/ABV/@HEIGHT)"/>
</xsl:template>
	
<xsl:template name="_calc_Max_Proc_SbsBuckets_Height___">

	<!-- Store the heights in a variable -->	
	<xsl:variable name="blwSbs_heights_">
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE)">
			<BLW HEIGHT="0"/>
		</xsl:if>
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE">
			<xsl:variable name="procInst_" select="@INSTANCE"/> 
			<xsl:variable name="sbsBktH_">
				<xsl:call-template name="_calc_Proc_SbsBuckets_Height">	
					<xsl:with-param name="procInst" select="$procInst_"/>
				</xsl:call-template>	
			</xsl:variable>
<!--			
			<xsl:message>Found Blw Sbs height as <xsl:value-of select="$blwSbs_"/></xsl:message>
-->			
			<BKT HEIGHT="{$sbsBktH_}"/>
		</xsl:for-each>
	</xsl:variable>
	
<!--	
	<xsl:message>Found Blw Sbs max as <xsl:value-of select="math:max(exsl:node-set($blwSbs_heights_)/BLW/@HEIGHT)"/></xsl:message>
-->	
	<!-- Return the max of them -->	
	<xsl:value-of select="math:max(exsl:node-set($blwSbs_heights_)/BKT/@HEIGHT)"/>
</xsl:template>
	
<xsl:template name="_calc_Max_SbsBuckets_Height___">
	
	
	<xsl:variable name="bkts_h_">
		
		<xsl:variable name="h_proc_sbsBkts_">
			<xsl:call-template name="_calc_Max_Proc_SbsBuckets_Height"/>	
		</xsl:variable>
		<BKT HEIGHT="{$h_proc_sbsBkts_}"/>
		
	</xsl:variable>	
	
<!--	
	<xsl:variable name="h_perisAbvSbs_">
		<xsl:call-template name="_calc_Proc_PerisAbvSbs_Height">	
			<xsl:with-param name="procInst" select="$procInst"/>
		</xsl:call-template>	
	</xsl:variable>
-->	
	<xsl:value-of select="math:max(exsl:node-set($bkts_h_)/BKT/@HEIGHT)"/>
</xsl:template>
	
	

<xsl:template name="_calc_MultiProc_Stack_Height">
	<xsl:param name="mpstack_blkd_x"  select="100"/>
	
		<xsl:variable name="mpStk_ShpHeights_">
			<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@HAS_MULTIPROCCONNS) and (@PSTACK_BLKD_X = $mpstack_blkd_x))])">
				<MPSHAPE HEIGHT="0"/>
			</xsl:if>
			
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@HAS_MULTIPROCCONNS) and (@PSTACK_BLKD_X = $mpstack_blkd_x))]">
				<xsl:variable name="shpClass_" select="@MODCLASS"/> 
				<xsl:variable name="shpHeight_">
					<xsl:choose>
						<xsl:when test="$shpClass_ = 'PERIPHERAL'">
<!--							
							<xsl:message>Found Multi Proc Peripheral</xsl:message> 
-->	
							<xsl:call-template name="_calc_PeriShape_Height">	
								<xsl:with-param name="shapeInst" select="MODULE/@INSTANCE"/>
							</xsl:call-template>	
						</xsl:when>
						<xsl:when test="$shpClass_ = 'MEMORY_UNIT'">
<!--							
							<xsl:message>Found Multi Proc Memory Unit</xsl:message> 
-->	
							<xsl:call-template name="_calc_MemoryUnit_Height">	
								<xsl:with-param name="cshpIndex"  select="@CSHAPE_INDEX"/>
							</xsl:call-template>	
						</xsl:when>
						<xsl:otherwise>0</xsl:otherwise>
					</xsl:choose>
				</xsl:variable>
				
<!--				
				<xsl:message>Found <xsl:value-of select="$shpHeight_"/></xsl:message>
-->				
				
				<MPSHAPE HEIGHT="{$shpHeight_}"/>
			</xsl:for-each>
	</xsl:variable>
	
<!--	
	<xsl:message>Found stack of height <xsl:value-of select="sum(exsl:node-set($mpStk_ShpHeights_)/MPSHAPE/@HEIGHT)"/></xsl:message>
-->	
	
	<xsl:value-of select="sum(exsl:node-set($mpStk_ShpHeights_)/MPSHAPE/@HEIGHT)"/>
</xsl:template>

<xsl:template name="_calc_Max_MultiProc_Stack_Height">
	
	<!-- Store the heights in a variable -->	
	
	<xsl:variable name="mpStks_Heights_">
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE)">
			<MPSTK HEIGHT="0"/>
		</xsl:if>
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@PSTACK_BLKD_X)]">
			<xsl:variable name="mpstack_height_">
				<xsl:call-template name="_calc_MultiProc_Stack_Height">
					<xsl:with-param name="mpstack_blkd_x" select="(@PSTACK_BLKD_X + 1)"/>
				</xsl:call-template>
			</xsl:variable>
			
<!--			
			<xsl:message>Found <xsl:value-of select="$mpstack_height_"/></xsl:message>
-->			
			<MPSTK HEIGHT="{$mpstack_height_}"/>
		</xsl:for-each>
		
	</xsl:variable>

		<!-- Return the max of them -->	
	<xsl:value-of select="math:max(exsl:node-set($mpStks_Heights_)/MPSTK/@HEIGHT)"/>
	
</xsl:template>



<xsl:template name="_calc_Stack_Shape_Y">
	
	<xsl:param name="horizIdx"  select="100"/>
	<xsl:param name="vertiIdx"  select="100"/>
	
<!--	
	<xsl:message>Y at H index <xsl:value-of select="$horizIdx"/></xsl:message>
	<xsl:message>Y at V index <xsl:value-of select="$vertiIdx"/></xsl:message>
	<xsl:param name="sbsGap"    select="0"/>
	<xsl:variable name="numSBSs_"     select="count(/EDKSYSTEM/BLKDSHAPES/SBSSHAPES/MODULE)"/>	
	<xsl:variable name="sbs_H_"    select="($numSBSs_ * $SBS_H)"/>
	<xsl:variable name="sbsGap_"   select="($PROC2SBS_GAP + $sbs_H_)"/>
-->	
	
	<xsl:variable name="sbsGap_" select="((count(/EDKSYSTEM/BLKDSHAPES/SBSSHAPES/MODULE) * $SBS_H) + $PROC2SBS_GAP)"/>	
	
	<xsl:if test="(not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))]) and  
		           not(/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(  (@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))])   and
		           not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(     (@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))]))">0</xsl:if>
	
	
	<xsl:if test="((/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))])   or  
		           (/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(  (@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))])   or
		           (/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(     (@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))]))">
		<!-- Store the spaces above this one in a variable -->
		<xsl:variable name="spaces_above_">
		
			<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX &lt; $vertiIdx))])">
				<SPACE HEIGHT="0"/>
			</xsl:if>
			
			<!-- Store the height of all peripherals and memory units above this one-->
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $horizIdx)  and (@SHAPE_VERTI_INDEX &lt; $vertiIdx))]">
				
				<xsl:if test="not(@MODCLASS='MEMORY_UNIT')">	
					<xsl:variable name="peri_height_">
						<xsl:call-template name="_calc_Shape_Height">	
							<xsl:with-param name="shapeId" select="@SHAPE_ID"/>
						</xsl:call-template>	
					</xsl:variable>
<!--					
					<xsl:message>Found peri height <xsl:value-of select="$peri_height_"/></xsl:message>
-->	
	
					<SPACE HEIGHT="{$peri_height_ + $BIF_H}"/>
				</xsl:if>
				
				<xsl:if test="(@MODCLASS='MEMORY_UNIT')">	
					<xsl:variable name="memu_height_">
						<xsl:call-template name="_calc_MemoryUnit_Height">	
							<xsl:with-param name="shapeId" select="@SHAPE_ID"/>
						</xsl:call-template>	
					</xsl:variable>
<!--					
					<xsl:message>Found unit height <xsl:value-of select="$memu_height_"/></xsl:message>
-->					
					<SPACE HEIGHT="{$memu_height_ + $BIF_H}"/>
				</xsl:if>
				
			</xsl:for-each>
			
			<!-- Store the height of all the processors above this one-->
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[((@STACK_HORIZ_INDEX = $horizIdx)  and (@SHAPE_VERTI_INDEX &lt; $vertiIdx))]">
				<xsl:variable name="proc_height_">
						<xsl:call-template name="_calc_PeriShape_Height">	
							<xsl:with-param name="shapeInst" select="@INSTANCE"/>
						</xsl:call-template>	
				</xsl:variable>
				
				<SPACE HEIGHT="{$proc_height_ + $BIF_H}"/>
			</xsl:for-each>
			
			
			<!-- If its a peripheral that is below the shared busses, or its a shared bus bucket -->
			<!-- add the height of the shared busses and the processor.                           -->
			<xsl:if  test="(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[((@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))]/@IS_BLWSBS)">
				<SPACE HEIGHT="{$sbsGap_}"/>
			</xsl:if>
			<xsl:if test="(/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[((@STACK_HORIZ_INDEX = $horizIdx) and (@SHAPE_VERTI_INDEX = $vertiIdx))])">
				<SPACE HEIGHT="{$sbsGap_}"/>
			</xsl:if>
			
			<!-- Store the height of all shared bus buckets above this one-->
			<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[((@STACK_HORIZ_INDEX = $horizIdx)  and (@SHAPE_VERTI_INDEX &lt; $vertiIdx))]">
				<xsl:variable name="bkt_height_">
					<xsl:call-template name="_calc_SbsBucket_Height">
						<xsl:with-param name="bucketId" select="@BUSINDEX"/>
					</xsl:call-template>	
				</xsl:variable>
				
				<SPACE HEIGHT="{$bkt_height_ + $BIF_H}"/>
			</xsl:for-each>
			
		</xsl:variable>
		
		<xsl:value-of select="sum(exsl:node-set($spaces_above_)/SPACE/@HEIGHT)"/>
	</xsl:if>
	
</xsl:template>
	
	
<xsl:template name="_calc_Max_BusConnLane_BifY">
	
	<xsl:param name="busname" select="'_busname_'"/>
	
	<!-- Store the heights in a variable -->	
	<xsl:variable name="busConnYs_">
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE/BUSCONNS/BUSCONNLANE/BUSCONN)">
			<BUSCONNY HEIGHT="0"/>
		</xsl:if>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE/BUSCONNS/BUSCONNLANE[(@BUSNAME = $busname)]/BUSCONN">
			
			<xsl:variable name="peri_cstk_y_">
				<xsl:call-template name="_calc_CStackShapesAbv_Height">
					<xsl:with-param name="cstackIndex"  select="../@CSTACK_INDEX"/>
					<xsl:with-param name="cstackModY"   select="@CSTACK_MODS_Y"/>
				</xsl:call-template>	
			</xsl:variable>	
				
				<xsl:variable name="peri_bif_dy_">
					<xsl:value-of select="(($BIF_H + $BIF_GAP)  * @BIF_Y)"/>
				</xsl:variable>
				
				<xsl:variable name="peri_bc_y_">
					<xsl:value-of select="($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP + $peri_bif_dy_ + ceiling($BIF_H div 2)) - ceiling($BIFC_H div 2)"/>
				</xsl:variable>
			
<!--			
			<xsl:message>Found a busconn lane</xsl:message>
-->			
			<BUSCONNY HEIGHT="{$peri_cstk_y_ + $peri_bif_dy_ + $peri_bc_y_}"/>
		</xsl:for-each>
		
	</xsl:variable>

		<!-- Return the max of them -->	
	<xsl:value-of select="math:max(exsl:node-set($busConnYs_)/BUSCONNY/@HEIGHT)"/>
	
</xsl:template>
	
	
<xsl:template name="_calc_Min_BusConnLane_BifY">
	
	<xsl:param name="busname" select="'_busname_'"/>
	
	<!-- Store the heights in a variable -->	
	<xsl:variable name="busConnYs_">
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE/BUSCONNS/BUSCONNLANE/BUSCONN)">
			<BUSCONNY HEIGHT="0"/>
		</xsl:if>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE/BUSCONNS/BUSCONNLANE[(@BUSNAME = $busname)]/BUSCONN">
			
			<xsl:variable name="peri_cstk_y_">
				<xsl:call-template name="_calc_CStackShapesAbv_Height">
					<xsl:with-param name="cstackIndex"  select="../@CSTACK_INDEX"/>
					<xsl:with-param name="cstackModY"   select="@CSTACK_MODS_Y"/>
				</xsl:call-template>	
			</xsl:variable>	
				
				<xsl:variable name="peri_bif_dy_">
					<xsl:value-of select="(($BIF_H + $BIF_GAP)  * @BIF_Y)"/>
				</xsl:variable>
				
				<xsl:variable name="peri_bc_y_">
					<xsl:value-of select="($MOD_LANE_H + $MOD_LABEL_H + $BIF_GAP + $peri_bif_dy_ + ceiling($BIF_H div 2)) - ceiling($BIFC_H div 2)"/>
				</xsl:variable>
			
<!--			
			<xsl:message>Found a busconn lane</xsl:message>
-->			
			<BUSCONNY HEIGHT="{$peri_cstk_y_ + $peri_bc_y_}"/>
		</xsl:for-each>
		
	</xsl:variable>

		<!-- Return the min of them -->	
	<xsl:value-of select="math:min(exsl:node-set($busConnYs_)/BUSCONNY/@HEIGHT)"/>
	
</xsl:template>
	
<xsl:template name="_calc_Stack_Height">
	<xsl:param name="stackIdx"  select="100"/>
	
	<xsl:variable name="stack_height_">
		<!-- if this is called with no vert index of a shape 
			 it defaults to the total height of the stack -->
		<xsl:call-template name="_calc_Stack_Shape_Y">
			<xsl:with-param name="horizIdx"  select="$stackIdx"/>
		</xsl:call-template>
	</xsl:variable>
	
	<xsl:value-of select="$stack_height_"/>
</xsl:template>
	
<!--	
-->	
	
	
<xsl:template name="_calc_Stack_Width">
	<xsl:param name="stackIdx"  select="100"/>
	
<!--	
	<xsl:message>=============Stack Idx <xsl:value-of select="$stackIdx"/>====</xsl:message>			
-->	
	<xsl:variable name="shape_widths_">	
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[@STACK_HORIZ_INDEX = $stackIdx])">
			<SHAPE WIDTH="0"/>
		</xsl:if>
			
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[@STACK_HORIZ_INDEX = $stackIdx])">
			<SHAPE WIDTH="0"/>
		</xsl:if>
			
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/PROCSHAPES/MODULE[(@STACK_HORIZ_INDEX = $stackIdx)]">
<!--			
			<xsl:variable name="proc_w_">
				<xsl:value-of select="$periMOD_W"/>
			</xsl:variable>
			<xsl:message>Found processor of width <xsl:value-of select="$proc_w_"/></xsl:message>
-->	
			<SHAPE WIDTH="{$periMOD_W}"/>
		</xsl:for-each>
			
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES/CMPLXSHAPE[(@STACK_HORIZ_INDEX = $stackIdx)]">
				
			<xsl:variable name="shpClass_" select="@MODCLASS"/> 
			<xsl:variable name="shape_w_">
				<xsl:choose>
						
						<xsl:when test="$shpClass_ = 'PERIPHERAL'">
							<xsl:value-of select="$periMOD_W"/>
						</xsl:when>
						
						<xsl:when test="$shpClass_ = 'MEMORY_UNIT'">
							<xsl:value-of select="($periMOD_W * @MODS_W)"/>
						</xsl:when>
						
						<xsl:otherwise>0</xsl:otherwise>
						
					</xsl:choose>
				</xsl:variable>
				
<!--		
			<xsl:message>Found shape width <xsl:value-of select="$shape_w_"/></xsl:message>
-->				
				
			<SHAPE WIDTH="{$shape_w_}"/>
		</xsl:for-each>
			
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX = $stackIdx)]">
			<xsl:variable name="bucket_w_">
				  <xsl:value-of select="(($MOD_BKTLANE_W * 2) + (($periMOD_W * @MODS_W) + ($MOD_BUCKET_G * (@MODS_W - 1))))"/>
			</xsl:variable>
			
<!--				
			<xsl:message>Found bucket of width <xsl:value-of select="$bucket_w_"/></xsl:message>
-->				
			<SHAPE WIDTH="{$bucket_w_}"/>
		</xsl:for-each>
			
	</xsl:variable>
	
	<xsl:value-of select="math:max(exsl:node-set($shape_widths_)/SHAPE/@WIDTH)"/>
</xsl:template>
	
	
<xsl:template name="_calc_Stack_X">
	<xsl:param name="stackIdx"  select="0"/>
	
	<!-- Store the stack widths in a variable -->	
	
<!--	
	<xsl:message>Looking for stack indexes less than <xsl:value-of select="$stackIdx"/></xsl:message>
-->	
	
	<xsl:variable name="stackspace_widths_">
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES[(@STACK_HORIZ_INDEX &lt; $stackIdx)])">
			<STACKSPACE WIDTH="0"/>
		</xsl:if>
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES[(@STACK_HORIZ_INDEX &lt; $stackIdx)])">
			<STACKSPACE WIDTH="0"/>
		</xsl:if>
		
		<xsl:if test="not(/EDKSYSTEM/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX &lt; $stackIdx)])">
			<STACKSPACE WIDTH="0"/>
		</xsl:if>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(@EAST &lt;= $stackIdx)]">
			
<!--		
			<xsl:message>==============================</xsl:message>
			<xsl:message>Found a space of index <xsl:value-of select="@EAST"/></xsl:message>
-->	
			
			<xsl:variable name="space_width_" select="($BUS_LANE_W * @BUSLANES_W)"/>
<!--			
			<xsl:message>Bus lane space width <xsl:value-of select="@BUSLANES_W"/></xsl:message>
			<xsl:message>Bus lane space is <xsl:value-of select="$space_width_"/></xsl:message>
-->	
			
				<xsl:variable name="stack_width_">
					<xsl:if test="not(@EAST = $stackIdx)">
						<xsl:call-template name="_calc_Stack_Width">
							<xsl:with-param name="stackIdx"  select="@EAST"/>
						</xsl:call-template>
					</xsl:if>
					<xsl:if test="(@EAST = $stackIdx)">0</xsl:if>
				</xsl:variable>
			
<!--			
			<xsl:message>Found stack of width <xsl:value-of select="$stack_width_"/></xsl:message>
			<xsl:message>==============================</xsl:message>
-->			
			
			<STACKSPACE WIDTH="{$stack_width_ + $space_width_}"/>
			
		</xsl:for-each>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[(not(@EAST) and (@WEST = ($stackIdx -1)))]">
			<xsl:variable name="space_width_" select="($BUS_LANE_W * @BUSLANES_W)"/>
			
<!--			
			<xsl:message>Found end space of <xsl:value-of select="$space_width_"/></xsl:message>
-->			
			<STACKSPACE WIDTH="{$space_width_}"/>
		</xsl:for-each>
		
		
	</xsl:variable>
	
	<xsl:value-of select="sum(exsl:node-set($stackspace_widths_)/STACKSPACE/@WIDTH)"/>
	
</xsl:template>	
	
<xsl:template name="_calc_Space_Width">
	
	<xsl:param name="stackToWest"  select="'NONE'"/>
	<xsl:param name="stackToEast"  select="'NONE'"/>
	
<!--	
	<xsl:message>Stack to West <xsl:value-of select="$stackToWest"/></xsl:message>
	<xsl:message>Stack to East <xsl:value-of select="$stackToEast"/></xsl:message>
-->	
	
	<xsl:variable name="spaceWidth_">
		<xsl:choose>
			<xsl:when test="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[((@EAST = $stackToEast) or (not($stackToWest = 'NONE') and (@WEST = $stackToWest)))]">
				<xsl:value-of select="((/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[((@EAST = $stackToEast) or (not($stackToWest = 'NONE') and (@WEST = $stackToWest)))]/@BUSLANES_W) * $BUS_LANE_W)"/>
			</xsl:when>	
			<xsl:otherwise>0</xsl:otherwise>	
		</xsl:choose>	
	</xsl:variable>	
	
<!--	
	<xsl:message>Space width <xsl:value-of select="$spaceWidth_"/></xsl:message>
-->	
	
	<xsl:value-of select="$spaceWidth_"/>
</xsl:template>
	
	
	
	
<xsl:template name="_calc_Space_X">
	
	<xsl:param name="stackToWest"  select="'NONE'"/>
	<xsl:param name="stackToEast"  select="'NONE'"/>
	
<!--	
	<xsl:message>Stack East <xsl:value-of select="$stackToEast"/></xsl:message>
	<xsl:message>Stack West <xsl:value-of select="$stackToWest"/></xsl:message>
-->	
	
	<!-- Store the stack widths in a variable -->	
	
<!--	
	<xsl:message>Looking for stack indexes less than <xsl:value-of select="$stackIdx"/></xsl:message>
-->	
	
	<xsl:variable name="stackspace_widths_">
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/CMPLXSHAPES[(@STACK_HORIZ_INDEX &lt; $stackToEast)])">
			<STACKSPACE WIDTH="0"/>
		</xsl:if>
		
		<xsl:if test="not(/EDKSYSTEM/BLKDSHAPES/PROCSHAPES[(@STACK_HORIZ_INDEX &lt; $stackToEast)])">
			<STACKSPACE WIDTH="0"/>
		</xsl:if>
		
		<xsl:if test="not(/EDKSYSTEM/SBSBUCKETS/SBSBUCKET[(@STACK_HORIZ_INDEX &lt; $stackToEast)])">
			<STACKSPACE WIDTH="0"/>
		</xsl:if>
		
		<xsl:for-each select="/EDKSYSTEM/BLKDSHAPES/BCLANESPACES/BCLANESPACE[((@EAST &lt; $stackToEast) or (not($stackToWest = 'NONE') and (@EAST &lt;= $stackToWest)))]">
			
<!--		
			<xsl:message>==============================</xsl:message>
			<xsl:message>Found a space of index <xsl:value-of select="@EAST"/></xsl:message>
-->	
			
			<xsl:variable name="space_width_" select="($BUS_LANE_W * @BUSLANES_W)"/>
<!--			
			<xsl:message>Bus lane space width <xsl:value-of select="@BUSLANES_W"/></xsl:message>
			<xsl:message>Bus lane space is <xsl:value-of select="$space_width_"/></xsl:message>
-->	
				<xsl:variable name="stack_width_">
					<xsl:call-template name="_calc_Stack_Width">
						<xsl:with-param name="stackIdx"  select="@EAST"/>
					</xsl:call-template>
				</xsl:variable>
			
<!--			
			<xsl:message>Found stack of width <xsl:value-of select="$stack_width_"/></xsl:message>
			<xsl:message>==============================</xsl:message>
-->			
			
			<STACKSPACE WIDTH="{$stack_width_ + $space_width_}"/>
		</xsl:for-each>
	</xsl:variable>
	
	<xsl:variable name = "stackToWest_W_">
		<xsl:choose>
			<xsl:when test="(($stackToEast = '0')   and     ($stackToWest = 'NONE'))">0</xsl:when>
			<xsl:when test="(($stackToEast = 'NONE') and not($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_Width">
					<xsl:with-param name="stackIdx"  select="$stackToWest"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="(not($stackToEast = '0') and ($stackToWest = 'NONE'))">
				<xsl:call-template name="_calc_Stack_Width">
					<xsl:with-param name="stackIdx"  select="($stackToEast - 1)"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>0</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	
<!--	
	<xsl:variable name = "stackToEast_W_">
		<xsl:call-template name="_calc_Stack_Width">
			<xsl:with-param name="stackIdx"  select="$stackToEast"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name ="extSpaceEast_W_" select="ceiling($stackToEast_W_ div 2)"/>
-->	
	
	<xsl:variable name ="extSpaceWest_W_" select="ceiling($stackToWest_W_ div 2)"/>
	 
	<xsl:value-of select="sum(exsl:node-set($stackspace_widths_)/STACKSPACE/@WIDTH) - $extSpaceWest_W_"/>
</xsl:template>	


			
<!-- ======================= END UTILITY FUNCTIONS  ======================= -->

</xsl:stylesheet>

