--------------------------------------------------------------------------------
-- Company:        Xilinx, Inc.
-- Engineer:       Greg Miller
--
-- Create Date:    3/29/07
-- Design Name:    bram_scrubber_bram
-- Module Name:    bram_scrubber_bram
-- Project Name:   TMR Tool Regression Tests
-- Target Device:  V2/V2Pro/V4
-- Tool versions:  
-- Description:    This is a sub module for the bram_scrubber_wrapper_tmr.
-- This modules pulls together the Block RAMs needed for the bram_scrubber. 
-- This File will Create Dual-Port RAMs based on Address/Data/WE bit size 
-- Automatically
-- Dependencies:
-- 
-- Revision: 8.2.3 - Added changeable BRAM Size... BRAM_Data_Size in pkg. Note: change the 
-- BRAM primitive accordingly. Also fixed WEB math
-- 8.2.2 - Need to change this for V4 specifically, ISE will replace with BRAM16s...
-- Revision 0.13 - This file will Instanitate the RAMs. Moving back for testing 
-- Additional Comments:
-- this is a test for instantited RAMs with separate wes.
-- Use for only for Singla Data Bits... (For now)
-- For example : can use 16kx1 but not 8kx2. (Make more customizable later).
-- You can change the Package file/This file to make other BRAMs but must match a 
-- BRAM primitive (see Data Sheet).
-------------------------------------------------------------------------------


library IEEE; 
use IEEE.std_logic_1164.all; 
use IEEE.std_logic_unsigned.all; 
Library work;
use work.bram_scrubber_pkg.all;

entity raminfr is 
  port( 
    clka  : in  std_logic; 
    clkb  : in  std_logic; 
    ena   : in  std_logic; 
    enb   : in  std_logic; 
    wea   : in  std_logic; 
    web   : in  std_logic_vector(data_width/we_num-1 downto 0); 
    SSRA  : in std_logic;
    SSRB  : in std_logic;
    addra : in  std_logic_vector((addr_width-1) downto 0); 
    addrb : in  std_logic_vector((addr_width-1) downto 0); 
    dia   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    dib   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    doa   : out std_logic_vector((data_width/we_num-1) downto 0); 
    dob   : out std_logic_vector((data_width/we_num-1) downto 0)); 
end raminfr; 

architecture syn of raminfr is

-------BRAM component---------------------
------------------------------------------
component RAMB16_S4_S4                                   --This primitive can be changed
port (    CLKA      : in std_logic;
          ENA       : in std_logic;
          SSRA      : in std_logic;
          WEA       : in std_logic;
          DIA       : in std_logic_vector(BRAM_Data_Size-1 downto 0);
          ADDRA     : in std_logic_vector(addr_width-1 downto 0);
          DOA       : out std_logic_vector(BRAM_Data_Size-1 downto 0);
          CLKB      : in std_logic;
          ENB       : in std_logic;
          SSRB      : in std_logic;
          WEB       : in std_logic;
          DIB       : in std_logic_vector(BRAM_Data_Size-1 downto 0);
          ADDRB     : in std_logic_vector(addr_width-1 downto 0);
          DOB       : out std_logic_vector(BRAM_Data_Size-1 downto 0)
          );
end component;

signal web_or_s     : std_logic_vector(data_width/we_num/BRAM_Data_Size-1 downto 0);


begin


web_or_sig : FOR i in 0 to (data_width/we_num/BRAM_Data_Size -1) generate
     web_or_s(i) <= '0' when web(BRAM_Data_Size+(i*BRAM_Data_Size)-1 downto BRAM_Data_Size*(i+1)-BRAM_Data_Size)=0 else '1';
end generate;



br: FOR i in 0 to (data_width/we_num/BRAM_Data_Size - 1) Generate
     ramb16_s4_s4_inst : RAMB16_S4_S4 --Change this to put the appropriate RAMB primitive, must match BRAM_Data_Size
     port map(      CLKA      => clka,
                    ENA       => ena,
                    SSRA      => SSRA,
                    WEA       => wea,
                    DIA       => DIA(BRAM_Data_Size+(i*BRAM_Data_Size)-1 downto BRAM_Data_Size*(i+1)-BRAM_Data_Size),
                    ADDRA     => ADDRA,
                    DOA       => DOA(BRAM_Data_Size+(i*BRAM_Data_Size)-1 downto BRAM_Data_Size*(i+1)-BRAM_Data_Size),
                    CLKB      => clkb,
                    ENB       => enb,
                    SSRB      => SSRB,
                    WEB       => web_or_s(i),
--                    WEB       => WEB(i),
                    DIB       => DIB(BRAM_Data_Size+(i*BRAM_Data_Size)-1 downto BRAM_Data_Size*(i+1)-BRAM_Data_Size),
                    ADDRB     => ADDRB,
                    DOB       => DOB(BRAM_Data_Size+(i*BRAM_Data_Size)-1 downto BRAM_Data_Size*(i+1)-BRAM_Data_Size)
	);
end Generate;



end syn; 

--######################################################################
--#######Generate the BRAMs based on the number of Write Enables########
--######################################################################

library IEEE; 
use IEEE.std_logic_1164.all; 
use IEEE.std_logic_unsigned.all; 
Library work;
use work.bram_scrubber_pkg.all;

entity bram_scrubber_bram_bitw is 
  port( 
    clka  : in  std_logic; 
    clkb  : in  std_logic; 
    ena   : in  std_logic; 
    enb   : in  std_logic; 
    wea   : in  std_logic_vector((we_num-1) downto 0); 
--    web   : in  std_logic_vector((we_num-1) downto 0); 
--    web   : in std_logic;
    web  : in std_logic_vector(data_width-1 downto 0); --for separate we testing
    SSRA : in std_logic;
    SSRB : in std_logic;
    addra : in  std_logic_vector((addr_width-1) downto 0); 
    addrb : in  std_logic_vector((addr_width-1) downto 0); 
    dia   : in  std_logic_vector((data_width-1) downto 0); 
    dib   : in  std_logic_vector((data_width-1) downto 0); 
    doa   : out std_logic_vector((data_width-1) downto 0); 
    dob   : out std_logic_vector((data_width-1) downto 0)); 
end bram_scrubber_bram_bitw; 

architecture struct of bram_scrubber_bram_bitw is 

component raminfr is
    port (
    clka  : in  std_logic; 
    clkb  : in  std_logic; 
    ena   : in  std_logic; 
    enb   : in  std_logic; 
    wea   : in  std_logic; 
    web   : in  std_logic_vector(data_width/we_num-1 downto 0); ----for separate we testing 
    SSRA  : in std_logic;
    SSRB  : in std_logic;
    addra : in  std_logic_vector((addr_width-1) downto 0); 
    addrb : in  std_logic_vector((addr_width-1) downto 0); 
    dia   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    dib   : in  std_logic_vector((data_width/we_num-1) downto 0); 
    doa   : out std_logic_vector((data_width/we_num-1) downto 0); 
    dob   : out std_logic_vector((data_width/we_num-1) downto 0)
    );
end component;


begin 
--bram : FOR i in 0 to (we_num-1)  Generate
bram : FOR i in (we_num-1) downto 0  Generate

    raminfr_inst : raminfr                                        
    port map (
     clka  => clka,
     clkb  => clkb, 
     ena   => ena, 
     enb   => enb, 
     wea   => wea(i), 
     web   => web( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
--     web   => web,
     SSRA => SSRA,
     SSRB => SSRB,
     addra => addra,
     addrb => addrb,
     dia   => dia( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     dib   => dib( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     doa   => doa( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num ),
     dob   => dob( data_width/we_num + i*(data_width/we_num) - 1 downto data_width/we_num*(i+1) - data_width/we_num )
    );

End Generate;
end struct;
