--------------------------------------------------------------------------------
-- Company:        Xilinx, Inc.
-- Engineer:       Greg Miller
--
-- Create Date:    5-10-07
-- Design Name:    bram_scrubber_data_voter
-- Module Name:    bram_scrubber_data_voter
-- Project Name:   TMR Tool Regression Tests
-- Target Device:  V2/V2Pro
-- Tool versions:  
-- Description:    This is a sub module for the bram_scrubber_wrapper_tmr.
-- This design votes on the data from the BRAMs. Clock is inverted Here
-- Dependencies:
-- 
-- Revision:       8.2.3 - Changed to match other HLD PLB BS version
-- Removed Registers
-- Additional Comments: configurable via the bram scrubber package
-- 

-------------------------------------------------------------------------------


library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
use ieee.std_logic_unsigned.all;
library work;
use work.bram_scrubber_pkg.all;
-- library synplify;
-- use synplify.attributes.all;

entity bram_scrubber_data_voter is
port (
     TR0     : in std_logic_vector(data_width-1 downto 0);
     TR1     : in std_logic_vector(data_width-1 downto 0); 
     TR2     : in std_logic_vector(data_width-1 downto 0);
--     PRE     : in std_logic; 
--     CE      : in std_logic;
--     C       : in std_logic;
--     CLR     : in std_logic;
     Din_B   : out std_logic_vector(data_width-1 downto 0);
     ERROR_B : out std_logic_vector(data_width-1 downto 0)
     );
end bram_scrubber_data_voter;

Architecture rtl of bram_scrubber_data_voter is
-- attribute syn_edif_bit_format of rtl : architecture is "%n<%i>";


component TRV_LUT
port (
   TR0 : in std_logic;
   TR1 : in std_logic;
   TR2 : in std_logic;
   V   : out std_logic);
end component;

component TMR_DET
port (
   Prime : in std_logic;
   redA  : in std_logic;
   redB  : in std_logic;
   ERROR : out std_logic);
end component;

--Signal ERR_B_sig : std_logic_vector(data_width-1 downto 0);
--Signal Din_B_sig : std_logic_vector(data_width-1 downto 0);

Begin

--------------------------------------------------
--Synthesize a FDCPE (Org Data_B_ErrDet_Reg)
--process (C, CLR, PRE, CE)
--begin
--     if CLR='1' then
--          ERROR_B <= (others => '0');
--     elsif PRE='1' then
--          ERROR_B <= (others => '1');
--     elsif C'event and C='0' then
--          if CE='1' then
--               ERROR_B <= ERR_B_sig;
--          end if;
--     end if;
--end process;

--Synthesize a FDCPE (Org Data_B_Voter_Reg)
--process (C, CLR, PRE, CE)
--begin
--     if CLR='1' then
--          Din_B <= (others => '0');
--     elsif PRE='1' then
--          Din_B <= (others => '1');
--     elsif C'event and C='0' then
--          if CE='1' then
--               Din_B <= Din_B_sig;
--          end if;
--     end if;
--end process;

--------------------------------------------------

--process(CLK, SET, RST, CE) 
--begin 
--    if CLK'event and CLK ='1' then
--	      if (RST = '1') then
--               CEN <= '0';
--          elsif (SET = '1') then
--               CEN <= '1';
--          elsif (CE = '1') then
--               if (wenb_s = '0') then
--                    CEN <= '1';
--               else
--                    CEN <= '0';
--               end if;
--          end if;
--     end if;
--end process;

--Synthesize a FDRSE (Org Data_B_ErrDet_Reg)
--process (C, CLR, PRE, CE)
--begin
--     if C'event and C='0' then
--          if (CLR = '1') then
--               ERROR_B <= (others => '0');
--          elsif (PRE = '1') then
--               ERROR_B <= (others => '1');
--          elsif (CE = '1') then
--               ERROR_B <= ERR_B_sig;
--          end if;
--     end if;
--end process;

--Synthesize a FDRSE (Org Data_B_Voter_Reg)
--process (C, CLR, PRE, CE)
--begin
--      if C'event and C='0' then
--          if (CLR = '1') then
--               Din_B <= (others => '0');
--          elsif (PRE = '1') then
--               Din_B <= (others => '1');
--          elsif (CE = '1') then
--               Din_B <= Din_B_sig;
--          end if;
--      end if;
--end process;

------------------------------------------
------------------------------------------

data_voter : FOR i in 0 to (data_width-1) Generate
DATA_B_Voter : TRV_LUT
   port map (
             TR0 => TR0(i),
             TR1 => TR1(i), 
             TR2 => TR2(i), 
--             V   => Din_B_sig(i)
             V => Din_B(i)
            );

DATA_B_ErrDet : TMR_DET
   port map (
             Prime => TR0(i),
             redA  => TR1(i),
             redB  => TR2(i), 
--             ERROR => ERR_B_sig(i)
             ERROR => ERROR_B(i)
            );
------------------------------------------
End Generate;

end rtl;

--   DATA_B_ErrDet_tr0_Reg: FDCPE
--   port map (
--        PRE =>  internal_gnd_TR0,
--             CE  =>  internal_vcc_TR0, 
--             C => CLKB_TR0, 
--             D => ERR_B_TR0_sig(i), 
--            Q => ERR_B_TR0(i), 
--             CLR => CENVTMR_TR0
--        );

--   DATA_B_Voter_tr0: TRV_LUT
--   port map (
--             TR0=>Dout_B_TR0(i),Dout_B_TR2
--             TR1=>Dout_B_TR1(i), 
--             TR2=>Dout_B_TR2(i), 
--             V=>Din_B_TR0_sig(i)
--        );

--   DATA_B_Voter_tr0_Reg: FDCPE
--   port map (
--        PRE =>  internal_gnd_TR0, 
--        CE =>   internal_vcc_TR0, 
--        C  =>   CLKB_TR0,
  --           D  =>   Din_B_TR0_sig(i),
    --         Q  =>   Din_B_TR0(i),
      --       CLR => CENVTMR_TR0
        --);

--   DATA_B_ErrDet_tr0: TMR_DET
--   port map (
--             Prime => Dout_B_TR0(i),
--             redA =>  Dout_B_TR1(i),
--             redB =>  Dout_B_TR2(i), 
--             ERROR => ERR_B_TR0_sig(i)
--        );

--   DATA_B_ErrDet_tr0_Reg: FDCPE
--   port map (
--        PRE =>  internal_gnd_TR0,
--             CE  =>  internal_vcc_TR0, 
--             C => CLKB_TR0, 
--             D => ERR_B_TR0_sig(i), 
--            Q => ERR_B_TR0(i), 
--             CLR => CENVTMR_TR0
--        );




-----------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;

entity TMR_DET is
port (
   Prime : in std_logic;
   redA : in std_logic;
   redB : in std_logic;
   ERROR : out std_logic);
end TMR_DET;

Architecture rtl of TMR_DET is


Begin
	ERROR <= (Prime xor redA) and (Prime xor redB);
end rtl;

-----------------------------------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
--library synplify;
--use synplify.attributes.all;
--library work;
--use work.FpgaPkg.all;

entity TRV_LUT is
port (
   TR0 : in std_logic;
   TR1 : in std_logic;
   TR2 : in std_logic;
   V : out std_logic);
end TRV_LUT;

Architecture rtl of TRV_LUT is

--attribute xc_map of rtl : architecture is "lut";

Begin
	V <= (TR0 and TR1) or (TR1 and TR2) or (TR2 and TR0);
end rtl;
-----------------------------------------------------------------


--   DATA_B_ErrDet_tr0_Reg: FDCPE
--   port map (
--        PRE =>  internal_gnd_TR0,
--             CE  =>  internal_vcc_TR0, 
--             C => CLKB_TR0, 
--             D => ERR_B_TR0_sig(i), 
--            Q => ERR_B_TR0(i), 
--             CLR => CENVTMR_TR0
--        );

--   DATA_B_Voter_tr0: TRV_LUT
--   port map (
--             TR0=>Dout_B_TR0(i),
--             TR1=>Dout_B_TR1(i), 
--             TR2=>Dout_B_TR2(i), 
--             V=>Din_B_TR0_sig(i)
--        );

--   DATA_B_Voter_tr0_Reg: FDCPE
--   port map (
--        PRE =>  internal_gnd_TR0, 
--        CE =>   internal_vcc_TR0, 
--        C  =>   CLKB_TR0,
  --           D  =>   Din_B_TR0_sig(i),
    --         Q  =>   Din_B_TR0(i),
      --       CLR => CENVTMR_TR0
        --);

--   DATA_B_ErrDet_tr0: TMR_DET
--   port map (
--             Prime => Dout_B_TR0(i),
--             redA =>  Dout_B_TR1(i),
--             redB =>  Dout_B_TR2(i), 
--             ERROR => ERR_B_TR0_sig(i)
--        );

--   DATA_B_ErrDet_tr0_Reg: FDCPE
--   port map (
--        PRE =>  internal_gnd_TR0,
--             CE  =>  internal_vcc_TR0, 
--             C => CLKB_TR0, 
--             D => ERR_B_TR0_sig(i), 
--            Q => ERR_B_TR0(i), 
--             CLR => CENVTMR_TR0
--        );

