--------------------------------------------------------------------------------
-- Company:        Xilinx, Inc.
-- Engineer:       Greg Miller
--
-- Create Date:    5-9-07
-- Design Name:    bram_scrubber_pkg
-- Module Name:    bram_scrubber_pkg
-- Project Name:   BRAM Scrubber functional Model
-- Target Device:  V2/V2Pro
-- Tool versions:  
-- Description:    This package must be used in any bram_scrubber design.
-- Change address/data and we_num for user applications
--
-- Dependencies: - bram_scrubber_pkg.vhd
-- 
-- Revision:       8.2.3 - Added BRAM_Data_Size/scrub_delay.
--                       - Updated to current BS version
-- Revision 1.2 - File Created
-- Changed for individual bit BRAM correction
-- Additional Comments: 
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

package bram_scrubber_pkg is

  
--  type <new_type> is
--    record
--        <type_name>        : std_logic_vector( 7 downto 0);
--        <type_name>        : std_logic;
--    end record;

-- Declare constants
    constant data_width     : integer := 64; --Always 64 for PPC Designs
    constant addr_width     : integer := 12; --For 13(0-0xffff) 12(0-0x8000)
    constant we_num         : integer := 8;  --Always 8 for PPC designs
    constant BRAM_Data_Size : integer := 4; --Size of BRAM Data Width (BRAM primitive must be changed to same size in bram_scrubber_bram_inst_bitw.vhd)
    constant scrub_delay    : integer := 1; --This is the number of cycles to hold address B on the scrub (default=1)
                                            --Increasing this number will decrease the scrubbing time. Might be useful
                                            --For faster clocks
    --Example : BRAM_Data_Size = 2, The RAM primitve in bram_scrubber_bram_inst_bitw.

--Components
-- Port B address Counter Component
component BSCNTR_XTMR
port (
--Removed for V4 HL extraction (may not be needed)
--     internal_gnd_tr0 : in std_logic;
--     internal_gnd_tr1 : in std_logic;
--     internal_gnd_tr2 : in std_logic;
--     internal_vcc_tr0 : in std_logic;
--     internal_vcc_tr1 : in std_logic;
--     internal_vcc_tr2 : in std_logic;
     CLK_TR0 : in std_logic;
     CLK_TR1 : in std_logic;
     CLK_TR2 : in std_logic;
     SET_TR0 : in std_logic;
     SET_TR1 : in std_logic;
     SET_TR2 : in std_logic;
     RST_TR0 : in std_logic;
     RST_TR1 : in std_logic;
     RST_TR2 : in std_logic;
     CE_TR0 : in std_logic;
     CE_TR1 : in std_logic;
     CE_TR2 : in std_logic;
     ADDRESS_TR0 : out std_logic_vector((addr_width-1) downto 0);
     ADDRESS_TR1 : out std_logic_vector((addr_width-1) downto 0);
     ADDRESS_TR2 : out std_logic_vector((addr_width-1) downto 0)
	 );
end component;

--Address Comparator Component
component Comparator_TMR
port (
	--internal_gnd_tr0 : in std_logic;
	--internal_gnd_tr1 : in std_logic;
	--internal_gnd_tr2 : in std_logic;
	--internal_vcc_tr0 : in std_logic;
	--internal_vcc_tr1 : in std_logic;
	--internal_vcc_tr2 : in std_logic;

--     CLK_TR0 : in std_logic; 
--     CLK_TR1 : in std_logic; 
--     CLK_TR2 : in std_logic;  

--     SET_TR0 : in std_logic; 
--     SET_TR1 : in std_logic; 
--     SET_TR2 : in std_logic;  

--     RST_TR0 : in std_logic; 
--     RST_TR1 : in std_logic; 
--     RST_TR2 : in std_logic;  

--     CE_TR0  : in std_logic;
--     CE_TR1  : in std_logic;
--     CE_TR2  : in std_logic;

     CEN_TR0 : out std_logic;
     CEN_TR1 : out std_logic;
     CEN_TR2 : out std_logic;

     ENB_TR0 : out std_logic;
     ENB_TR1 : out std_logic;
     ENB_TR2 : out std_logic;

--     CENV_TR0 : out std_logic;
--     CENV_TR1 : out std_logic;
--     CENV_TR2 : out std_logic;

     ADDR_A_TR0 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_A_TR1 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_A_TR2 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_B_TR0 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_B_TR1 : in std_logic_vector ((addr_width-1) downto 0);
     ADDR_B_TR2 : in std_logic_vector ((addr_width-1) downto 0);

	 WEN_A_TR0  : in std_logic_vector((we_num-1)     downto 0);
	 WEN_A_TR1  : in std_logic_vector((we_num-1)     downto 0);
	 WEN_A_TR2  : in std_logic_vector((we_num-1)     downto 0);

--     WEN_B_TR0 : inout std_logic_vector (0 to data_width-1);
--     WEN_B_TR1 : inout std_logic_vector (0 to data_width-1);
--     WEN_B_TR2 : inout std_logic_vector (0 to data_width-1);
     WEN_B_TR0 : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR1 : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR2 : out std_logic_vector(data_width-1 downto 0);

     ERR_B_TR0 : in std_logic_vector (data_width-1 downto 0);
     ERR_B_TR1 : in std_logic_vector (data_width-1 downto 0);
     ERR_B_TR2 : in std_logic_vector (data_width-1 downto 0)
          );
end component;

component bram_scrubber_bram
     port (
          CLKA  : in std_logic;
          ENA   : in std_logic;
          SSRA  : in std_logic;   --Add For Full HL Extraction
          WEA   : in std_logic_vector((we_num-1) downto 0);
          DIA   : in std_logic_vector(data_width-1 downto 0);
          ADDRA : in std_logic_vector(addr_width-1 downto 0);
          DOA   : out std_logic_vector(data_width-1 downto 0);
          CLKB  : in std_logic;
          ENB   : in std_logic;
          SSRB  : in std_logic;   --Add For Full HL Extraction
          WEB   : in std_logic;
          DIB   : in std_logic_vector(data_width-1 downto 0);
          ADDRB : in std_logic_vector(addr_width-1 downto 0);
          DOB   : out std_logic_vector(data_width-1 downto 0)
            );
end component;

component bram_scrubber_bram_bitw
     port (
          CLKA  : in std_logic;
          ENA   : in std_logic;
          SSRA  : in std_logic;   --Add For Full HL Extraction
          WEA   : in std_logic_vector((we_num-1) downto 0);
          DIA   : in std_logic_vector(data_width-1 downto 0);
          ADDRA : in std_logic_vector(addr_width-1 downto 0);
          DOA   : out std_logic_vector(data_width-1 downto 0);
          CLKB  : in std_logic;
          ENB   : in std_logic;
          SSRB  : in std_logic;   --Add For Full HL Extraction
          WEB   : in std_logic_vector(data_width-1 downto 0);
          DIB   : in std_logic_vector(data_width-1 downto 0);
          ADDRB : in std_logic_vector(addr_width-1 downto 0);
          DOB   : out std_logic_vector(data_width-1 downto 0)
            );
end component;

component bram_scrubber_data_voter
     port (
          TR0                 : in std_logic_vector(data_width-1 downto 0);
          TR1                 : in std_logic_vector(data_width-1 downto 0); 
          TR2                 : in std_logic_vector(data_width-1 downto 0);
--          PRE                 : in std_logic; 
--          CE                  : in std_logic;
--          C                   : in std_logic;
--          CLR                 : in std_logic;
          Din_B               : out std_logic_vector(data_width-1  downto 0);
          ERROR_B             : out std_logic_vector(data_width-1 downto 0)
        );
end component;

component TRV_LUT
port (
   TR0 : in std_logic;
   TR1 : in std_logic;
   TR2 : in std_logic;
   V : out std_logic);
end component;


component TMR_DET
port (
   Prime : in std_logic;
   redA : in std_logic;
   redB : in std_logic;
   ERROR : out std_logic);
end component;

component CEN_bscntr_tmr 
port (
     BOGUS_VCC : in std_logic;
     CEN_TR0 : in std_logic; 
     CEN_TR1 : in std_logic;
     CEN_TR2 : in std_logic; 
     CEN_bscntr_tmr : out std_logic);
end component;

component CENVTMR 
port (
     BOGUS_GND : in std_logic;
     CENV_TR0 : in std_logic; 
     CENV_TR1 : in std_logic;
     CENV_TR2 : in std_logic; 
     CENVTMR  : out std_logic);
end component; 
-- Declare functions and procedure

--  function <function_name>  (signal <signal_name> : in <type_declaration>) return <type_declaration>;
--  procedure <procedure_name>	(<type_declaration> <constant_name>	: in <type_declaration>);

end bram_scrubber_pkg;


-- package body bram_scrubber_pkg_body is

-- Example 1
--  function <function_name>  (signal <signal_name> : in <type_declaration>  ) return <type_declaration> is
--    variable <variable_name>     : <type_declaration>;
--  begin
--    <variable_name> := <signal_name> xor <signal_name>);
--    return <variable_name>; 
--  end <function_name>;


-- Example 2
--  function <function_name>  (signal <signal_name> : in <type_declaration>;
--                         signal <signal_name>   : in <type_declaration>  ) return <type_declaration> is
--  begin
--    if (<signal_name> = '1') then
--      return <signal_name>;
--    else
--      return 'Z';
--    end if;
--  end <function_name>;

-- Procedure Example
--  procedure <procedure_name>  (<type_declaration> <constant_name>  : in <type_declaration>) is
    
--  begin
    
--  end <procedure_name>;
 
-- end package body bram_scrubber_pkg_body;
