--------------------------------------------------------------------------------
-- Company:        Xilinx, Inc.
-- Engineer:       Greg Miller
--
-- Create Date:    5/8/07
-- Design Name:    plb_bram_if_cntlr_1_bram_wrapper_tmr
-- Module Name:    plb_bram_if_cntlr_1_bram_wrapper_tmr
-- Project Name:   BRAM Scrubber functional Model
-- Target Device:  V2/V2Pro/V4
-- Tool versions:  
-- Description:    This is the actual BRAM scrubber. To be used as the BRAM
-- replacement on the plb bus.
--
-- Dependencies: - bram_scrubber_pkg.vhd
-- 
-- Revision:       8.2.3 - V4/ and some code cleaning - Matches 
-- Revision 8.0 - Changed to match project rev. Also, changed the AND/OR gates such that they will not be removed because they are redundant.
-- -- Additional Comments: Now customizable with address/data & number of num_we
-- DP BRAM is inferred. Have to use XST until Synplicity UTHOTA
-- Got Rid of CENVTMR Use comparitor instead to control we from errors.
-- Has optional testing outputs - WEB/Addr_B
-- Has separate we for BRAMs -> 
-- Now Comparator is Hand Triplicated and is in this file
-- This version gets rid of the redundant large AND gate for reporting of WEBs. 
-- The overall code is the same as 8.2. 8.2.1 is just getting rid of the large gates. 
-- Using the gates from the Comparitor that already exist.
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- plb_bram_if_cntrl_1_bram_wrapper_tmr.vhd
-------------------------------------------------------------------------------
--NOTE: Use the Package to Choose BRAM size. Change the addr_width and
-- Data_width as needed. However, below is locked for the plb bus. To get
-- your addr_width : size of log2(size of BRAM address/8).
-- Example (0-0xffff) = 64k bits is /8 = 8129 = 13 bits of address (used in a later
-- calulation

--NOTE : Address/Data are fixed for PLB Bus

-----------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
--library synplify;
--use synplify.attributes.all;
--library work;
--use work.FpgaPkg.all;

entity CEN_bscntr_tmr is
port (
     BOGUS_VCC : in std_logic;
     CEN_TR0 : in std_logic; 
     CEN_TR1 : in std_logic;
     CEN_TR2 : in std_logic; 
     CEN_bscntr_tmr  : out std_logic);
end CEN_bscntr_tmr;

Architecture rtl of CEN_bscntr_tmr is

--attribute xc_map of rtl : architecture is "lut";

Begin
     CEN_bscntr_tmr <= CEN_TR0 and CEN_TR1 and CEN_TR2 and BOGUS_VCC;
end rtl;
----------------------------------------

----------------------------------------
--library IEEE;
--use IEEE.Std_Logic_1164.all;
--use IEEE.Std_logic_arith.all;
--library synplify;
--use synplify.attributes.all;
--library work;
--use work.FpgaPkg.all;

--entity CENVTMR is
--port (
--     BOGUS_GND : in std_logic;
--     CENV_TR0 : in std_logic; 
--     CENV_TR1 : in std_logic;
--     CENV_TR2 : in std_logic; 
--     CENVTMR  : out std_logic);
--end CENVTMR;
--
--Architecture rtl of CENVTMR is
--
----attribute xc_map of rtl : architecture is "lut";
--
--Begin
--     CENVTMR <= CENV_TR0 or CENV_TR1 or CENV_TR2 or BOGUS_GND;
--end rtl;
---------------------------------------

---------------------------------------
library IEEE;
use IEEE.Std_Logic_1164.all;
use IEEE.Std_logic_arith.all;
Library work;
use work.bram_scrubber_pkg.all;

entity Comparator_TMR is
port (
--     internal_gnd_tr0 : in std_logic;
--     internal_gnd_tr1 : in std_logic;
--     internal_gnd_tr2 : in std_logic;
--     internal_vcc_tr0 : in std_logic;
--     internal_vcc_tr1 : in std_logic;
--     internal_vcc_tr2 : in std_logic;
--     CLK_TR0          : in std_logic;
--     CLK_TR1          : in std_logic;
--     CLK_TR2          : in std_logic;
--     SET_TR0          : in std_logic;
--     SET_TR1          : in std_logic;
--     SET_TR2          : in std_logic;
--     RST_TR0          : in std_logic;
--     RST_TR1          : in std_logic;
--     RST_TR2          : in std_logic; 
--     CE_TR0           : in std_logic;
--     CE_TR1           : in std_logic;
--     CE_TR2           : in std_logic;
     ADDR_A_TR0       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_A_TR1       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_A_TR2       : in std_logic_vector((addr_width-1) downto 0);
	WEN_A_TR0        : in std_logic_vector((we_num-1)     downto 0);
	WEN_A_TR1        : in std_logic_vector((we_num-1)     downto 0);
	WEN_A_TR2        : in std_logic_vector((we_num-1)     downto 0);
     ADDR_B_TR0       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_B_TR1       : in std_logic_vector((addr_width-1) downto 0);
     ADDR_B_TR2       : in std_logic_vector((addr_width-1) downto 0); 
     ERR_B_TR0        : in std_logic_vector(data_width-1 downto 0);
     ERR_B_TR1        : in std_logic_vector(data_width-1 downto 0);
     ERR_B_TR2        : in std_logic_vector(data_width-1 downto 0);
	CEN_TR0          : out std_logic;
     CEN_TR1          : out std_logic;
     CEN_TR2          : out std_logic;
	WEN_B_TR0        : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR1        : out std_logic_vector(data_width-1 downto 0);
     WEN_B_TR2        : out std_logic_vector(data_width-1 downto 0);
     ENB_TR0          : out std_logic;
     ENB_TR1          : out std_logic;
     ENB_TR2          : out std_logic
--     WEN_B_ind0       : out std_logic; --NOTE : These exist in the BS for testing, not in PLB Version
--     WEN_B_ind1       : out std_logic;
--     WEN_B_ind2       : out std_logic
          );
end Comparator_TMR;

Architecture rtl of Comparator_TMR is

----Signals---
signal address_conflict_TR0_s, address_conflict_TR1_s, address_conflict_TR2_s : std_logic;
signal WEN_B_TR0_s, WEN_B_TR1_s, WEN_B_TR2_s                                  : std_logic_vector(data_width-1 downto 0);
--NOTE : These exist in the BS for testing, not in PLB Version
--signal WEN_B_ind0_s, WEN_B_ind1_s, WEN_B_ind2_s                               : std_logic; --Write Enable signal indicators
--------------
begin
-----------------------------------------------

-----------------------------------------------
--Make write enable b indication by reducing the write anable signals using an or gate.
--Will make a one if there is an error
--WEN_B_ind0_s <= '0' when (WEN_B_TR0_s = (WEN_B_TR0_s'range => '0')) else '1';
--WEN_B_ind1_s <= '0' when (WEN_B_TR1_s = (WEN_B_TR1_s'range => '0')) else '1';
--WEN_B_ind2_s <= '0' when (WEN_B_TR2_s = (WEN_B_TR2_s'range => '0')) else '1';

--WEN_B_ind0_s <= '1';
--WEN_B_ind1_s <= '0';
--WEN_B_ind2_s <= '1';

--WEN_B_ind0   <= WEN_B_ind0_s;
--WEN_B_ind1   <= WEN_B_ind1_s;
--WEN_B_ind2   <= WEN_B_ind2_s;
-----------------------------------------------



--Check address conflict--
ac0 : process(Addr_A_TR0, Addr_B_TR0, WEN_A_TR0)
begin
     if ( (Addr_A_TR0 = Addr_B_TR0) )  and (WEN_A_TR0 /= (WEN_A_TR0'range => '0'))  then
               address_conflict_TR0_s <= '1';
          else
               address_conflict_TR0_s <= '0';
     end if;
end process;

--Check address conflict--
ac1 : process(Addr_A_TR1, Addr_B_TR1, WEN_A_TR1)
begin
     if ( (Addr_A_TR1 = Addr_B_TR1) )  and (WEN_A_TR1 /= (WEN_A_TR1'range => '0'))  then
               address_conflict_TR1_s <= '1';
          else
               address_conflict_TR1_s <= '0';
     end if;
end process;

--Check address conflict--
ac2 : process(Addr_A_TR2, Addr_B_TR2, WEN_A_TR2)
begin
     if ( (Addr_A_TR2 = Addr_B_TR2) )  and (WEN_A_TR2 /= (WEN_A_TR2'range => '0'))  then
               address_conflict_TR2_s <= '1';
          else
               address_conflict_TR2_s <= '0';
     end if;
end process;
--------------------------------------------------

--Feed though WEB from errors no matter what---
WEN_B_TR0_s <= ERR_B_TR0;
WEN_B_TR1_s <= ERR_B_TR1;
WEN_B_TR2_s <= ERR_B_TR2;

WEN_B_TR0 <= WEN_B_TR0_s;
WEN_B_TR1 <= WEN_B_TR1_s;
WEN_B_TR2 <= WEN_B_TR2_s;

--------------------------------------------------
comp_tr0 : process(address_conflict_TR0_s, WEN_B_TR0_s)
begin 
	if address_conflict_TR0_s = '1' then
		ENB_TR0 <= '0';                                    --Don't Correct by turning off BRAM
		CEN_TR0 <= '1';                                    --Turn On Address Counter
	else
        ENB_TR0 <= '1'; 
		if (WEN_B_TR0_s = (WEN_B_TR0_s'range => '0')) then
			CEN_TR0 <= '1';                                --When there are no errors (WEB) then counter is on
		else
			CEN_TR0 <= '0';                                --Turn off counter so that it has time to write
		end if;
	end if;
end process;

comp_tr1 : process(address_conflict_TR1_s, WEN_B_TR1_s)
begin 
	if address_conflict_TR1_s = '1' then
		ENB_TR1 <= '0';                                    --Don't Correct by turning off BRAM
		CEN_TR1 <= '1';                                    --Turn On Address Counter
	else
        ENB_TR1 <= '1'; 
		if (WEN_B_TR1_s = (WEN_B_TR1_s'range => '0')) then
			CEN_TR1 <= '1';                                --When there are no errors (WEB) then counter is on
		else
			CEN_TR1 <= '0';                                --Turn off counter so that it has time to write
		end if;
	end if;
end process;

comp_tr2 : process(address_conflict_TR2_s, WEN_B_TR2_s)
begin 
	if address_conflict_TR2_s = '1' then
		ENB_TR2 <= '0';                                    --Don't Correct by turning off BRAM
		CEN_TR2 <= '1';                                    --Turn On Address Counter
	else
        ENB_TR2 <= '1'; 
		if (WEN_B_TR2_s = (WEN_B_TR2_s'range => '0')) then
			CEN_TR2 <= '1';                                --When there are no errors (WEB) then counter is on
		else
			CEN_TR2 <= '0';                                --Turn off counter so that it has time to write
		end if;
	end if;
end process;
--------------------------------------------------

--------------------------------------------------
--Control WEB and turn off address counter for each BRAM--
--comp_tr0 : process(address_conflict_TR0_s, ERR_B_TR0)
--begin 
--	if address_conflict_TR0_s = '1' then
--		WEN_B_TR0_s <= (others => '0');                    --Don't Correct
--		CEN_TR0 <= '1';                                    --Turn On Address Counter
--	else
--		for i in 0 to (data_width-1) loop
--			WEN_B_TR0_s(i) <= ERR_B_TR0(i);                --Just feed the errors through
--		end loop;
--		if (WEN_B_TR0_s = (WEN_B_TR0_s'range => '0')) then
--			CEN_TR0 <= '1';                                --When there are no errors (WEB) then counter is on
--		else
--			CEN_TR0 <= '0';                                --Turn off counter so that it has time to write
--		end if;
--	end if;
--end process;

--Control WEB and turn off address counter for each BRAM--
--comp_tr1 : process(address_conflict_TR1_s, ERR_B_TR1)
--begin 
--	if address_conflict_TR1_s = '1' then
--		WEN_B_TR1_s <= (others => '0');
--		CEN_TR1 <= '1';
--	else
--		for i in 0 to (data_width-1) loop
--			WEN_B_TR1_s(i) <= ERR_B_TR1(i);
--		end loop;
--		if (WEN_B_TR1_s = (WEN_B_TR1_s'range => '0')) then
--			CEN_TR1 <= '1';
--		else
--			CEN_TR1 <= '0';
--		end if;
--	end if;
--end process;

--Control WEB and turn off address counter for each BRAM--
--comp_tr2 : process(address_conflict_TR2_s, ERR_B_TR2)
--begin 
--	if address_conflict_TR2_s = '1' then
--		WEN_B_TR2_s <= (others => '0');
--		CEN_TR2 <= '1';
--	else
--		for i in 0 to (data_width-1) loop
--			WEN_B_TR2_s(i) <= ERR_B_TR2(i);
--		end loop;
--		if (WEN_B_TR2_s = (WEN_B_TR2_s'range => '0')) then
--			CEN_TR2 <= '1';
--		else
--			CEN_TR2 <= '0';
--		end if;
--	end if;
--end process;

WEN_B_TR0 <= WEN_B_TR0_s;
WEN_B_TR1 <= WEN_B_TR1_s;
WEN_B_TR2 <= WEN_B_TR2_s;

end rtl;
----------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
Library work;
use work.bram_scrubber_pkg.all;


entity plb_bram_if_cntlr_1_bram_wrapper_tmr is
  port (
--    internal_gnd : in std_logic;
--    internal_vcc : in std_logic;
--    internal_gnd_TR2 : in std_logic;
--    internal_vcc_TR0 : in std_logic;
--    internal_vcc_TR1 : in std_logic;
--    internal_vcc_TR2 : in std_logic;
    BRAM_Rst_A_TR0 	: in std_logic;
    BRAM_Rst_A_TR1 	: in std_logic;
    BRAM_Rst_A_TR2 	: in std_logic;
    BRAM_Clk_A_TR0 	: in std_logic;
    BRAM_Clk_A_TR1 	: in std_logic;
    BRAM_Clk_A_TR2 	: in std_logic;
    BRAM_EN_A_TR0 	: in std_logic;
    BRAM_EN_A_TR1 	: in std_logic;
    BRAM_EN_A_TR2 	: in std_logic;
    BRAM_WEN_A_TR0 	: in std_logic_vector(0 to we_num-1);
    BRAM_WEN_A_TR1 	: in std_logic_vector(0 to we_num-1);
    BRAM_WEN_A_TR2 	: in std_logic_vector(0 to we_num-1);
    BRAM_Addr_A_TR0  : in std_logic_vector(0 to 31);
    BRAM_Addr_A_TR1  : in std_logic_vector(0 to 31);
    BRAM_Addr_A_TR2  : in std_logic_vector(0 to 31);
    BRAM_Din_A_TR0 	: out std_logic_vector(0 to 63);
    BRAM_Din_A_TR1 	: out std_logic_vector(0 to 63);
    BRAM_Din_A_TR2 	: out std_logic_vector(0 to 63);
    BRAM_Dout_A_TR0  : in std_logic_vector(0 to 63);
    BRAM_Dout_A_TR1  : in std_logic_vector(0 to 63);
    BRAM_Dout_A_TR2  : in std_logic_vector(0 to 63);
    BRAM_Rst_B_TR0 	: in std_logic;					--not used
    BRAM_Rst_B_TR1 	: in std_logic;					--not used
    BRAM_Rst_B_TR2 	: in std_logic;					--not used
    BRAM_Clk_B_TR0 	: in std_logic;					--not used
    BRAM_Clk_B_TR1 	: in std_logic;					--not used
    BRAM_Clk_B_TR2 	: in std_logic;					--not used
    BRAM_EN_B_TR0 	: in std_logic;					--not used
    BRAM_EN_B_TR1 	: in std_logic;					--not used
    BRAM_EN_B_TR2 	: in std_logic;					--not used
    BRAM_WEN_B_TR0 	: in std_logic_vector(0 to we_num-1);		--not used
    BRAM_WEN_B_TR1 	: in std_logic_vector(0 to we_num-1);		--not used
    BRAM_WEN_B_TR2 	: in std_logic_vector(0 to we_num-1);		--not used
    BRAM_Addr_B_TR0  : in std_logic_vector(0 to 31);		--not used
    BRAM_Addr_B_TR1  : in std_logic_vector(0 to 31);		--not used
    BRAM_Addr_B_TR2  : in std_logic_vector(0 to 31);		--not used
    BRAM_Din_B_TR0 	: out std_logic_vector(0 to 63);		--not used
    BRAM_Din_B_TR1 	: out std_logic_vector(0 to 63);		--not used
    BRAM_Din_B_TR2 	: out std_logic_vector(0 to 63);		--not used
    BRAM_Dout_B_TR0  : in std_logic_vector(0 to 63);		--not used
    BRAM_Dout_B_TR1  : in std_logic_vector(0 to 63);		--not used
    BRAM_Dout_B_TR2  : in std_logic_vector(0 to 63)		--not used
     
  );

end plb_bram_if_cntlr_1_bram_wrapper_tmr;

architecture rtl of plb_bram_if_cntlr_1_bram_wrapper_tmr is

---------------------------CS Optional-----------------------------
-------------------------------------------------------------------
 --
 --  ICON Pro core component declaration
 --
 -------------------------------------------------------------------
 component icon
   port
   (
     control0    :   out std_logic_vector(35 downto 0)
   );
 end component;


 -------------------------------------------------------------------
 --
 --  ILA Pro core component declaration
 --
 -------------------------------------------------------------------
 component ila
   port
   (
     control     : in    std_logic_vector(35 downto 0);
     clk         : in    std_logic;
--     data        : in    std_logic_vector(255 downto 0);
     trig0       : in    std_logic_vector(255 downto 0)
   );
 end component;

 attribute syn_black_box : boolean;
 attribute syn_black_box of icon : component is TRUE;
 attribute syn_black_box of ila : component is TRUE;
 attribute syn_noprune : boolean;
 attribute syn_noprune of icon : component is TRUE;
 attribute syn_noprune of ila : component is TRUE;

 -------------------------------------------------------------------
 --
 --  ILA Pro core signal declarations
 --
 -------------------------------------------------------------------
 signal control    : std_logic_vector(35 downto 0);
 signal trig0      : std_logic_vector(255  downto 0);

 ------------End CS Optional----------------------------------------



--Remove for TB
Signal    CEN_TR0, CEN_TR1, CEN_TR2                                 : std_logic;
--Signal    CENV_TR0, CENV_TR1, CENV_TR2                              : std_logic;
Signal    ERR_B_TR0, ERR_B_TR1, ERR_B_TR2                           : std_logic_vector(0 to 63);
Signal    WEN_B_TR0, WEN_B_TR1, WEN_B_TR2                           : std_logic_vector(63 downto 0);
Signal    Addr_B_TR0, Addr_B_TR1, Addr_B_TR2                        : std_logic_vector(addr_width-1 downto 0);
Signal    Addr_A_TR0, Addr_A_TR1, Addr_A_TR2                        : std_logic_vector(addr_width-1 downto 0);
Signal    Din_B_TR0, Din_B_TR1, Din_B_TR2                           : std_logic_vector(0 to 63);
Signal    Dout_B_TR0, Dout_B_TR1, Dout_B_TR2                        : std_logic_vector(0 to 63);

--These Stay For TB
Signal    CLKB_TR0, CLKB_TR1, CLKB_TR2                                : std_logic;
signal    CEN_bscntr_tmr_TR0, CEN_bscntr_tmr_TR1, CEN_bscntr_tmr_TR2  : std_logic;
--Signal    CENVTMR_TR0, CENVTMR_TR1, CENVTMR_TR2                       : std_logic;

Signal    BRAM_Din_A_TR2_s 	                                        :  std_logic_vector(0 to 63);
Signal    BRAM_Din_A_TR1_s 	                                        :  std_logic_vector(0 to 63);
Signal    BRAM_Din_A_TR0_s 	                                        :  std_logic_vector(0 to 63);

Signal    ENB_TR0_s, ENB_TR1_s, ENB_TR2_s                           : std_logic;

-----------------------------------------------------------------------------------
----These were "internal Signals coming in" now they do not exist. Force with -----
----Signals For now. Need to check these...----------------------------------------
----This is because the TMR Tool Does not see these as registers-------------------
----Because this section only sees BRAMS--------------------------------------------
Signal    internal_gnd_TR0_s : std_logic;
Signal    internal_gnd_TR1_s : std_logic;
Signal    internal_gnd_TR2_s : std_logic;
Signal    internal_vcc_TR0_s : std_logic;
Signal    internal_vcc_TR1_s : std_logic;
Signal    internal_vcc_TR2_s : std_logic;

-----------------------------------------------------------------------------------


attribute keep: string;

--attribute keep of CEN_bscntr_tmr_TR0: signal is "true";
--attribute keep of CEN_bscntr_tmr_TR1: signal is "true";
--attribute keep of CEN_bscntr_tmr_TR2: signal is "true";

--attribute keep of CENVTMR_TR0: signal is "true";
--attribute keep of CENVTMR_TR1: signal is "true";
--attribute keep of CENVTMR_TR2: signal is "true";

--attribute keep of Din_B_TR0: signal is "true";
--attribute keep of Din_B_TR1: signal is "true";
--attribute keep of Din_B_TR2: signal is "true";

-- attribute keep of Din_B_TR0_sig: signal is "true";
-- attribute keep of Din_B_TR1_sig: signal is "true";
-- attribute keep of Din_B_TR2_sig: signal is "true";

--attribute keep of Dout_B_TR0: signal is "true";
--attribute keep of Dout_B_TR1: signal is "true";
--attribute keep of Dout_B_TR2: signal is "true";

-- attribute keep of WEN_B_TR0: signal is "true";
-- attribute keep of WEN_B_TR1: signal is "true";
-- attribute keep of WEN_B_TR2: signal is "true";

--attribute keep of CEN_TR0: signal is "true";
--attribute keep of CEN_TR1: signal is "true";
--attribute keep of CEN_TR2: signal is "true";

--attribute keep of CENV_TR0: signal is "true";
--attribute keep of CENV_TR1: signal is "true";
--attribute keep of CENV_TR2: signal is "true";

--attribute keep of CLK_TR0: signal is "true";
--attribute keep of CLK_TR1: signal is "true";
--attribute keep of CLK_TR2: signal is "true";

-- attribute keep of ERR_B_TR0_sig: signal is "true";
-- attribute keep of ERR_B_TR1_sig: signal is "true";
-- attribute keep of ERR_B_TR2_sig: signal is "true";


begin

------------------------------------------------------
-- CLock were inverted here (now in Module)
------------------------------------------------------
CLKB_TR0  <= (BRAM_Clk_A_TR0); --Clock inverted in Module
CLKB_TR1  <= (BRAM_Clk_A_TR1); --Clock inverted in Module
CLKB_TR2  <= (BRAM_Clk_A_TR2); --Clock inverted in Module
------------------------------------------------------
--This Drops the addresses that are not needed. From
-- plb_bram_if_cntlr_1_bram_wrapper.vhd
Addr_A_TR0(addr_width-1 downto 0) <= BRAM_Addr_A_TR0(28-addr_width+1 to 28);--; 
Addr_A_TR1(addr_width-1 downto 0) <= BRAM_Addr_A_TR1(28-addr_width+1 to 28);--; 
Addr_A_TR2(addr_width-1 downto 0) <= BRAM_Addr_A_TR2(28-addr_width+1 to 28);--; 
------------------------------------------------------

-----------------------------------------------------------------------------------
----These were "internal Signals coming in" now they do not exist. Force with -----
----Signals For now. Need to check these...----------------------------------------
----This is because the TMR Tool Does not see these as registers-------------------
----Because this sction only sees BRAMS--------------------------------------------
--  internal_gnd_TR0_s <= '0';
--  internal_gnd_TR1_s <= '0';
--  internal_gnd_TR2_s <= '0';
--  internal_vcc_TR0_s <= '1';
--  internal_vcc_TR1_s <= '1';
--  internal_vcc_TR2_s <= '1';

--    internal_gnd <= '0';
--    internal_vcc_TR1 <= '0';
--    internal_gnd_TR2 <= '0';
--    internal_vcc_TR0 <= '1';
--    internal_vcc_TR1 <= '1';
--    internal_vcc_TR2 <= '1';


  internal_gnd_TR0_s <= BRAM_Rst_B_TR0; 
  internal_gnd_TR1_s <= BRAM_Rst_B_TR1;
  internal_gnd_TR2_s <= BRAM_Rst_B_TR2;
  internal_vcc_TR0_s <= not BRAM_Rst_A_TR0;
  internal_vcc_TR1_s <= not BRAM_Rst_A_TR1;
  internal_vcc_TR2_s <= not BRAM_Rst_A_TR2;
------------------------------------------------------------------------------------

CEN_bscntr_tmr_inst_TR0: CEN_bscntr_tmr 
port map (
     BOGUS_VCC => internal_vcc_TR0_s,
     CEN_TR0 => CEN_TR0, 
     CEN_TR1 => CEN_TR1, 
     CEN_TR2 => CEN_TR2, 
     CEN_bscntr_tmr => CEN_bscntr_tmr_TR0);

CEN_bscntr_tmr_inst_TR1: CEN_bscntr_tmr 
port map (
     BOGUS_VCC => internal_vcc_TR1_s,
     CEN_TR0 => CEN_TR0, 
     CEN_TR1 => CEN_TR1, 
     CEN_TR2 => CEN_TR2, 
     CEN_bscntr_tmr => CEN_bscntr_tmr_TR1);

CEN_bscntr_tmr_inst_TR2: CEN_bscntr_tmr 
port map (
     BOGUS_VCC => internal_vcc_TR2_s,
     CEN_TR0 => CEN_TR0, 
     CEN_TR1 => CEN_TR1, 
     CEN_TR2 => CEN_TR2, 
     CEN_bscntr_tmr => CEN_bscntr_tmr_TR2);

--CENVTMR_inst_TR0: CENVTMR 
--port map (
--     BOGUS_GND => internal_gnd_TR0_s,
--     CENV_TR0 => CENV_TR0, 
--     CENV_TR1 => CENV_TR1, 
--     CENV_TR2 => CENV_TR2, 
--     CENVTMR  => CENVTMR_TR0);

--CENVTMR_inst_TR1: CENVTMR 
--port map (
--     BOGUS_GND => internal_gnd_TR1_s,
--     CENV_TR0 => CENV_TR0, 
--     CENV_TR1 => CENV_TR1, 
--     CENV_TR2 => CENV_TR2, 
--     CENVTMR  => CENVTMR_TR1);

--CENVTMR_inst_TR2: CENVTMR 
--port map (
--     BOGUS_GND => internal_gnd_TR2_s,
--     CENV_TR0 => CENV_TR0, 
--     CENV_TR1 => CENV_TR1, 
--     CENV_TR2 => CENV_TR2, 
--     CENVTMR  => CENVTMR_TR2);


Comparator: Comparator_TMR
port map (
--     internal_gnd_tr0 => internal_gnd_tr0_s,
--     internal_gnd_tr1 => internal_gnd_tr1_s,
--     internal_gnd_tr2 => internal_gnd_tr2_s,
--     internal_vcc_tr0 => internal_vcc_tr0_s,
--     internal_vcc_tr1 => internal_vcc_tr1_s,
--     internal_vcc_tr2 => internal_vcc_tr2_s,

--     CLK_TR0 => BRAM_Clk_A_TR0, 
--     CLK_TR1 => BRAM_Clk_A_TR1, 
--     CLK_TR2 => BRAM_Clk_A_TR2,     
 
--     SET_TR0 => internal_gnd_tr0_s,
--     SET_TR1 => internal_gnd_tr1_s,
--     SET_TR2 => internal_gnd_tr2_s, 

--     RST_TR0 => internal_gnd_tr0_s, 
--     RST_TR1 => internal_gnd_tr1_s, 
--     RST_TR2 => internal_gnd_tr2_s,

       ENB_TR0 => ENB_TR0_s,
       ENB_TR1 => ENB_TR1_s,
       ENB_TR2 => ENB_TR2_s,

     CEN_TR0 => CEN_TR0,
     CEN_TR1 => CEN_TR1,
     CEN_TR2 => CEN_TR2,

--     CE_TR0      => internal_vcc_tr0_s,
--     CE_TR1      => internal_vcc_tr1_s,
--     CE_TR2      => internal_vcc_tr2_s,

--     CENV_TR0 => CENV_TR0,
--     CENV_TR1 => CENV_TR1,
--     CENV_TR2 => CENV_TR2,

     ADDR_A_TR0 => ADDR_A_TR0,
     ADDR_A_TR1 => ADDR_A_TR1,
     ADDR_A_TR2 => ADDR_A_TR2,

	 WEN_A_TR0  => BRAM_WEN_A_TR0, 
	 WEN_A_TR1  => BRAM_WEN_A_TR1, 
	 WEN_A_TR2  => BRAM_WEN_A_TR2, 

     ADDR_B_TR0 => Addr_B_TR0,
     ADDR_B_TR1 => Addr_B_TR1,
     ADDR_B_TR2 => Addr_B_TR2,

     WEN_B_TR0 => WEN_B_TR0,
     WEN_B_TR1 => WEN_B_TR1,
     WEN_B_TR2 => WEN_B_TR2,

     ERR_B_TR0 => ERR_B_TR0,
     ERR_B_TR1 => ERR_B_TR1,
     ERR_B_TR2 => ERR_B_TR2
          );


scrub_counter: BSCNTR_XTMR
port map (
--     internal_gnd_tr0 => internal_gnd_tr0_s,
--     internal_gnd_tr1 => internal_gnd_tr1_s,
--     internal_gnd_tr2 => internal_gnd_tr2_s,
--     internal_vcc_tr0 => internal_vcc_tr0_s,
--     internal_vcc_tr1 => internal_vcc_tr1_s,
--     internal_vcc_tr2 => internal_vcc_tr2_s,
     CLK_TR0 => BRAM_Clk_A_TR0, 
     CLK_TR1 => BRAM_Clk_A_TR1, 
     CLK_TR2 => BRAM_Clk_A_TR2, 
     SET_TR0 => internal_gnd_tr0_s,
     SET_TR1 => internal_gnd_tr1_s,
     SET_TR2 => internal_gnd_tr2_s,
     RST_TR0 => BRAM_Rst_A_TR0,
     RST_TR1 => BRAM_Rst_A_TR1,
     RST_TR2 => BRAM_Rst_A_TR2,
     CE_TR0 => CEN_bscntr_tmr_TR0,      
     CE_TR1 => CEN_bscntr_tmr_TR1,      
     CE_TR2 => CEN_bscntr_tmr_TR2,    
     ADDRESS_TR0 => Addr_B_TR0,
     ADDRESS_TR1 => Addr_B_TR1,
     ADDRESS_TR2 => Addr_B_TR2);

--This Generates the Triple Block RAMs and is inferred
-- use bram_scrubber_bram
-----------------------------------------------------------

BRAM_Din_A_TR2 <= BRAM_Din_A_TR2_s;
BRAM_Din_A_TR1 <= BRAM_Din_A_TR1_s;	                                        
BRAM_Din_A_TR0 <= BRAM_Din_A_TR0_s;	                                        

bram_scrubber_bram_TR2: bram_scrubber_bram_bitw
      port map(
          CLKA  => BRAM_Clk_A_TR2,
          ENA   => BRAM_EN_A_TR2, --Add For Full HL Extraction
          SSRA   => internal_gnd_TR2_s,
          WEA   => BRAM_WEN_A_TR2,
          DIA   => BRAM_Dout_A_TR2,
          ADDRA => BRAM_Addr_A_TR2(28-addr_width+1 to 28),
          DOA   => BRAM_Din_A_TR2_s,
          CLKB  => BRAM_Clk_A_TR2,
          ENB   => ENB_TR2_s,
          SSRB  =>  internal_gnd_TR2_s,       --Add For Full HL Extraction
          WEB   => WEN_B_TR2,
          DIB   => Din_B_TR2,
          ADDRB => Addr_B_TR2,
          DOB   => Dout_B_TR2
            );

bram_scrubber_bram_TR1: bram_scrubber_bram_bitw
     port map(
          CLKA  => BRAM_Clk_A_TR1,
          ENA   => BRAM_EN_A_TR1,
          SSRA   => internal_gnd_TR1_s,            --Add For Full HL Extraction
          WEA   => BRAM_WEN_A_TR1,
          DIA   => BRAM_Dout_A_TR1,
          ADDRA => BRAM_Addr_A_TR1(28-addr_width+1 to 28),
          DOA   => BRAM_Din_A_TR1_s,
          CLKB  => BRAM_Clk_A_TR1,
          ENB   => ENB_TR1_s,
          SSRB  => internal_gnd_TR1_s,              --Add For Full HL Extraction
          WEB   => WEN_B_TR1,
          DIB   => Din_B_TR1,
          ADDRB => Addr_B_TR1,
          DOB   => Dout_B_TR1
            );

bram_scrubber_bram_TR0: bram_scrubber_bram_bitw
     port map(
          CLKA  => BRAM_Clk_A_TR0,
          ENA   => BRAM_EN_A_TR0,
          SSRA  => internal_gnd_TR0_s,               --Add For Full HL Extraction
          WEA   => BRAM_WEN_A_TR0,
          DIA   => BRAM_Dout_A_TR0,
          ADDRA => BRAM_Addr_A_TR0(28-addr_width+1 to 28),
          DOA   => BRAM_Din_A_TR0_s,
          CLKB  => BRAM_Clk_A_TR0,
          ENB   => ENB_TR0_s,
          SSRB  => internal_gnd_TR0_s,                --Add For Full HL Extraction
          WEB   => WEN_B_TR0,
          DIB   => Din_B_TR0,
          ADDRB => Addr_B_TR0,
          DOB   => Dout_B_TR0
            );
-----------------------------------------------------------
------------------------------------
bram_scrubber_data_voter_tr0: bram_scrubber_data_voter
     port map (
          TR0                 =>  Dout_B_TR0,  --Prime
          TR1                 =>  Dout_B_TR1,  --ReadA
          TR2                 =>  Dout_B_TR2,  --ReadB
--          PRE                 =>  internal_gnd_TR0_s, 
--          CE                  =>  internal_vcc_TR0_s, 
--          C                   =>  CLKB_TR0,
--		CLR                 =>  CENVTMR_TR0,
--          CLR                 =>  internal_gnd_TR0_s,
          Din_B               =>  Din_B_TR0,
          ERROR_B             =>  ERR_B_TR0
        );


bram_scrubber_data_voter_tr1: bram_scrubber_data_voter
     port map (
          TR0                 =>  Dout_B_TR1, --Prime
          TR1                 =>  Dout_B_TR2, --ReadA
          TR2                 =>  Dout_B_TR0, --ReadB
--          PRE                 =>  internal_gnd_TR1_s, 
--          CE                  =>  internal_vcc_TR1_s, 
--          C                   =>  CLKB_TR1,
--		CLR                 =>  CENVTMR_TR0,
--          CLR                 =>  internal_gnd_TR1_s,
          Din_B               =>  Din_B_TR1,
          ERROR_B             =>  ERR_B_TR1
        );


bram_scrubber_data_voter_tr2: bram_scrubber_data_voter
     port map (
          TR0                 =>  Dout_B_TR2, --Prime
          TR1                 =>  Dout_B_TR0, --ReadA
          TR2                 =>  Dout_B_TR1, --ReadB
--          PRE                 =>  internal_gnd_TR2_s, 
--          CE                  =>  internal_vcc_TR2_s, 
--          C                   =>  CLKB_TR2,
--		CLR                 =>  CENVTMR_TR0,
--          CLR                 =>  internal_gnd_TR2_s,
          Din_B               =>  Din_B_TR2,
          ERROR_B             =>  ERR_B_TR2
        );

--Optional Register output for Write Enable B

--WEN_B_out_TR0_s <= WEN_B_ind0_s;
--WEN_B_out_TR1_s <= WEN_B_ind1_s;
--WEN_B_out_TR2_s <= WEN_B_ind2_s;

------------------------------------
------Optional Testing Outputs------
---Pull Out for Testing-------------
--Check address conflict--

--WEN_B_out_TR0 <= WEN_B_out_TR0_s;
--WEN_B_out_TR1 <= WEN_B_out_TR1_s;
--WEN_B_out_TR2 <= WEN_B_out_TR2_s;

--WEN_B_out_TR0 <= WEN_B_TR0'RANGE=>'1'; 
--WEN_B_out_TR1 <= WEN_B_TR1;
--WEN_B_out_TR2 <= WEN_B_TR2;

--Addr_B_out_TR0_s <= Addr_B_TR0(13 downto 10) & Addr_B_TR0(3 downto 0); --First and Last Nibble
--Addr_B_out_TR1_s <= Addr_B_TR1(13 downto 10) & Addr_B_TR1(3 downto 0); --First and Last Nibble
--Addr_B_out_TR2_s <= Addr_B_TR2(13 downto 10) & Addr_B_TR2(3 downto 0); --First and Last Nibble

--Addr_B_out_TR0_s <= Din_B_TR0(13 downto 10) & Din_B_TR0(3 downto 0); --First and Last Nibble
--Addr_B_out_TR1_s <= Din_B_TR1(13 downto 10) & Din_B_TR1(3 downto 0); --First and Last Nibble
--Addr_B_out_TR2_s <= Din_B_TR2(13 downto 10) & Din_B_TR2(3 downto 0); --First and Last Nibble

--Addr_B_out_TR0 <= Addr_B_out_TR0_s;
--Addr_B_out_TR1 <= Addr_B_out_TR1_s;
--Addr_B_out_TR2 <= Addr_B_out_TR2_s;

--------------------Chipscope Optional-----------------------------
---------------------------------------------------------------------
-- I_icon : Icon
--   Port Map
--   (
--     Control0   => Control
--   );
-----------------------------------------------------------------------
-- I_ila : Ila
--   Port Map
--   (
--     Control   => Control,
--     Clk       => Bram_clk_a_tr2,
----     Data      => Trig0,
--     Trig0     => Trig0
--   );
----
-------------------------  Signals To Observe --------------------------
----
--Trig0(11 Downto 0)	     <= Bram_addr_a_tr2(28-addr_width+1 to 28);
--Trig0(44 Downto 13)      <= Bram_dout_a_tr2(0 To 31);
--Trig0(45)                <= Bram_en_a_tr2;
--Trig0(46)			     <= Bram_wen_a_tr2(0);
--Trig0(78 Downto 47)      <= Bram_din_a_tr2_s(0 To 31);
--Trig0(79)		          <= Internal_vcc_tr2_s;
--Trig0(80)		          <= Wen_b_tr2(0);
--Trig0(112 Downto 81)     <= Din_b_tr2(0 To 31);
--Trig0(124 Downto 113)	<= Addr_b_tr2;
--Trig0(157 Downto 126)    <= Dout_b_tr2(0 To 31);
--Trig0(158)               <= Internal_gnd_tr0_s;
--Trig0(159)               <= Internal_gnd_tr1_s;
--Trig0(160)               <= Internal_gnd_tr2_s;
--Trig0(161)               <= Internal_vcc_tr0_s;
--Trig0(162)               <= Internal_vcc_tr1_s;
--Trig0(163)               <= Internal_vcc_tr2_s;
--Trig0(164)               <= CEN_bscntr_tmr_TR0;
--Trig0(165)               <= CEN_bscntr_tmr_TR1;
--Trig0(166)               <= CEN_bscntr_tmr_TR2;
--Trig0(167)               <= CEN_TR0; 
--Trig0(168)               <= CEN_TR1; 
--Trig0(169)               <= CEN_TR2; 
--Trig0(170)               <= ENB_TR0_s;
--Trig0(171)               <= ENB_TR1_s;
--Trig0(172)               <= ENB_TR2_s;
--Trig0(236 downto 173)    <= Wen_b_tr2;
--Trig0(237)               <= BRAM_Rst_A_TR0;
--Trig0(238)               <= BRAM_Rst_A_TR1;
--Trig0(239)               <= BRAM_Rst_A_TR2;



-----------------End Chipscope Optional----------------------------


------------------------------------

end architecture RTL;
